@@ -1,14 +1,10 @@
 /**
- * $Id: main.cpp,v 1.4 2003/05/01 19:52:02 sirdude Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,13 +22,9 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "LOD_decimation.h"
 #include "ply/ply.h"
 #include "MEM_SmartPtr.h"
@@ -254,9 +246,9 @@ init(MT_Vector3 min,MT_Vector3 max)
   /* Setup the view of the cube. */
   glMatrixMode(GL_PROJECTION);
 
-	// centre of the box + 3* depth of box
+	// center of the box + 3* depth of box
 
-  MT_Vector3 centre = (min + max) * 0.5;
+  MT_Vector3 center = (min + max) * 0.5;
   MT_Vector3 diag = max - min;
 
 	float depth = diag.length();
@@ -272,8 +264,8 @@ init(MT_Vector3 min,MT_Vector3 max)
 
 
   gluLookAt(
-	centre.x(), centre.y(), centre.z() + distance*depth,  /* eye is at (0,0,5) */
-    centre.x(), centre.y(), centre.z(),      /* center is at (0,0,0) */
+	center.x(), center.y(), center.z() + distance*depth,  /* eye is at (0,0,5) */
+    center.x(), center.y(), center.z(),      /* center is at (0,0,0) */
     0.0, 1.0, 0.);      /* up is in positive Y direction */
 
   glPushMatrix();	
