@@ -1,14 +1,10 @@
-/**
- * $Id: GHOST_C-api.h,v 1.5 2002/12/28 22:26:45 maarten Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,12 +22,12 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
-/**
- * @file	GHOST_C-api.h
- * GHOST C-API function and type declarations.
- * The C-API wraps the C++ objects with the 
+/** \ingroup GHOST
+ *
+ * \file	GHOST_C-api.h
+ * \brief GHOST C-API function and type declarations.
  */
 
 #ifndef	GHOST_C_API_H
@@ -47,9 +43,8 @@ extern "C" {
  * Creates a &quot;handle&quot; for a C++ GHOST object.
  * A handle is just an opaque pointer to an empty struct.
  * In the API the pointer is casted to the actual C++ class.
- * @param	name	Name of the handle to create.
+ * \param	name	Name of the handle to create.
  */
-#define GHOST_DECLARE_HANDLE(name) typedef struct name##__ { int unused; } *name
 
 GHOST_DECLARE_HANDLE(GHOST_SystemHandle);
 GHOST_DECLARE_HANDLE(GHOST_TimerTaskHandle);
@@ -165,17 +160,20 @@ extern void GHOST_GetMainDisplayDimensions(GHOST_SystemHandle systemhandle,
  * @param	height	The height the window.
  * @param	state	The state of the window when opened.
  * @param	type	The type of drawing context installed in this window.
+ * @param stereoVisual		Stereo visual for quad buffered stereo.
+ * @param numOfAASamples	Number of samples used for AA (zero if no AA)
  * @return	A handle to the new window ( == NULL if creation failed).
  */
 extern GHOST_WindowHandle GHOST_CreateWindow(GHOST_SystemHandle systemhandle,
-	char* title,
+	const char* title,
 	GHOST_TInt32 left,
 	GHOST_TInt32 top,
 	GHOST_TUns32 width,
 	GHOST_TUns32 height,
 	GHOST_TWindowState state,
 	GHOST_TDrawingContextType type,
-	const int stereoVisual);
+	const int stereoVisual,
+	const GHOST_TUns16 numOfAASamples);
 
 /**
  * Returns the window user data.
@@ -263,8 +261,32 @@ extern int GHOST_DispatchEvents(GHOST_SystemHandle systemhandle);
  */
 extern GHOST_TSuccess GHOST_AddEventConsumer(GHOST_SystemHandle systemhandle,
 											  GHOST_EventConsumerHandle consumerhandle);
-	
 
+/**
+ * Remove the given event consumer to our list.
+ * @param systemhandle The handle to the system
+ * @param consumerhandle The event consumer to remove.
+ * @return Indication of success.
+ */
+extern GHOST_TSuccess GHOST_RemoveEventConsumer(GHOST_SystemHandle systemhandle,
+											  GHOST_EventConsumerHandle consumerhandle);
+
+/***************************************************************************************
+ ** Progress bar functionality
+ ***************************************************************************************/
+
+/**
+ * Sets the progress bar value displayed in the window/application icon
+ * @param windowhandle The handle to the window
+ * @param progress The progress % (0.0 to 1.0)
+ */
+extern GHOST_TSuccess GHOST_SetProgressBar(GHOST_WindowHandle windowhandle, float progress);
+
+/**
+ * Hides the progress bar in the icon
+ * @param windowhandle The handle to the window
+ */
+extern GHOST_TSuccess GHOST_EndProgressBar(GHOST_WindowHandle windowhandle);
 
 /***************************************************************************************
  ** Cursor management functionality
@@ -300,6 +322,23 @@ extern GHOST_TSuccess GHOST_SetCustomCursorShape(GHOST_WindowHandle windowhandle
 												 GHOST_TUns8 mask[16][2], 
 												 int hotX, 
 												 int hotY);
+/**
+ * Set the shape of the cursor to a custom cursor of specified size.
+ * @param windowhandle The handle to the window
+ * @param	bitmap	The bitmap data for the cursor.
+ * @param	mask	The mask data for the cursor.
+ * @parm    sizex, sizey  The size of the cursor
+ * @param	hotX	The X coordinate of the cursor hotspot.
+ * @param	hotY	The Y coordinate of the cursor hotspot.
+ * @param   fg_color, bg_color  Colors of the cursor
+ * @return	Indication of success.
+  */
+extern GHOST_TSuccess GHOST_SetCustomCursorShapeEx(GHOST_WindowHandle windowhandle,
+												 GHOST_TUns8 *bitmap, 
+												 GHOST_TUns8 *mask, 
+												 int sizex, int sizey, 
+												 int hotX,  int hotY,
+												 int fg_color, int bg_color );
 
 /**
  * Returns the visibility state of the cursor.
@@ -340,6 +379,19 @@ extern GHOST_TSuccess GHOST_SetCursorPosition(GHOST_SystemHandle systemhandle,
 											  GHOST_TInt32 x,
 											  GHOST_TInt32 y);
 
+/**
+ * Grabs the cursor for a modal operation, to keep receiving
+ * events when the mouse is outside the window. X11 only, others
+ * do this automatically.
+ * @param windowhandle The handle to the window
+ * @param	mode The new grab state of the cursor.
+ * @param	bounds The grab ragion (optional) - left,top,right,bottom
+ * @return	Indication of success.
+ */
+extern GHOST_TSuccess GHOST_SetCursorGrab(GHOST_WindowHandle windowhandle,
+												GHOST_TGrabCursorMode mode,
+												int* bounds);
+
 /***************************************************************************************
  ** Access to mouse button and keyboard states.
  ***************************************************************************************/
@@ -366,6 +418,17 @@ extern GHOST_TSuccess GHOST_GetButtonState(GHOST_SystemHandle systemhandle,
 										   GHOST_TButtonMask mask,
 										   int* isDown);
 
+
+/***************************************************************************************
+ ** Drag'n'drop operations
+ ***************************************************************************************/
+
+/**
+ * Tells if the ongoing drag'n'drop object can be accepted upon mouse drop
+ */
+extern void GHOST_setAcceptDragOperation(GHOST_WindowHandle windowhandle, GHOST_TInt8 canAccept);
+	
+	
 /**
  * Returns the event type.
  * @param eventhandle The handle to the event
@@ -454,7 +517,7 @@ extern GHOST_TSuccess GHOST_SetDrawingContextType(GHOST_WindowHandle windowhandl
  * @param title	The title to display in the title bar.
  */
 extern void GHOST_SetTitle(GHOST_WindowHandle windowhandle,
-						   char* title);
+						   const char* title);
 
 /**
  * Returns the title displayed in the title bar. The title
@@ -560,6 +623,16 @@ extern GHOST_TWindowState GHOST_GetWindowState(GHOST_WindowHandle windowhandle);
 extern GHOST_TSuccess GHOST_SetWindowState(GHOST_WindowHandle windowhandle,
 										   GHOST_TWindowState state);
 
+	
+/**
+ * Sets the window "modified" status, indicating unsaved changes
+ * @param windowhandle The handle to the window
+ * @param isUnsavedChanges Unsaved changes or not
+ * @return Indication of success.
+ */
+extern GHOST_TSuccess GHOST_SetWindowModifiedState(GHOST_WindowHandle windowhandle,
+												   GHOST_TUns8 isUnsavedChanges);
+	
 /**
  * Sets the order of the window (bottom, top).
  * @param windowhandle The handle to the window
@@ -590,6 +663,13 @@ extern GHOST_TSuccess GHOST_ActivateWindowDrawingContext(GHOST_WindowHandle wind
  */
 extern GHOST_TSuccess GHOST_InvalidateWindow(GHOST_WindowHandle windowhandle);
 
+/**
+ * Returns the status of the tablet
+ * @param windowhandle The handle to the window
+ * @return Status of tablet
+ */
+extern const GHOST_TabletData *GHOST_GetTabletData(GHOST_WindowHandle windowhandle);
+
 /**
  * Access to rectangle width.
  * @param rectanglehandle The handle to the rectangle
@@ -735,6 +815,32 @@ extern void GHOST_SetRectangleCenter(GHOST_RectangleHandle rectanglehandle,
 extern GHOST_TSuccess GHOST_ClipRectangle(GHOST_RectangleHandle rectanglehandle,
 										  GHOST_RectangleHandle anotherrectanglehandle);
 
+/**
+ * Return the data from the clipboad
+ * @param	return the selection instead, X11 only feature
+ * @return	clipboard data
+ */
+extern GHOST_TUns8* GHOST_getClipboard(int selection);
+
+/**
+ * Put data to the Clipboard
+ * @param	set the selection instead, X11 only feature
+ */
+extern void GHOST_putClipboard(GHOST_TInt8 *buffer, int selection);
+
+
+
+/**
+ * Toggles console
+ * @action	0 - Hides
+ *			1 - Shows
+ *			2 - Toggles
+ *			3 - Hides if it runs not from  command line
+ *			* - Does nothing
+ * @return current status (1 -visible, 0 - hidden)
+ */
+extern int GHOST_toggleConsole(int action);
+
 #ifdef __cplusplus
 }
 #endif
