@@ -1,14 +1,10 @@
-/**
- * $Id: GHOST_IWindow.h,v 1.6 2002/12/28 22:26:45 maarten Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,10 +22,11 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
-/**
- * @file	GHOST_IWindow.h
+
+/** \file ghost/GHOST_IWindow.h
+ *  \ingroup GHOST
  * Declaration of GHOST_IWindow interface class.
  */
 
@@ -75,11 +72,17 @@ public:
 	 */
 	virtual	bool getValid() const = 0;
 
+	/**
+	 * Returns the associated OS object/handle
+	 * @return The associated OS object/handle
+	 */
+	virtual void* getOSWindow() const = 0;
+
 	/**
 	 * Returns the type of drawing context used in this window.
 	 * @return The current type of drawing context.
 	 */
-	inline virtual GHOST_TDrawingContextType getDrawingContextType() = 0;
+	virtual GHOST_TDrawingContextType getDrawingContextType() = 0;
 
 	/**
 	 * Tries to install a rendering context in this window.
@@ -151,6 +154,17 @@ public:
 	 */
 	virtual	void clientToScreen(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const = 0;
 
+	/**
+	 * Tells if the ongoing drag'n'drop object can be accepted upon mouse drop
+	 */
+	virtual void setAcceptDragOperation(bool canAccept) = 0;
+	
+	/**
+	 * Returns acceptance of the dropped object
+	 * Usually called by the "object dropped" event handling function
+	 */
+	virtual bool canAcceptDragOperation() const = 0;
+	
 	/**
 	 * Returns the state of the window (normal, minimized, maximized).
 	 * @return The state of the window.
@@ -164,6 +178,19 @@ public:
 	 */
 	virtual GHOST_TSuccess setState(GHOST_TWindowState state) = 0;
 
+	/**
+	 * Sets the window "modified" status, indicating unsaved changes
+	 * @param isUnsavedChanges Unsaved changes or not
+	 * @return Indication of success.
+	 */
+	virtual GHOST_TSuccess setModifiedState(bool isUnsavedChanges) = 0;
+	
+	/**
+	 * Gets the window "modified" status, indicating unsaved changes
+	 * @return True if there are unsaved changes
+	 */
+	virtual bool getModifiedState() = 0;
+	
 	/**
 	 * Sets the order of the window (bottom, top).
 	 * @param order The order of the window.
@@ -193,7 +220,7 @@ public:
 	 * Returns the window user data.
 	 * @return The window user data.
 	 */
-	inline virtual GHOST_TUserDataPtr getUserData() const = 0;
+	virtual GHOST_TUserDataPtr getUserData() const = 0;
 	
 	/**
 	 * Changes the window user data.
@@ -201,6 +228,27 @@ public:
 	 */
 	virtual void setUserData(const GHOST_TUserDataPtr userData) = 0;
 	
+	/**
+	 * Returns the tablet data (pressure etc).
+	 * @return The tablet data (pressure etc).
+	 */
+	virtual const GHOST_TabletData* GetTabletData() = 0;
+	
+	/***************************************************************************************
+	 ** Progress bar functionality
+	 ***************************************************************************************/
+	
+	/**
+     * Sets the progress bar value displayed in the window/application icon
+	 * @param progress The progress %
+	 */
+	virtual GHOST_TSuccess setProgressBar(float progress) = 0;
+	
+	/**
+	 * Hides the progress bar in the icon
+	 */
+	virtual GHOST_TSuccess endProgressBar() = 0;
+	
 	/***************************************************************************************
 	 ** Cursor management functionality
 	 ***************************************************************************************/
@@ -230,6 +278,12 @@ public:
 												GHOST_TUns8 mask[16][2], 
 												int hotX, 
 												int hotY) = 0;
+												
+	virtual GHOST_TSuccess setCustomCursorShape(GHOST_TUns8 *bitmap, 
+												GHOST_TUns8 *mask, 
+												int sizex, int sizey, 
+												int hotX, int hotY, 
+												int fg_color, int bg_color) = 0;
 
 	/**
 	 * Returns the visibility state of the cursor.
@@ -243,6 +297,19 @@ public:
 	 * @return	Indication of success.
 	 */
 	virtual GHOST_TSuccess setCursorVisibility(bool visible) = 0;
+
+	/**
+	 * Grabs the cursor for a modal operation.
+	 * @param	grab The new grab state of the cursor.
+	 * @return	Indication of success.
+	 */
+	virtual GHOST_TSuccess setCursorGrab(GHOST_TGrabCursorMode mode, GHOST_Rect *bounds) { return GHOST_kSuccess; };
+
+#ifdef WITH_CXX_GUARDEDALLOC
+public:
+	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GHOST:GHOST_IWindow"); }
+	void operator delete( void *mem ) { MEM_freeN(mem); }
+#endif
 };
 
 #endif // _GHOST_IWINDOW_H_
