@@ -1,14 +1,10 @@
-/**
- * $Id: GHOST_SystemCarbon.cpp,v 1.17 2004/08/01 22:28:56 lukep Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,21 +22,23 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
+/** \file ghost/intern/GHOST_SystemCarbon.cpp
+ *  \ingroup GHOST
+ */
+
+
 /**
 
- * $Id: GHOST_SystemCarbon.cpp,v 1.17 2004/08/01 22:28:56 lukep Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 7, 2001
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include <Carbon/Carbon.h>
+#include <ApplicationServices/ApplicationServices.h>
 #include "GHOST_SystemCarbon.h"
 
 #include "GHOST_DisplayManagerCarbon.h"
@@ -48,22 +46,36 @@
 #include "GHOST_EventButton.h"
 #include "GHOST_EventCursor.h"
 #include "GHOST_EventWheel.h"
+#ifdef WITH_INPUT_NDOF
+#include "GHOST_EventNDOF.h"
+#endif
+
 #include "GHOST_TimerManager.h"
 #include "GHOST_TimerTask.h"
 #include "GHOST_WindowManager.h"
 #include "GHOST_WindowCarbon.h"
+#include "GHOST_NDOFManager.h"
+#include "AssertMacros.h"
 
 #define GHOST_KEY_SWITCH(mac, ghost) { case (mac): ghostKey = (ghost); break; }
 
+/* blender class and types events */
+enum {
+  kEventClassBlender              = 'blnd'
+};
+
+enum {
+	kEventBlenderNdofAxis			= 1,
+	kEventBlenderNdofButtons		= 2
+};
+
 const EventTypeSpec	kEvents[] =
 {
 	{ kEventClassAppleEvent, kEventAppleEvent },
-	
 /*
 	{ kEventClassApplication, kEventAppActivated },
 	{ kEventClassApplication, kEventAppDeactivated },
 */	
-
 	{ kEventClassKeyboard, kEventRawKeyDown },
 	{ kEventClassKeyboard, kEventRawKeyRepeat },
 	{ kEventClassKeyboard, kEventRawKeyUp },
@@ -84,7 +96,12 @@ const EventTypeSpec	kEvents[] =
 	{ kEventClassWindow, kEventWindowActivated },
 	{ kEventClassWindow, kEventWindowDeactivated },
 	{ kEventClassWindow, kEventWindowUpdate },
-	{ kEventClassWindow, kEventWindowBoundsChanged }
+	{ kEventClassWindow, kEventWindowBoundsChanged },
+	
+	{ kEventClassBlender, kEventBlenderNdofAxis },
+	{ kEventClassBlender, kEventBlenderNdofButtons }
+	
+	
 	
 };
 
@@ -375,7 +392,7 @@ GHOST_TUns8 GHOST_SystemCarbon::getNumDisplays() const
 void GHOST_SystemCarbon::getMainDisplayDimensions(GHOST_TUns32& width, GHOST_TUns32& height) const
 {
 	BitMap screenBits;
-    Rect bnds = GetQDGlobalsScreenBits(&screenBits)->bounds;
+	Rect bnds = GetQDGlobalsScreenBits(&screenBits)->bounds;
 	width = bnds.right - bnds.left;
 	height = bnds.bottom - bnds.top;
 }
@@ -389,54 +406,81 @@ GHOST_IWindow* GHOST_SystemCarbon::createWindow(
 	GHOST_TUns32 height,
 	GHOST_TWindowState state,
 	GHOST_TDrawingContextType type,
-	bool stereoVisual
+	bool stereoVisual,
+	const GHOST_TUns16 numOfAASamples,
+	const GHOST_TEmbedderWindowID parentWindow
 )
 {
-    GHOST_IWindow* window = 0;
+	GHOST_IWindow* window = 0;
 
 	window = new GHOST_WindowCarbon (title, left, top, width, height, state, type);
 
-    if (window) {
-        if (window->getValid()) {
-            // Store the pointer to the window 
-            GHOST_ASSERT(m_windowManager, "m_windowManager not initialized");
-            m_windowManager->addWindow(window);
-            m_windowManager->setActiveWindow(window);
-            pushEvent(new GHOST_Event(getMilliSeconds(), GHOST_kEventWindowSize, window));
-        }
-        else {
+	if (window) {
+		if (window->getValid()) {
+			// Store the pointer to the window
+			GHOST_ASSERT(m_windowManager, "m_windowManager not initialized");
+			m_windowManager->addWindow(window);
+			m_windowManager->setActiveWindow(window);
+			pushEvent(new GHOST_Event(getMilliSeconds(), GHOST_kEventWindowSize, window));
+		}
+		else {
 			GHOST_PRINT("GHOST_SystemCarbon::createWindow(): window invalid\n");
-            delete window;
-            window = 0;
-        }
-    }
+			delete window;
+			window = 0;
+		}
+	}
 	else {
 		GHOST_PRINT("GHOST_SystemCarbon::createWindow(): could not create window\n");
 	}
-    return window;
+	return window;
 }
 
+GHOST_TSuccess GHOST_SystemCarbon::beginFullScreen(const GHOST_DisplaySetting& setting, GHOST_IWindow** window, const bool stereoVisual)
+{	
+	GHOST_TSuccess success = GHOST_kFailure;
 
+	// need yo make this Carbon all on 10.5 for fullscreen to work correctly
+	CGCaptureAllDisplays();
+	
+	success = GHOST_System::beginFullScreen( setting, window, stereoVisual);
+	
+	if( success != GHOST_kSuccess ) {
+			// fullscreen failed for other reasons, release
+			CGReleaseAllDisplays();	
+	}
+
+	return success;
+}
+
+GHOST_TSuccess GHOST_SystemCarbon::endFullScreen(void)
+{	
+	CGReleaseAllDisplays();
+	return GHOST_System::endFullScreen();
+}
+
+/* this is an old style low level event queue.
+  As we want to handle our own timers, this is ok.
+  the full screen hack should be removed */
 bool GHOST_SystemCarbon::processEvents(bool waitForEvent)
 {
 	bool anyProcessed = false;
 	EventRef event;
 	
+//	SetMouseCoalescingEnabled(false, NULL);
+
 	do {
 		GHOST_TimerManager* timerMgr = getTimerManager();
-
+		
 		if (waitForEvent) {
-			GHOST_TUns64 curtime = getMilliSeconds();
 			GHOST_TUns64 next = timerMgr->nextFireTime();
 			double timeOut;
 			
 			if (next == GHOST_kFireTimeNever) {
 				timeOut = kEventDurationForever;
 			} else {
-				if (next<=curtime)
+				timeOut = (double)(next - getMilliSeconds())/1000.0;
+				if (timeOut < 0.0)
 					timeOut = 0.0;
-				else
-					timeOut = (double) (next - getMilliSeconds())/1000.0;
 			}
 			
 			::ReceiveNextEvent(0, NULL, timeOut, false, &event);
@@ -455,6 +499,7 @@ bool GHOST_SystemCarbon::processEvents(bool waitForEvent)
 			}
 		}
 
+		/* end loop when no more events available */
 		while (::ReceiveNextEvent(0, NULL, 0, true, &event)==noErr) {
 			OSStatus status= ::SendEventToEventTarget(event, ::GetEventDispatcherTarget());
 			if (status==noErr) {
@@ -466,66 +511,78 @@ bool GHOST_SystemCarbon::processEvents(bool waitForEvent)
 					 * are, but we get a lot of them
 					 */
 				if (i!='cgs ') {
-					//printf("Missed - Class: '%.4s', Kind: %d\n", &i, ::GetEventKind(event));
+					if (i!='tblt') {  // tablet event. we use the one packaged in the mouse event
+						; //printf("Missed - Class: '%.4s', Kind: %d\n", &i, ::GetEventKind(event));
+					}
 				}
 			}
 			::ReleaseEvent(event);
 		}
 	} while (waitForEvent && !anyProcessed);
 	
-    return anyProcessed;
+	return anyProcessed;
 }
 	
 
 GHOST_TSuccess GHOST_SystemCarbon::getCursorPosition(GHOST_TInt32& x, GHOST_TInt32& y) const
 {
-    Point mouseLoc;
-    // Get the position of the mouse in the active port
-    ::GetGlobalMouse(&mouseLoc);
-    // Convert the coordinates to screen coordinates
-    x = (GHOST_TInt32)mouseLoc.h;
-    y = (GHOST_TInt32)mouseLoc.v;
-    return GHOST_kSuccess;
+	Point mouseLoc;
+	// Get the position of the mouse in the active port
+	::GetGlobalMouse(&mouseLoc);
+	// Convert the coordinates to screen coordinates
+	x = (GHOST_TInt32)mouseLoc.h;
+	y = (GHOST_TInt32)mouseLoc.v;
+	return GHOST_kSuccess;
 }
 
 
-GHOST_TSuccess GHOST_SystemCarbon::setCursorPosition(GHOST_TInt32 x, GHOST_TInt32 y) const
+GHOST_TSuccess GHOST_SystemCarbon::setCursorPosition(GHOST_TInt32 x, GHOST_TInt32 y)
 {
 	float xf=(float)x, yf=(float)y;
 
-	CGPostMouseEvent(CGPointMake(xf, yf), TRUE, 1, FALSE, 0);
+	CGAssociateMouseAndMouseCursorPosition(false);
+	CGSetLocalEventsSuppressionInterval(0);
+	CGWarpMouseCursorPosition(CGPointMake(xf, yf));
+	CGAssociateMouseAndMouseCursorPosition(true);
 
-    return GHOST_kSuccess;
+//this doesn't work properly, see game engine mouse-look scripts
+//	CGWarpMouseCursorPosition(CGPointMake(xf, yf));
+	// this call below sends event, but empties other events (like shift)
+	// CGPostMouseEvent(CGPointMake(xf, yf), TRUE, 1, FALSE, 0);
+
+	return GHOST_kSuccess;
 }
 
 
 GHOST_TSuccess GHOST_SystemCarbon::getModifierKeys(GHOST_ModifierKeys& keys) const
 {
-    UInt32 modifiers = ::GetCurrentKeyModifiers();
+	UInt32 modifiers = ::GetCurrentKeyModifiers();
 
-    keys.set(GHOST_kModifierKeyCommand, (modifiers & cmdKey) ? true : false);
-    keys.set(GHOST_kModifierKeyLeftAlt, (modifiers & optionKey) ? true : false);
-    keys.set(GHOST_kModifierKeyLeftShift, (modifiers & shiftKey) ? true : false);
-    keys.set(GHOST_kModifierKeyLeftControl, (modifiers & controlKey) ? true : false);
+	keys.set(GHOST_kModifierKeyOS, (modifiers & cmdKey) ? true : false);
+	keys.set(GHOST_kModifierKeyLeftAlt, (modifiers & optionKey) ? true : false);
+	keys.set(GHOST_kModifierKeyLeftShift, (modifiers & shiftKey) ? true : false);
+	keys.set(GHOST_kModifierKeyLeftControl, (modifiers & controlKey) ? true : false);
 	
-    return GHOST_kSuccess;
+	return GHOST_kSuccess;
 }
 
-	/* XXX, incorrect for multibutton mice */
+/* XXX, incorrect for multibutton mice */
 GHOST_TSuccess GHOST_SystemCarbon::getButtons(GHOST_Buttons& buttons) const
 {
-    Boolean theOnlyButtonIsDown = ::Button();
-    buttons.clear();
-    buttons.set(GHOST_kButtonMaskLeft, theOnlyButtonIsDown);
-    return GHOST_kSuccess;
+	Boolean theOnlyButtonIsDown = ::Button();
+	buttons.clear();
+	buttons.set(GHOST_kButtonMaskLeft, theOnlyButtonIsDown);
+	return GHOST_kSuccess;
 }
 
+#define FIRSTFILEBUFLG 512
 static bool g_hasFirstFile = false;
 static char g_firstFileBuf[512];
 
-extern "C" int GHOST_HACK_getFirstFile(char buf[512]) { 
+extern "C" int GHOST_HACK_getFirstFile(char buf[FIRSTFILEBUFLG]) { 
 	if (g_hasFirstFile) {
-		strcpy(buf, g_firstFileBuf);
+		strncpy(buf, g_firstFileBuf, FIRSTFILEBUFLG - 1);
+		buf[FIRSTFILEBUFLG - 1] = '\0';
 		return 1;
 	} else {
 		return 0; 
@@ -600,33 +657,35 @@ OSErr GHOST_SystemCarbon::sAEHandlerQuit(const AppleEvent *event, AppleEvent *re
 
 GHOST_TSuccess GHOST_SystemCarbon::init()
 {
-    GHOST_TSuccess success = GHOST_System::init();
-    if (success) {
+
+	GHOST_TSuccess success = GHOST_System::init();
+	if (success) {
 		/*
-         * Initialize the cursor to the standard arrow shape (so that we can change it later on).
-         * This initializes the cursor's visibility counter to 0.
-         */
-        ::InitCursor();
+		 * Initialize the cursor to the standard arrow shape (so that we can change it later on).
+		 * This initializes the cursor's visibility counter to 0.
+		 */
+		::InitCursor();
 
 		MenuRef windMenu;
 		::CreateStandardWindowMenu(0, &windMenu);
 		::InsertMenu(windMenu, 0);
 		::DrawMenuBar();
 
-        ::InstallApplicationEventHandler(sEventHandlerProc, GetEventTypeCount(kEvents), kEvents, this, &m_handler);
+		::InstallApplicationEventHandler(sEventHandlerProc, GetEventTypeCount(kEvents), kEvents, this, &m_handler);
 		
 		::AEInstallEventHandler(kCoreEventClass, kAEOpenApplication, sAEHandlerLaunch, (SInt32) this, false);
 		::AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, sAEHandlerOpenDocs, (SInt32) this, false);
 		::AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments, sAEHandlerPrintDocs, (SInt32) this, false);
 		::AEInstallEventHandler(kCoreEventClass, kAEQuitApplication, sAEHandlerQuit, (SInt32) this, false);
-    }
-    return success;
+		
+	}
+	return success;
 }
 
 
 GHOST_TSuccess GHOST_SystemCarbon::exit()
 {
-    return GHOST_System::exit();
+	return GHOST_System::exit();
 }
 
 
@@ -684,6 +743,80 @@ OSStatus GHOST_SystemCarbon::handleWindowEvent(EventRef event)
 	return err;
 }
 
+OSStatus GHOST_SystemCarbon::handleTabletEvent(EventRef event)
+{
+	GHOST_IWindow* window = m_windowManager->getActiveWindow();
+	TabletPointRec tabletPointRecord;
+	TabletProximityRec	tabletProximityRecord;
+	UInt32 anInt32;
+	GHOST_TabletData& ct=((GHOST_WindowCarbon*)window)->GetCarbonTabletData();
+	OSStatus err = eventNotHandledErr;
+	
+	ct.Pressure = 0;
+	ct.Xtilt = 0;
+	ct.Ytilt = 0;
+	
+	// is there an embedded tablet event inside this mouse event? 
+	if(noErr == GetEventParameter(event, kEventParamTabletEventType, typeUInt32, NULL, sizeof(UInt32), NULL, (void *)&anInt32))
+	{
+		// yes there is one!
+		// Embedded tablet events can either be a proximity or pointer event.
+		if(anInt32 == kEventTabletPoint)
+		{
+			//GHOST_PRINT("Embedded pointer event!\n");
+			
+			// Extract the tablet Pointer Event. If there is no Tablet Pointer data
+			// in this event, then this call will return an error. Just ignore the
+			// error and go on. This can occur when a proximity event is embedded in
+			// a mouse event and you did not check the mouse event to see which type
+			// of tablet event was embedded.
+			if(noErr == GetEventParameter(event, kEventParamTabletPointRec,
+										  typeTabletPointRec, NULL,
+										  sizeof(TabletPointRec),
+										  NULL, (void *)&tabletPointRecord))
+			{
+				ct.Pressure = tabletPointRecord.pressure / 65535.0f;
+				ct.Xtilt = tabletPointRecord.tiltX / 32767.0f; /* can be positive or negative */
+				ct.Ytilt = tabletPointRecord.tiltY / 32767.0f; /* can be positive or negative */
+			}
+		} else {
+			//GHOST_PRINT("Embedded proximity event\n");
+			
+			// Extract the Tablet Proximity record from the event.
+			if(noErr == GetEventParameter(event, kEventParamTabletProximityRec,
+										  typeTabletProximityRec, NULL,
+										  sizeof(TabletProximityRec),
+										  NULL, (void *)&tabletProximityRecord))
+			{
+				if (tabletProximityRecord.enterProximity) {
+					//pointer is entering tablet area proximity
+					
+					switch(tabletProximityRecord.pointerType)
+					{
+						case 1: /* stylus */
+							ct.Active = GHOST_kTabletModeStylus;
+							break;
+						case 2: /* puck, not supported so far */
+							ct.Active = GHOST_kTabletModeNone;
+							break;
+						case 3: /* eraser */
+							ct.Active = GHOST_kTabletModeEraser;
+							break;
+						default:
+							ct.Active = GHOST_kTabletModeNone;
+							break;
+					}
+				} else {
+					// pointer is leaving - return to mouse
+					ct.Active = GHOST_kTabletModeNone;
+				}
+			}
+		}
+	err = noErr;
+	}
+	return err;
+}
+
 OSStatus GHOST_SystemCarbon::handleMouseEvent(EventRef event)
 {
     OSStatus err = eventNotHandledErr;
@@ -704,23 +837,30 @@ OSStatus GHOST_SystemCarbon::handleMouseEvent(EventRef event)
 				
 				/* Window still gets mouse up after command-H */
 				if (m_windowManager->getActiveWindow()) {
+					// handle any tablet events that may have come with the mouse event (optional)
+					handleTabletEvent(event);
+					
 					::GetEventParameter(event, kEventParamMouseButton, typeMouseButton, NULL, sizeof(button), NULL, &button);
 					pushEvent(new GHOST_EventButton(getMilliSeconds(), type, window, convertButton(button)));
 					err = noErr;
 				}
 			}
-            break;
+			break;
 			
 		case kEventMouseMoved:
-        case kEventMouseDragged:
+		case kEventMouseDragged: {
 			Point mousePos;
+
 			if (window) {
+				//handle any tablet events that may have come with the mouse event (optional)
+				handleTabletEvent(event);
+
 				::GetEventParameter(event, kEventParamMouseLocation, typeQDPoint, NULL, sizeof(Point), NULL, &mousePos);
 				pushEvent(new GHOST_EventCursor(getMilliSeconds(), GHOST_kEventCursorMove, window, mousePos.h, mousePos.v));
 				err = noErr;
 			}
-            break;
-
+			break;
+		}
 		case kEventMouseWheelMoved:
 			{
 				OSStatus status;
@@ -789,7 +929,7 @@ OSStatus GHOST_SystemCarbon::handleKeyEvent(EventRef event)
 				} else {
 					type = GHOST_kEventKeyUp;
 				}
-				pushEvent( new GHOST_EventKey( getMilliSeconds(), type, window, key, ascii) );
+				pushEvent( new GHOST_EventKey( getMilliSeconds(), type, window, key, ascii, NULL) );
 //			}
 			break;
 	
@@ -806,7 +946,7 @@ OSStatus GHOST_SystemCarbon::handleKeyEvent(EventRef event)
 				pushEvent( new GHOST_EventKey(getMilliSeconds(), (modifiers & optionKey)?GHOST_kEventKeyDown:GHOST_kEventKeyUp, window, GHOST_kKeyLeftAlt) );
 			}
 			if ((modifiers & cmdKey) != (m_modifierMask & cmdKey)) {
-				pushEvent( new GHOST_EventKey(getMilliSeconds(), (modifiers & cmdKey)?GHOST_kEventKeyDown:GHOST_kEventKeyUp, window, GHOST_kKeyCommand) );
+				pushEvent( new GHOST_EventKey(getMilliSeconds(), (modifiers & cmdKey)?GHOST_kEventKeyDown:GHOST_kEventKeyUp, window, GHOST_kKeyOS) );
 			}
 			
 			m_modifierMask = modifiers;
@@ -846,10 +986,16 @@ bool GHOST_SystemCarbon::handleMouseDown(EventRef event)
 			 * events. By setting m_ignoreWindowSizedMessages these are suppressed.
 			 * @see GHOST_SystemCarbon::handleWindowEvent(EventRef event)
 			 */
+			/* even worse: scale window also generates a load of events, and nothing 
+			   is handled (read: client's event proc called) until you release mouse (ton) */
+			
 			GHOST_ASSERT(validWindow(ghostWindow), "GHOST_SystemCarbon::handleMouseDown: invalid window");
 			m_ignoreWindowSizedMessages = true;
 			::DragWindow(window, mousePos, &GetQDGlobalsScreenBits(&screenBits)->bounds);
 			m_ignoreWindowSizedMessages = false;
+			
+			pushEvent( new GHOST_Event(getMilliSeconds(), GHOST_kEventWindowMove, ghostWindow) );
+
 			break;
 		
 		case inContent:
@@ -871,7 +1017,7 @@ bool GHOST_SystemCarbon::handleMouseDown(EventRef event)
 			GHOST_ASSERT(ghostWindow, "GHOST_SystemCarbon::handleMouseEvent: ghostWindow==0");
 			if (::TrackGoAway(window, mousePos))
 			{
-				// todo: add option-close, because its in the HIG
+				// todo: add option-close, because itÿs in the HIG
 				// if (event.modifiers & optionKey) {
 					// Close the clean documents, others will be confirmed one by one.
 				//}
@@ -900,7 +1046,7 @@ bool GHOST_SystemCarbon::handleMouseDown(EventRef event)
 							::ZoomWindow(window, part, true);
 							ghostWindow->setMac_windowState(1);
 					} else { // need to force size again
-						GHOST_TUns32 scr_x,scr_y;
+					//	GHOST_TUns32 scr_x,scr_y; /*unused*/
 						Rect outAvailableRect;
 						
 						ghostWindow->setMac_windowState(2);
@@ -946,32 +1092,134 @@ bool GHOST_SystemCarbon::handleMenuCommand(GHOST_TInt32 menuResult)
 	}
 
 	::HiliteMenu(0);
-    return handled;
+	return handled;
 }
 
+
 OSStatus GHOST_SystemCarbon::sEventHandlerProc(EventHandlerCallRef handler, EventRef event, void* userData)
 {
 	GHOST_SystemCarbon* sys = (GHOST_SystemCarbon*) userData;
-    OSStatus err = eventNotHandledErr;
-
-    switch (::GetEventClass(event))
-    {
+	OSStatus err = eventNotHandledErr;
+	GHOST_IWindow* window;
+#ifdef WITH_INPUT_NDOF
+	GHOST_TEventNDOFData data;
+#endif
+	UInt32 kind;
+	
+	switch (::GetEventClass(event))
+	{
 		case kEventClassAppleEvent:
 			EventRecord eventrec;
 			if (ConvertEventRefToEventRecord(event, &eventrec)) {
 				err = AEProcessAppleEvent(&eventrec);
 			}
 			break;
-        case kEventClassMouse:
-            err = sys->handleMouseEvent(event);
-            break;
+		case kEventClassMouse:
+			err = sys->handleMouseEvent(event);
+			break;
 		case kEventClassWindow:
 			err = sys->handleWindowEvent(event);
 			break;
 		case kEventClassKeyboard:
 			err = sys->handleKeyEvent(event);
 			break;
-    }
+		case kEventClassBlender :
+#ifdef WITH_INPUT_NDOF
+			window = sys->m_windowManager->getActiveWindow();
+			sys->m_ndofManager->GHOST_NDOFGetDatas(data);
+			kind = ::GetEventKind(event);
+			
+			switch (kind)
+			{
+				case 1:
+					sys->m_eventManager->pushEvent(new GHOST_EventNDOF(sys->getMilliSeconds(), GHOST_kEventNDOFMotion, window, data));
+	//				printf("motion\n");
+					break;
+				case 2:
+					sys->m_eventManager->pushEvent(new GHOST_EventNDOF(sys->getMilliSeconds(), GHOST_kEventNDOFButton, window, data));
+//					printf("button\n");
+					break;
+			}
+#endif
+			err = noErr;
+			break;
+		default : 
+			;
+			break;
+	}
 
-    return err;
+	return err;
+}
+
+GHOST_TUns8* GHOST_SystemCarbon::getClipboard(bool selection) const
+{
+	PasteboardRef inPasteboard;
+	PasteboardItemID itemID;
+	CFDataRef flavorData;
+	OSStatus err = noErr;
+	GHOST_TUns8 * temp_buff;
+	CFRange range;
+	OSStatus syncFlags;
+	
+	err = PasteboardCreate(kPasteboardClipboard, &inPasteboard);
+	if(err != noErr) { return NULL;}
+
+	syncFlags = PasteboardSynchronize( inPasteboard );
+		/* as we always get in a new string, we can safely ignore sync flags if not an error*/
+	if(syncFlags <0) { return NULL;}
+
+
+	err = PasteboardGetItemIdentifier( inPasteboard, 1, &itemID );
+	if(err != noErr) { return NULL;}
+
+	err = PasteboardCopyItemFlavorData( inPasteboard, itemID, CFSTR("public.utf8-plain-text"), &flavorData);
+	if(err != noErr) { return NULL;}
+
+	range = CFRangeMake(0, CFDataGetLength(flavorData));
+	
+	temp_buff = (GHOST_TUns8*) malloc(range.length+1); 
+
+	CFDataGetBytes(flavorData, range, (UInt8*)temp_buff);
+	
+	temp_buff[range.length] = '\0';
+	
+	if(temp_buff) {
+		return temp_buff;
+	} else {
+		return NULL;
+	}
+}
+
+void GHOST_SystemCarbon::putClipboard(GHOST_TInt8 *buffer, bool selection) const
+{
+	if(selection) {return;} // for copying the selection, used on X11
+
+	PasteboardRef inPasteboard;
+	CFDataRef textData = NULL;
+	OSStatus err = noErr; /*For error checking*/
+	OSStatus syncFlags;
+	
+	err = PasteboardCreate(kPasteboardClipboard, &inPasteboard);
+	if(err != noErr) { return;}
+	
+	syncFlags = PasteboardSynchronize( inPasteboard ); 
+	/* as we always put in a new string, we can safely ignore sync flags */
+	if(syncFlags <0) { return;}
+	
+	err = PasteboardClear( inPasteboard );
+	if(err != noErr) { return;}
+	
+	textData = CFDataCreate(kCFAllocatorDefault, (UInt8*)buffer, strlen(buffer));
+	
+	if (textData) {
+		err = PasteboardPutItemFlavor( inPasteboard, (PasteboardItemID)1, CFSTR("public.utf8-plain-text"), textData, 0);
+			if(err != noErr) { 
+				if(textData) { CFRelease(textData);}
+				return;
+			}
+	}
+	
+	if(textData) {
+		CFRelease(textData);
+	}
 }
