@@ -1,14 +1,10 @@
-/**
- * $Id: GHOST_Window.h,v 1.6 2002/12/28 22:26:45 maarten Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,10 +22,11 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
-/**
- * @file	GHOST_Window.h
+
+/** \file ghost/intern/GHOST_Window.h
+ *  \ingroup GHOST
  * Declaration of GHOST_Window class.
  */
 
@@ -78,24 +75,20 @@ public:
 	 * Constructor.
 	 * Creates a new window and opens it.
 	 * To check if the window was created properly, use the getValid() method.
-	 * @param title		The text shown in the title bar of the window.
-	 * @param left		The coordinate of the left edge of the window.
-	 * @param top		The coordinate of the top edge of the window.
-	 * @param width		The width the window.
-	 * @param heigh		The height the window.
-	 * @param state		The state the window is initially opened with.
-	 * @param type		The type of drawing context installed in this window.
-	 * @param stereoVisual	Stereo visual for quad buffered stereo.
+	 * @param width				The width the window.
+	 * @param heigh				The height the window.
+	 * @param state				The state the window is initially opened with.
+	 * @param type				The type of drawing context installed in this window.
+	 * @param stereoVisual		Stereo visual for quad buffered stereo.
+	 * @param numOfAASamples	Number of samples used for AA (zero if no AA)
 	 */
 	GHOST_Window(
-		const STR_String& title, 
-		GHOST_TInt32 left,
-		GHOST_TInt32 top,
 		GHOST_TUns32 width,
 		GHOST_TUns32 height,
 		GHOST_TWindowState state,
 		GHOST_TDrawingContextType type = GHOST_kDrawingContextTypeNone,
-		const bool stereoVisual = false);
+		const bool stereoVisual = false,
+		const GHOST_TUns16 numOfAASamples = 0);
 
 	/**
 	 * @section Interface inherited from GHOST_IWindow left for derived class
@@ -124,6 +117,12 @@ public:
 	 */
 	virtual ~GHOST_Window();
 
+	/**
+	 * Returns the associated OS object/handle
+	 * @return The associated OS object/handle
+	 */
+	virtual void* getOSWindow() const;
+	
 	/**
 	 * Returns the current cursor shape.
 	 * @return	The current cursor shape.
@@ -149,12 +148,22 @@ public:
 												GHOST_TUns8 mask[16][2], 
 												int hotX, 
 												int hotY);
+												
+	virtual GHOST_TSuccess setCustomCursorShape(GHOST_TUns8 *bitmap, 
+												GHOST_TUns8 *mask, 
+												int sizex, int sizey,
+												int hotX,  int hotY,
+												int fg_color, int bg_color);
 	
 	/**
 	 * Returns the visibility state of the cursor.
 	 * @return	The visibility state of the cursor.
 	 */
 	inline virtual bool getCursorVisibility() const;
+	inline virtual GHOST_TGrabCursorMode getCursorGrabMode() const;
+	inline virtual void getCursorGrabInitPos(GHOST_TInt32 &x, GHOST_TInt32 &y) const;
+	inline virtual void getCursorGrabAccum(GHOST_TInt32 &x, GHOST_TInt32 &y) const;
+	inline virtual void setCursorGrabAccum(GHOST_TInt32 x, GHOST_TInt32 y);
 
 	/**
 	 * Shows or hides the cursor.
@@ -163,6 +172,54 @@ public:
 	 */
 	virtual GHOST_TSuccess setCursorVisibility(bool visible);
 
+	/**
+	 * Sets the cursor grab.
+	 * @param	mode The new grab state of the cursor.
+	 * @return	Indication of success.
+	 */
+	virtual GHOST_TSuccess setCursorGrab(GHOST_TGrabCursorMode mode, GHOST_Rect *bounds);
+
+	/**
+	 * Gets the cursor grab region, if unset the window is used.
+	 * reset when grab is disabled.
+	 */
+	virtual GHOST_TSuccess getCursorGrabBounds(GHOST_Rect& bounds);
+
+	/**
+     * Sets the progress bar value displayed in the window/application icon
+	 * @param progress The progress % (0.0 to 1.0)
+	 */
+	virtual GHOST_TSuccess setProgressBar(float progress) {return GHOST_kFailure;};
+	
+	/**
+	 * Hides the progress bar in the icon
+	 */
+	virtual GHOST_TSuccess endProgressBar() {return GHOST_kFailure;};
+	
+	/**
+	 * Tells if the ongoing drag'n'drop object can be accepted upon mouse drop
+	 */
+	virtual void setAcceptDragOperation(bool canAccept);
+	
+	/**
+	 * Returns acceptance of the dropped object
+	 * Usually called by the "object dropped" event handling function
+	 */
+	virtual bool canAcceptDragOperation() const;
+	
+	/**
+	 * Sets the window "modified" status, indicating unsaved changes
+	 * @param isUnsavedChanges Unsaved changes or not
+	 * @return Indication of success.
+	 */
+	virtual GHOST_TSuccess setModifiedState(bool isUnsavedChanges);
+	
+	/**
+	 * Gets the window "modified" status, indicating unsaved changes
+	 * @return True if there are unsaved changes
+	 */
+	virtual bool getModifiedState();
+	
 	/**
 	 * Returns the type of drawing context used in this window.
 	 * @return The current type of drawing context.
@@ -215,6 +272,12 @@ protected:
 	 * native window system calls.
 	 */
 	virtual GHOST_TSuccess setWindowCursorVisibility(bool visible) = 0;
+
+	/**
+	 * Sets the cursor grab on the window using
+	 * native window system calls.
+	 */
+	virtual GHOST_TSuccess setWindowCursorGrab(GHOST_TGrabCursorMode mode) { return GHOST_kSuccess; };
 	
 	/**
 	 * Sets the cursor shape on the window using
@@ -226,8 +289,11 @@ protected:
 	 * Sets the cursor shape on the window using
 	 * native window system calls.
 	 */
-	virtual GHOST_TSuccess setWindowCustomCursorShape(GHOST_TUns8 bitmap[16][2], GHOST_TUns8 mask[16][2], int hotX, int hotY) = 0;
-
+	virtual GHOST_TSuccess setWindowCustomCursorShape(GHOST_TUns8 bitmap[16][2], GHOST_TUns8 mask[16][2],
+							 int hotX, int hotY) = 0;
+	
+	virtual GHOST_TSuccess setWindowCustomCursorShape(GHOST_TUns8 *bitmap, GHOST_TUns8 *mask, 
+						int szx, int szy, int hotX, int hotY, int fg, int bg) = 0;
 	/** The the of drawing context installed in this window. */
 	GHOST_TDrawingContextType m_drawingContextType;
 	
@@ -236,10 +302,31 @@ protected:
 
 	/** The current visibility of the cursor */
 	bool m_cursorVisible;
-	
+
+	/** The current grabbed state of the cursor */
+	GHOST_TGrabCursorMode m_cursorGrab;
+
+	/** Initial grab location. */
+	GHOST_TInt32 m_cursorGrabInitPos[2];
+
+	/** Accumulated offset from m_cursorGrabInitPos. */
+	GHOST_TInt32 m_cursorGrabAccumPos[2];
+
+	/** Wrap the cursor within this region. */
+	GHOST_Rect m_cursorGrabBounds;
+
 	/** The current shape of the cursor */
 	GHOST_TStandardCursor m_cursorShape;
     
+	/** The presence of progress indicator with the application icon */
+	bool m_progressBarVisible;
+	
+	/** The acceptance of the "drop candidate" of the current drag'n'drop operation */
+	bool m_canAcceptDragOperation;
+	
+	/** Modified state : are there unsaved changes */
+	bool m_isUnsavedChanges;
+	
 	/** Stores wether this is a full screen window. */
 	bool m_fullScreen;
 
@@ -248,11 +335,14 @@ protected:
 	 *  the graphics h/w
 	 */
 	bool m_stereoVisual;
-    
-    /** Full-screen width */
-    GHOST_TUns32 m_fullScreenWidth;
-    /** Full-screen height */
-    GHOST_TUns32 m_fullScreenHeight;
+	
+	/** Number of samples used in anti-aliasing, set to 0 if no AA **/
+	GHOST_TUns16 m_numOfAASamples;
+
+	/** Full-screen width */
+	GHOST_TUns32 m_fullScreenWidth;
+	/** Full-screen height */
+	GHOST_TUns32 m_fullScreenHeight;
 };
 
 
@@ -266,6 +356,29 @@ inline bool GHOST_Window::getCursorVisibility() const
 	return m_cursorVisible;
 }
 
+inline GHOST_TGrabCursorMode GHOST_Window::getCursorGrabMode() const
+{
+	return m_cursorGrab;
+}
+
+inline void GHOST_Window::getCursorGrabInitPos(GHOST_TInt32 &x, GHOST_TInt32 &y) const
+{
+	x = m_cursorGrabInitPos[0];
+	y = m_cursorGrabInitPos[1];
+}
+
+inline void GHOST_Window::getCursorGrabAccum(GHOST_TInt32 &x, GHOST_TInt32 &y) const
+{
+	x= m_cursorGrabAccumPos[0];
+	y= m_cursorGrabAccumPos[1];
+}
+
+inline void GHOST_Window::setCursorGrabAccum(GHOST_TInt32 x, GHOST_TInt32 y)
+{
+	m_cursorGrabAccumPos[0]= x;
+	m_cursorGrabAccumPos[1]= y;
+}
+
 inline GHOST_TStandardCursor GHOST_Window::getCursorShape() const
 {
 	return m_cursorShape;
