@@ -1,14 +1,10 @@
-/**
- * $Id: GHOST_WindowCarbon.cpp,v 1.10 2003/12/26 20:12:40 rwenzlaff Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,21 +22,21 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
+/** \file ghost/intern/GHOST_WindowCarbon.cpp
+ *  \ingroup GHOST
+ */
+
+
 /**
 
- * $Id: GHOST_WindowCarbon.cpp,v 1.10 2003/12/26 20:12:40 rwenzlaff Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 10, 2001
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "GHOST_WindowCarbon.h"
 #include "GHOST_Debug.h"
 
@@ -49,25 +45,50 @@ AGLContext GHOST_WindowCarbon::s_firstaglCtx = NULL;
 const GHOST_TInt32 GHOST_WindowCarbon::s_sizeRectSize = 16;
 #endif //GHOST_DRAW_CARBON_GUTTER
 
-static const GLint sPreferredFormatWindow[9] = {
-AGL_RGBA,			GL_TRUE,
-AGL_DOUBLEBUFFER,	GL_TRUE,
-AGL_DEPTH_SIZE,		16,
-AGL_AUX_BUFFERS,     1,
+static const GLint sPreferredFormatWindow[10] = {
+AGL_RGBA,
+AGL_DOUBLEBUFFER,	
+AGL_ACCELERATED,
+AGL_DEPTH_SIZE,		32,
 AGL_NONE,
 };
 
-static const GLint sPreferredFormatFullScreen[7] = {
+static const GLint sPreferredFormatFullScreen[11] = {
 AGL_RGBA,
 AGL_DOUBLEBUFFER,
 AGL_ACCELERATED,
 AGL_FULLSCREEN,
-AGL_DEPTH_SIZE,		16,
+AGL_DEPTH_SIZE,		32,
 AGL_NONE,
 };
 
+
+
 WindowRef ugly_hack=NULL;
 
+const EventTypeSpec	kWEvents[] = {
+	{ kEventClassWindow, kEventWindowZoom },  /* for new zoom behaviour */ 
+};
+
+static OSStatus myWEventHandlerProc(EventHandlerCallRef handler, EventRef event, void* userData) {
+	WindowRef mywindow;
+	GHOST_WindowCarbon *ghost_window;
+	OSStatus err;
+	int theState;
+	
+	if (::GetEventKind(event) == kEventWindowZoom) {
+		err =  ::GetEventParameter (event,kEventParamDirectObject,typeWindowRef,NULL,sizeof(mywindow),NULL, &mywindow);
+		ghost_window = (GHOST_WindowCarbon *) GetWRefCon(mywindow);
+		theState = ghost_window->getMac_windowState();
+		if (theState == 1) 
+			ghost_window->setMac_windowState(2);
+		else if (theState == 2)
+			ghost_window->setMac_windowState(1);
+
+	}
+	return eventNotHandledErr;
+}
+
 GHOST_WindowCarbon::GHOST_WindowCarbon(
 	const STR_String& title,
 	GHOST_TInt32 left,
@@ -76,9 +97,10 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
 	GHOST_TUns32 height,
 	GHOST_TWindowState state,
 	GHOST_TDrawingContextType type,
-	const bool stereoVisual
+	const bool stereoVisual,
+	const GHOST_TUns16 numOfAASamples
 ) :
-	GHOST_Window(title, left, top, width, height, state, GHOST_kDrawingContextTypeNone),
+	GHOST_Window(width, height, state, GHOST_kDrawingContextTypeNone),
 	m_windowRef(0),
 	m_grafPtr(0),
 	m_aglCtx(0),
@@ -86,21 +108,48 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
 	m_fullScreenDirty(false)
 {
     Str255 title255;
+	OSStatus err;
+	
+	//fprintf(stderr," main screen top %i left %i height %i width %i\n", top, left, height, width);
+	
+	if (state >= GHOST_kWindowState8Normal ) {
+		if(state == GHOST_kWindowState8Normal) state= GHOST_kWindowStateNormal;
+		else if(state == GHOST_kWindowState8Maximized) state= GHOST_kWindowStateMaximized;
+		else if(state == GHOST_kWindowState8Minimized) state= GHOST_kWindowStateMinimized;
+		else if(state == GHOST_kWindowState8FullScreen) state= GHOST_kWindowStateFullScreen;
+		
+		// state = state - 8;	this was the simple version of above code, doesnt work in gcc 4.0
+		
+		setMac_windowState(1);
+	} else 
+		setMac_windowState(0);
 
 	if (state != GHOST_kWindowStateFullScreen) {
         Rect bnds = { top, left, top+height, left+width };
-        Boolean visible = (state == GHOST_kWindowStateNormal) || (state == GHOST_kWindowStateMaximized);
+        // Boolean visible = (state == GHOST_kWindowStateNormal) || (state == GHOST_kWindowStateMaximized); /*unused*/
         gen2mac(title, title255);
         
-        m_windowRef = ::NewCWindow(
-            nil,							// Storage 
-            &bnds,							// Bounding rectangle of the window
-            title255,						// Title of the window
-            visible,						// Window initially visible
-            kWindowFullZoomGrowDocumentProc, //kWindowGrowDocumentProc,		// procID
-            (WindowRef)-1L,					// Put window before all other windows
-            true,							// Window has minimize box
-            (SInt32)this);					// Store a pointer to the class in the refCon
+		err =  ::CreateNewWindow( kDocumentWindowClass,
+								 kWindowStandardDocumentAttributes+kWindowLiveResizeAttribute,
+								 &bnds,
+								 &m_windowRef);
+		
+		if ( err != noErr) {
+			fprintf(stderr," error creating window %i \n",(int)err);
+		} else {
+			
+			::SetWRefCon(m_windowRef,(SInt32)this);
+			setTitle(title);
+			err = InstallWindowEventHandler (m_windowRef, myWEventHandlerProc, GetEventTypeCount(kWEvents), kWEvents,NULL,NULL); 
+			if ( err != noErr) {
+				fprintf(stderr," error creating handler %i \n",(int)err);
+			} else {
+				//	::TransitionWindow (m_windowRef,kWindowZoomTransitionEffect,kWindowShowTransitionAction,NULL);
+				::ShowWindow(m_windowRef);
+				::MoveWindow (m_windowRef, left, top,true);
+				
+			}
+		}
         if (m_windowRef) {
             m_grafPtr = ::GetWindowPort(m_windowRef);
             setDrawingContextType(type);
@@ -130,9 +179,11 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
             (SInt32)this);					// Store a pointer to the class in the refCon
     */
         //GHOST_PRINT("GHOST_WindowCarbon::GHOST_WindowCarbon(): creating full-screen OpenGL context\n");
-        setDrawingContextType(GHOST_kDrawingContextTypeOpenGL);
+        setDrawingContextType(GHOST_kDrawingContextTypeOpenGL);;installDrawingContext(GHOST_kDrawingContextTypeOpenGL);
         updateDrawingContext();
         activateDrawingContext();        
+
+	m_tablet.Active = GHOST_kTabletModeNone;
     }
 }
 
@@ -199,7 +250,9 @@ void GHOST_WindowCarbon::getClientBounds(GHOST_Rect& bounds) const
 {
 	Rect rect;
 	GHOST_ASSERT(getValid(), "GHOST_WindowCarbon::getClientBounds(): window invalid")
-	::GetPortBounds(m_grafPtr, &rect);
+	//::GetPortBounds(m_grafPtr, &rect);
+	::GetWindowBounds(m_windowRef, kWindowContentRgn, &rect);
+
 	bounds.m_b = rect.bottom;
 	bounds.m_l = rect.left;
 	bounds.m_r = rect.right;
@@ -273,7 +326,7 @@ GHOST_TWindowState GHOST_WindowCarbon::getState() const
 {
 	GHOST_ASSERT(getValid(), "GHOST_WindowCarbon::getState(): window invalid")
 	GHOST_TWindowState state;
-	if (::IsWindowVisible(m_windowRef)) {
+	if (::IsWindowVisible(m_windowRef) == false) {
 		state = GHOST_kWindowStateMinimized;
 	}
 	else if (::IsWindowInStandardState(m_windowRef, nil, nil)) {
@@ -325,6 +378,12 @@ GHOST_TSuccess GHOST_WindowCarbon::setState(GHOST_TWindowState state)
 	case GHOST_kWindowStateMinimized:
             ::HideWindow(m_windowRef);
             break;
+	case GHOST_kWindowStateModified:
+		SetWindowModified(m_windowRef, 1);
+		break;
+	case GHOST_kWindowStateUnModified:
+		SetWindowModified(m_windowRef, 0);
+		break;
 	case GHOST_kWindowStateMaximized:
 	case GHOST_kWindowStateNormal:
         default:
@@ -349,9 +408,19 @@ GHOST_TSuccess GHOST_WindowCarbon::setOrder(GHOST_TWindowOrder order)
     return GHOST_kSuccess;
 }
 
+/*#define  WAIT_FOR_VSYNC 1*/
+#ifdef WAIT_FOR_VSYNC
+#include <OpenGL/OpenGL.h>
+#endif
 
 GHOST_TSuccess GHOST_WindowCarbon::swapBuffers()
 {
+#ifdef WAIT_FOR_VSYNC
+/* wait for vsync, to avoid tearing artifacts */
+long VBL = 1;
+CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &VBL);
+#endif
+
     GHOST_TSuccess succeeded = GHOST_kSuccess;
     if (m_drawingContextType == GHOST_kDrawingContextTypeOpenGL) {
         if (m_aglCtx) {
@@ -425,7 +494,7 @@ GHOST_TSuccess GHOST_WindowCarbon::installDrawingContext(GHOST_TDrawingContextTy
             }
             else {
                 //GHOST_PRINT("GHOST_WindowCarbon::installDrawingContext(): init full-screen OpenGL\n");
-                pixelFormat = ::aglChoosePixelFormat(0, 0, sPreferredFormatFullScreen);
+GDHandle device=::GetMainDevice();pixelFormat=::aglChoosePixelFormat(&device,1,sPreferredFormatFullScreen);
                 m_aglCtx = ::aglCreateContext(pixelFormat, 0);
                 if (!m_aglCtx) break;
 				if (!s_firstaglCtx) s_firstaglCtx = m_aglCtx;
@@ -565,6 +634,7 @@ void GHOST_WindowCarbon::loadCursor(bool visible, GHOST_TStandardCursor cursor)
 		GCMAP( GHOST_kStandardCursorTopRightCorner,			kThemeArrowCursor);
 		GCMAP( GHOST_kStandardCursorBottomRightCorner,		kThemeArrowCursor);
 		GCMAP( GHOST_kStandardCursorBottomLeftCorner,		kThemeArrowCursor);
+		GCMAP( GHOST_kStandardCursorCopy,					kThemeCopyArrowCursor);
 		};
 #undef GCMAP
 
@@ -602,6 +672,7 @@ GHOST_TSuccess GHOST_WindowCarbon::setWindowCursorShape(GHOST_TStandardCursor sh
 	return GHOST_kSuccess;
 }
 
+#if 0
 /** Reverse the bits in a GHOST_TUns8 */
 static GHOST_TUns8 uns8ReverseBits(GHOST_TUns8 ch)
 {
@@ -610,6 +681,8 @@ static GHOST_TUns8 uns8ReverseBits(GHOST_TUns8 ch)
 	ch= ((ch>>4)&0x0F) | ((ch<<4)&0xF0);
 	return ch;
 }
+#endif
+
 
 /** Reverse the bits in a GHOST_TUns16 */
 static GHOST_TUns16 uns16ReverseBits(GHOST_TUns16 shrt)
@@ -635,8 +708,14 @@ GHOST_TSuccess GHOST_WindowCarbon::setWindowCustomCursorShape(GHOST_TUns8 *bitma
 	if (!m_customCursor) return GHOST_kFailure;
 	
 	for (y=0; y<16; y++) {
+#if !defined(__LITTLE_ENDIAN__)
 		m_customCursor->data[y] = uns16ReverseBits((bitmap[2*y]<<0) | (bitmap[2*y+1]<<8));
 		m_customCursor->mask[y] = uns16ReverseBits((mask[2*y]<<0) | (mask[2*y+1]<<8));
+#else
+		m_customCursor->data[y] = uns16ReverseBits((bitmap[2*y+1]<<0) | (bitmap[2*y]<<8));
+		m_customCursor->mask[y] = uns16ReverseBits((mask[2*y+1]<<0) | (mask[2*y]<<8));
+#endif
+			
 	}
 	
 	m_customCursor->hotSpot.h = hotX;
@@ -652,5 +731,16 @@ GHOST_TSuccess GHOST_WindowCarbon::setWindowCustomCursorShape(GHOST_TUns8 *bitma
 GHOST_TSuccess GHOST_WindowCarbon::setWindowCustomCursorShape(GHOST_TUns8 bitmap[16][2], 
 												GHOST_TUns8 mask[16][2], int hotX, int hotY)
 {
-	setWindowCustomCursorShape((GHOST_TUns8*)bitmap, (GHOST_TUns8*) mask, 16, 16, hotX, hotY, 0, 1);
+	return setWindowCustomCursorShape((GHOST_TUns8*)bitmap, (GHOST_TUns8*) mask, 16, 16, hotX, hotY, 0, 1);
+}
+
+
+void GHOST_WindowCarbon::setMac_windowState(short value)
+{
+	mac_windowState = value;
+}
+
+short GHOST_WindowCarbon::getMac_windowState()
+{
+	return mac_windowState;
 }
