@@ -1,24 +1,10 @@
-/**
- * blenlib/BKE_main.h (mar-2001 nzc)
- *
- * Main is the root of the 'database' of a Blender context. All data
- * is stuffed into lists, and all these lists are knotted to here. A
- * Blender file is not much more but a binary dump of these
- * lists. This list of lists is not serialized itself.
- *
- * Oops... this should be a _types.h file.
- *
- * $Id: BKE_main.h,v 1.6 2003/12/14 01:18:02 ianwill Exp $ 
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -36,19 +22,39 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef BKE_MAIN_H
 #define BKE_MAIN_H
 
+/** \file BKE_main.h
+ *  \ingroup bke
+ *  \since March 2001
+ *  \author nzc
+ *  \section aboutmain Main struct
+ * Main is the root of the 'database' of a Blender context. All data
+ * is stuffed into lists, and all these lists are knotted to here. A
+ * Blender file is not much more but a binary dump of these
+ * lists. This list of lists is not serialized itself.
+ *
+ * Oops... this should be a _types.h file.
+ *
+ */
 #include "DNA_listBase.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct Library;
 
 typedef struct Main {
 	struct Main *next, *prev;
-	char name[160];
-	short versionfile, rt;
+	char name[240];
+	short versionfile, subversionfile;
+	short minversionfile, minsubversionfile;
+	int revision;	/* svn revision of binary that saved file */
+	
 	struct Library *curlib;
 	ListBase scene;
 	ListBase library;
@@ -59,24 +65,35 @@ typedef struct Main {
 	ListBase mat;
 	ListBase tex;
 	ListBase image;
-	ListBase ika;
-	ListBase wave;
 	ListBase latt;
 	ListBase lamp;
 	ListBase camera;
-	ListBase ipo;
+	ListBase ipo;	// XXX depreceated
 	ListBase key;
 	ListBase world;
 	ListBase screen;
 	ListBase script;
 	ListBase vfont;
 	ListBase text;
+	ListBase speaker;
 	ListBase sound;
 	ListBase group;
-	ListBase armature;	/* NLA */
-	ListBase action;	/* NLA */
+	ListBase armature;
+	ListBase action;
+	ListBase nodetree;
+	ListBase brush;
+	ListBase particle;
+	ListBase wm;
+	ListBase gpencil;
+	ListBase movieclip;
+
+	char id_tag_update[256];
 } Main;
 
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
 
