@@ -1,23 +1,10 @@
-/**
- * @file BLI_dynstr.h
- * 
- * A dynamically sized string ADT.
- * This ADT is designed purely for dynamic string creation
- * through appending, not for general usage, the intent is
- * to build up dynamic strings using a DynStr object, then
- * convert it to a c-string and work with that.
- * 
- * $Id: BLI_dynstr.h,v 1.5 2002/12/27 13:10:12 mein Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -35,12 +22,24 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
  
 #ifndef BLI_DYNSTR_H
 #define BLI_DYNSTR_H
 
+/** \file BLI_dynstr.h
+ *  \ingroup bli
+ *  \brief A dynamically sized string ADT.
+ *  \section aboutdynstr Dynamic String
+ * This ADT is designed purely for dynamic string creation
+ * through appending, not for general usage, the intent is
+ * to build up dynamic strings using a DynStr object, then
+ * convert it to a c-string and work with that.
+ */
+
+#include <stdarg.h>
+
 struct DynStr;
 
 	/** The abstract DynStr type */
@@ -59,7 +58,29 @@ DynStr*	BLI_dynstr_new					(void);
 	 * @param ds The DynStr to append to.
 	 * @param cstr The c-string to append.
 	 */
-void	BLI_dynstr_append				(DynStr *ds, char *cstr);
+void	BLI_dynstr_append				(DynStr *ds, const char *cstr);
+
+/**
+ * Append a length clamped c-string to a DynStr.
+ *
+ * @param ds The DynStr to append to.
+ * @param cstr The c-string to append.
+ * @param len The maximum length of the c-string to copy.
+ */
+void	BLI_dynstr_nappend				(DynStr *ds, const char *cstr, int len);
+
+	/**
+	 * Append a c-string to a DynStr, but with formatting like printf.
+	 * 
+	 * @param ds The DynStr to append to.
+	 * @param format The printf format string to use.
+	 */
+void	BLI_dynstr_appendf				(DynStr *ds, const char *format, ...)
+#ifdef __GNUC__
+__attribute__ ((format (printf, 2, 3)))
+#endif
+;
+void	BLI_dynstr_vappendf				(DynStr *ds, const char *format, va_list args);
 
 	/**
 	 * Find the length of a DynStr.
@@ -72,7 +93,7 @@ int		BLI_dynstr_get_len				(DynStr *ds);
 	/**
 	 * Get a DynStr's contents as a c-string.
 	 * <i> The returned c-string should be free'd
-	 * using BLI_freeN. </i>
+	 * using MEM_freeN. </i>
 	 * 
 	 * @param ds The DynStr of interest.
 	 * @return The contents of @a ds as a c-string.
