@@ -1,15 +1,10 @@
-/**
- * $Id: BLI_linklist.c,v 1.3 2002/11/25 12:01:55 mein Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,19 +22,19 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * Support for linked lists.
  */
 
+/** \file blender/blenlib/intern/BLI_linklist.c
+ *  \ingroup bli
+ */
+
+
 #include "MEM_guardedalloc.h"
-#include "BLI_blenlib.h"
 #include "BLI_linklist.h"
 #include "BLI_memarena.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 int BLI_linklist_length(LinkNode *list) {
 	if (0) {
 		return list?(1+BLI_linklist_length(list->next)):0;
@@ -53,6 +48,28 @@ int BLI_linklist_length(LinkNode *list) {
 	}
 }
 
+int BLI_linklist_index(LinkNode *list, void *ptr)
+{
+	int index;
+	
+	for (index = 0; list; list= list->next, index++)
+		if (list->link == ptr)
+			return index;
+	
+	return -1;
+}
+
+LinkNode *BLI_linklist_find(LinkNode *list, int index)
+{
+	int i;
+	
+	for (i = 0; list; list= list->next, i++)
+		if (i == index)
+			return list;
+
+	return NULL;
+}
+
 void BLI_linklist_reverse(LinkNode **listp) {
 	LinkNode *rhead= NULL, *cur= *listp;
 	
@@ -76,6 +93,23 @@ void BLI_linklist_prepend(LinkNode **listp, void *ptr) {
 	*listp= nlink;
 }
 
+void BLI_linklist_append(LinkNode **listp, void *ptr) {
+	LinkNode *nlink= MEM_mallocN(sizeof(*nlink), "nlink");
+	LinkNode *node = *listp;
+	
+	nlink->link = ptr;
+	nlink->next = NULL;
+	
+	if(node == NULL){
+		*listp = nlink;
+	} else {
+		while(node->next != NULL){
+			node = node->next;   
+		}
+		node->next = nlink;
+	}
+}
+
 void BLI_linklist_prepend_arena(LinkNode **listp, void *ptr, MemArena *ma) {
 	LinkNode *nlink= BLI_memarena_alloc(ma, sizeof(*nlink));
 	nlink->link= ptr;
@@ -84,6 +118,22 @@ void BLI_linklist_prepend_arena(LinkNode **listp, void *ptr, MemArena *ma) {
 	*listp= nlink;
 }
 
+void BLI_linklist_insert_after(LinkNode **listp, void *ptr) {
+	LinkNode *nlink= MEM_mallocN(sizeof(*nlink), "nlink");
+	LinkNode *node = *listp;
+
+	nlink->link = ptr;
+
+	if(node) {
+		nlink->next = node->next;
+		node->next = nlink;
+	}
+	else {
+		nlink->next = NULL;
+		*listp = nlink;
+	}
+}
+
 void BLI_linklist_free(LinkNode *list, LinkNodeFreeFP freefunc) {
 	while (list) {
 		LinkNode *next= list->next;
@@ -96,7 +146,7 @@ void BLI_linklist_free(LinkNode *list, LinkNodeFreeFP freefunc) {
 	}
 }
 
-void BLI_linklist_apply(LinkNode *list, LinkNodeApplyFP applyfunc) {
+void BLI_linklist_apply(LinkNode *list, LinkNodeApplyFP applyfunc, void *userdata) {
 	for (; list; list= list->next)
-		applyfunc(list->link);
+		applyfunc(list->link, userdata);
 }
