@@ -1,17 +1,10 @@
-/**
- * blenlib/DNA_key_types.h (mar-2001 nzc)
- *
- * $Id: DNA_key_types.h,v 1.7 2003/11/19 17:07:17 ton Exp $ 
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -29,26 +22,34 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 #ifndef DNA_KEY_TYPES_H
 #define DNA_KEY_TYPES_H
 
+/** \file DNA_key_types.h
+ *  \ingroup DNA
+ */
+
+#include "DNA_defs.h"
 #include "DNA_listBase.h"
 #include "DNA_ID.h"
 
+struct AnimData;
 struct Ipo;
 
 typedef struct KeyBlock {
 	struct KeyBlock *next, *prev;
 	
 	float pos;
-	short flag, pad;
-	short type, rt;
-	int totelem;
+	float curval;
+	short type, adrcode, relative, flag;	/* relative == 0 means first key is reference */
+	int totelem, pad2;
 	
 	void *data;
+	float *weights;
 	char  name[32];
+	char vgroup[32];
 
 	float slidermin;
 	float slidermax;
@@ -57,32 +58,39 @@ typedef struct KeyBlock {
 
 typedef struct Key {
 	ID id;
+	struct AnimData *adt;	/* animation data (must be immediately after id for utilities to use it) */ 
 	
 	KeyBlock *refkey;
 	char elemstr[32];
 	int elemsize;
-	float curval;
+	float curval  DNA_DEPRECATED;
 	
 	ListBase block;
-	struct Ipo *ipo;
+	struct Ipo *ipo  DNA_DEPRECATED;  /* old animation system, deprecated for 2.5 */
 	
 	ID *from;
 
 	short type, totkey;
-	short slurph, actkey;
-	
+	short slurph, flag;
 } Key;
 
 /* **************** KEY ********************* */
 
-/* type */
+/* key->type */
 #define KEY_NORMAL      0
 #define KEY_RELATIVE    1
 
+/* key->flag */
+#define KEY_DS_EXPAND	1
+
 /* keyblock->type */
 #define KEY_LINEAR      0
 #define KEY_CARDINAL    1
 #define KEY_BSPLINE     2
 
-#endif
+/* keyblock->flag */
+#define KEYBLOCK_MUTE			(1<<0)
+#define KEYBLOCK_SEL			(1<<1)
+#define KEYBLOCK_LOCKED			(1<<2)
 
+#endif
