@@ -1,18 +1,10 @@
-/* DNA_packedFile_types.h 
- * 
- * 12-oct-2000 nzc
- * 
- * $Id: DNA_packedFile_types.h,v 1.5 2002/12/27 13:10:28 mein Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -30,17 +22,21 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #ifndef DNA_PACKEDFILE_TYPES_H
 #define DNA_PACKEDFILE_TYPES_H
 
+/** \file DNA_packedFile_types.h
+ *  \ingroup DNA
+ *  \author nzc
+ *  \since 12-oct-2000 nzc
+ */
+
 typedef struct PackedFile {
 	int size;
 	int seek;
-	int flags;
-	int pad;
 	void * data;
 } PackedFile;
 
@@ -55,6 +51,7 @@ enum PF_FileStatus
 	PF_USE_LOCAL,
 	PF_USE_ORIGINAL,
 	PF_KEEP,
+	PF_REMOVE,
 	PF_NOOP,
 			
 	PF_ASK
@@ -62,3 +59,4 @@ enum PF_FileStatus
 
 #endif /* PACKEDFILE_TYPES_H */
 
+
