@@ -1,15 +1,10 @@
-/**
- * $Id: makesdna.c,v 1.7 2003/12/16 01:27:37 hos Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,10 +22,14 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
- *
- * Struct muncher for making SDNA
+ * ***** END GPL LICENSE BLOCK *****
+ */
+
+/**  \file makesdna.c
+ *   \brief Struct muncher for making SDNA.
+ *   \ingroup DNA
  *
+ * \section aboutmakesdnac About makesdna tool
  * Originally by Ton, some mods by Frank, and some cleaning and
  * extension by Nzc.
  *
@@ -45,11 +44,11 @@
  * and the offsets for reaching a particular one.
  *
  * There is a facility to get verbose output from sdna. Search for
- * debugSDNA. This int can be set to 0 (no output) to some int. Higher
+ * \ref debugSDNA. This int can be set to 0 (no output) to some int. Higher
  * numbers give more output.
  * */
 
-#define DNA_VERSION_DATE "$Id: makesdna.c,v 1.7 2003/12/16 01:27:37 hos Exp $"
+#define DNA_VERSION_DATE "FIXME-DNA_VERSION_DATE"
 
 #include <string.h>
 #include <stdlib.h>
@@ -58,16 +57,14 @@
 #include "MEM_guardedalloc.h"
 #include "DNA_sdna_types.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "BLO_sys_types.h" // for intptr_t support
 
 #define SDNA_MAX_FILENAME_LENGTH 255
 
 
 /* Included the path relative from /source/blender/ here, so we can move     */
 /* headers around with more freedom.                                         */
-char *includefiles[] = {
+static const char *includefiles[] = {
 
 	// if you add files here, please add them at the end
 	// of makesdna.c (this file) as well
@@ -77,14 +74,12 @@ char *includefiles[] = {
 	"DNA_ID.h",
 	"DNA_ipo_types.h",
 	"DNA_key_types.h",
-	"DNA_scriptlink_types.h",
 	"DNA_text_types.h",
 	"DNA_packedFile_types.h",
 	"DNA_camera_types.h",
 	"DNA_image_types.h",
 	"DNA_texture_types.h",
 	"DNA_lamp_types.h",
-	"DNA_wave_types.h",
 	"DNA_material_types.h",
 	"DNA_vfont_types.h",
 	// if you add files here, please add them at the end
@@ -92,10 +87,13 @@ char *includefiles[] = {
 	"DNA_meta_types.h",
 	"DNA_curve_types.h",
 	"DNA_mesh_types.h",
+	"DNA_meshdata_types.h",
+	"DNA_modifier_types.h",
 	"DNA_lattice_types.h",	
 	"DNA_object_types.h",
+	"DNA_object_force.h",
+	"DNA_object_fluidsim.h",
 	"DNA_world_types.h",
-	"DNA_radio_types.h",
 	"DNA_scene_types.h",
 	"DNA_view3d_types.h",
 	"DNA_view2d_types.h",	
@@ -108,8 +106,7 @@ char *includefiles[] = {
 	"DNA_fileglobal_types.h",
 	"DNA_sequence_types.h",
 	"DNA_effect_types.h",
-	"DNA_ika_types.h",
-	"DNA_oops_types.h",
+	"DNA_outliner_types.h",
 	"DNA_property_types.h",
 	"DNA_sensor_types.h",
 	"DNA_controller_types.h",
@@ -120,75 +117,94 @@ char *includefiles[] = {
 	"DNA_action_types.h",
 	"DNA_constraint_types.h",
 	"DNA_nla_types.h",
+	"DNA_node_types.h",
+	"DNA_color_types.h",
+	"DNA_brush_types.h",
+	"DNA_customdata_types.h",
+	"DNA_particle_types.h",
+	"DNA_cloth_types.h",
+	"DNA_gpencil_types.h",
 	// if you add files here, please add them at the end
 	// of makesdna.c (this file) as well
+	"DNA_windowmanager_types.h",
+	"DNA_anim_types.h",
+	"DNA_boid_types.h",
+	"DNA_smoke_types.h",
+	"DNA_speaker_types.h",
+	"DNA_movieclip_types.h",
+	"DNA_tracking_types.h",
+	"DNA_dynamicpaint_types.h",
 
 	// empty string to indicate end of includefiles
 	""
 };
 
-int maxdata= 500000, maxnr= 50000;
-int nr_names=0;
-int nr_types=0;
-int nr_structs=0;
-char **names, *namedata;		/* at adress names[a] is string a */
-char **types, *typedata;		/* at adress types[a] is string a */
-short *typelens;				/* at typelens[a] is de length of type a */
-short *alphalens;			    /* contains sizes as they are calculated on the alpha */
-short **structs, *structdata;	/* at sp= structs[a] is the first adress of a struct definition
-								   sp[0] is type number
-								   sp[1] is amount of elements
-								   sp[2] sp[3] is typenr,  namenr (etc) */
-/*
+static int maxdata= 500000, maxnr= 50000;
+static int nr_names=0;
+static int nr_types=0;
+static int nr_structs=0;
+static char **names, *namedata;		/* at address names[a] is string a */
+static char **types, *typedata;		/* at address types[a] is string a */
+static short *typelens;				/* at typelens[a] is de length of type a */
+static short *alphalens;			/* contains sizes as they are calculated on the DEC Alpha (64 bits), infact any 64bit system */
+static short **structs, *structdata;/* at sp= structs[a] is the first address of a struct definition
+                                       sp[0] is type number
+                                       sp[1] is amount of elements
+                                       sp[2] sp[3] is typenr,  namenr (etc) */
+/**
+ * Variable to control debug output of makesdna.
  * debugSDNA:
  *  - 0 = no output, except errors
  *  - 1 = detail actions
  *  - 2 = full trace, tell which names and types were found
  *  - 4 = full trace, plus all gritty details
  */
-int debugSDNA = 0;
-int additional_slen_offset;
+static int debugSDNA = 0;
+static int additional_slen_offset;
 
 /* ************************************************************************** */
 /* Functions                                                                  */
 /* ************************************************************************** */
 
 /**
- * Add type <str> to struct indexed by <len>, if it was not yet found.
+ * Add type \c str to struct indexed by \c len, if it was not yet found.
+ * \param str char
+ * \param len int
  */
-int add_type(char *str, int len);
+static int add_type(const char *str, int len);
 
 /**
- * Add variable <str> to 
+ * Add variable \c str to 
+ * \param str
  */
-int add_name(char *str);
+static int add_name(const char *str);
 
 /**
  * Search whether this structure type was already found, and if not,
  * add it.
  */
-short *add_struct(int namecode);
+static short *add_struct(int namecode);
 
 /**
  * Remove comments from this buffer. Assumes that the buffer refers to
  * ascii-code text.
  */
-int preprocess_include(char *maindata, int len);
+static int preprocess_include(char *maindata, int len);
 
 /**
  * Scan this file for serializable types.
  */ 
-int convert_include(char *filename);
+static int convert_include(char *filename);
 
 /**
  * Determine how many bytes are needed for an array.
  */ 
-int arraysize(char *astr, int len);
+static int arraysize(char *astr, int len);
 
 /**
  * Determine how many bytes are needed for each struct.
  */ 
-int calculate_structlens(void);
+static int calculate_structlens(int);
 
 /**
  * Construct the DNA.c file
@@ -196,20 +212,10 @@ int calculate_structlens(void);
 void dna_write(FILE *file, void *pntr, int size);
 
 /**
- * Report all structures found so far, and print their lenghts.
+ * Report all structures found so far, and print their lengths.
  */
 void printStructLenghts(void);
 
-/**
- *
- */ 
-int make_structDNA(FILE *file);
-
-/**
- *
- */ 
-int main(int argc, char ** argv); 
-
 
 
 /* ************************************************************************** */
@@ -218,7 +224,7 @@ int main(int argc, char ** argv);
 
 /* ************************* MAKEN DNA ********************** */
 
-int add_type(char *str, int len)
+static int add_type(const char *str, int len)
 {
 	int nr;
 	char *cp;
@@ -248,7 +254,7 @@ int add_type(char *str, int len)
 	
 	if(nr_types>=maxnr) {
 		printf("too many types\n");
-		return nr_types-1;;
+		return nr_types-1;
 	}
 	nr_types++;
 	
@@ -264,22 +270,27 @@ int add_type(char *str, int len)
  * cases, unfortunately. These are explicitly checked.
  *
  * */
-int add_name(char *str)
+static int add_name(const char *str)
 {
 	int nr, i, j, k;
 	char *cp;
 	char buf[255]; /* stupid limit, change it :) */
-	char *name;
+	const char *name;
 
 	additional_slen_offset = 0;
 	
-	if((str[0]==0) /*  || (str[1]==0) */) return -1;
+	if(str[0]==0 /*  || (str[1]==0) */) return -1;
 
 	if (str[0] == '(' && str[1] == '*') {
-		if (debugSDNA > 3) printf("\t\t\t\t*** Function pointer found\n");
+		/* we handle function pointer and special array cases here, e.g.
+		   void (*function)(...) and float (*array)[..]. the array case
+		   name is still converted to (array*)() though because it is that
+		   way in old dna too, and works correct with elementsize() */
+		int isfuncptr = (strchr(str+1, '(')) != NULL;
+
+		if (debugSDNA > 3) printf("\t\t\t\t*** Function pointer or multidim array pointer found\n");
 		/* functionpointer: transform the type (sometimes) */
 		i = 0;
-		j = 0;
 
 		while (str[i] != ')') {
 			buf[i] = str[i];
@@ -298,10 +309,19 @@ int add_name(char *str)
 			if (debugSDNA > 3) printf("seen %c ( %d) \n", str[j], str[j]);
 			j++;
 		}
-		if (debugSDNA > 3) printf("seen %c ( %d) \n", str[j], str[j]); 
-		if (debugSDNA > 3) printf("special after offset %d\n", j); 
+		if (debugSDNA > 3) printf("seen %c ( %d) \n"
+		                          "special after offset%d\n",
+		                          str[j], str[j], j);
 				
-		if (str[j] == 0 ) {
+		if (!isfuncptr) {
+			/* multidimensional array pointer case */
+			if(str[j] == 0) {
+				if (debugSDNA > 3) printf("offsetting for multidim array pointer\n");
+			}
+			else
+				printf("Error during tokening multidim array pointer\n");
+		}
+		else if (str[j] == 0 ) {
 			if (debugSDNA > 3) printf("offsetting for space\n"); 
 			/* get additional offset */
 			k = 0;
@@ -320,8 +340,8 @@ int add_name(char *str)
 				
 		/*
 		 * Put )(void) at the end? Maybe )(). Should check this with
-         * old sdna. Actually, sometimes )(), sometimes )(void...)
-         * Alas.. such is the nature of braindamage :(
+		 * old sdna. Actually, sometimes )(), sometimes )(void...)
+		 * Alas.. such is the nature of braindamage :(
 		 *
 		 * Sorted it out: always do )(), except for headdraw and
 		 * windraw, part of ScrArea. This is important, because some
@@ -380,7 +400,7 @@ int add_name(char *str)
 	return nr_names-1;
 }
 
-short *add_struct(int namecode)
+static short *add_struct(int namecode)
 {
 	int len;
 	short *sp;
@@ -406,12 +426,16 @@ short *add_struct(int namecode)
 	return sp;
 }
 
-int preprocess_include(char *maindata, int len)
+static int preprocess_include(char *maindata, int len)
 {
 	int a, newlen, comment = 0;
 	char *cp, *temp, *md;
 	
-	temp= MEM_mallocN(len, "preprocess_include");
+	/* note: len + 1, last character is a dummy to prevent
+	 * comparisons using uninitialized memory */
+	temp= MEM_mallocN(len + 1, "preprocess_include");
+	temp[len]= ' ';
+
 	memcpy(temp, maindata, len);
 	
 	// remove all c++ comments
@@ -451,6 +475,13 @@ int preprocess_include(char *maindata, int len)
 		if(comment);
 		else if( cp[0]==' ' && cp[1]==' ' );
 		else if( cp[-1]=='*' && cp[0]==' ' );	/* pointers with a space */
+
+		/* skip special keywords */
+		else if (strncmp("DNA_DEPRECATED", cp, 14)==0) {
+			/* single values are skipped already, so decrement 1 less */
+			a -= 13;
+			cp += 13;
+		}
 		else {
 			md[0]= cp[0];
 			md++;
@@ -463,7 +494,7 @@ int preprocess_include(char *maindata, int len)
 	return newlen;
 }
 
-void *read_file_data(char *filename, int *len_r)
+static void *read_file_data(char *filename, int *len_r)
 {
 #ifdef WIN32
 	FILE *fp= fopen(filename, "rb");
@@ -484,19 +515,22 @@ void *read_file_data(char *filename, int *len_r)
 	data= MEM_mallocN(*len_r, "read_file_data");
 	if (!data) {
 		*len_r= -1;
+		fclose(fp);
 		return NULL;
 	}
 
 	if (fread(data, *len_r, 1, fp)!=1) {
 		*len_r= -1;
 		MEM_freeN(data);
+		fclose(fp);
 		return NULL;
 	}
-
+	
+	fclose(fp);
 	return data;
 }
 
-int convert_include(char *filename)
+static int convert_include(char *filename)
 {
 	/* read include file, skip structs with a '#' before it.
 	   store all data in temporal arrays.
@@ -519,7 +553,7 @@ int convert_include(char *filename)
 	overslaan= 0;
 	while(count<filelen) {
 		
-		/* code for skipping a struct: two hashes. (preprocess added a space) */
+		/* code for skipping a struct: two hashes on 2 lines. (preprocess added a space) */
 		if(md[0]=='#' && md[1]==' ' && md[2]=='#') {
 			overslaan= 1;
 		}
@@ -548,7 +582,7 @@ int convert_include(char *filename)
 					/* first lets make it all nice strings */
 					md1= md+1;
 					while(*md1 != '}') {
-						if( ((long)md1) > ((long)mainend) ) break;
+						if(md1>mainend) break;
 						
 						if(*md1==',' || *md1==' ') *md1= 0;
 						md1++;
@@ -557,12 +591,13 @@ int convert_include(char *filename)
 					/* read types and names until first character that is not '}' */
 					md1= md+1;
 					while( *md1 != '}' ) {
-						if( ((long)md1) > ((long)mainend) ) break;
+						if(md1>mainend) break;
 						
-						/* skip when it says 'struct' or 'unsigned' */
+						/* skip when it says 'struct' or 'unsigned' or 'const' */
 						if(*md1) {
 							if( strncmp(md1, "struct", 6)==0 ) md1+= 7;
-							if( strncmp(md1, "unsigned", 6)==0 ) md1+= 9;
+							if( strncmp(md1, "unsigned", 8)==0 ) md1+= 9;
+							if( strncmp(md1, "const", 5)==0 ) md1+= 6;
 							
 							/* we've got a type! */
 							type= add_type(md1, 0);
@@ -574,13 +609,13 @@ int convert_include(char *filename)
 							
 							/* read until ';' */
 							while( *md1 != ';' ) {
-								if( ((long)md1) > ((long)mainend) ) break;
+								if(md1>mainend) break;
 								
 								if(*md1) {
 									/* We've got a name. slen needs
 									 * correction for function
 									 * pointers! */
-									slen= strlen(md1);
+									slen= (int) strlen(md1);
 									if( md1[slen-1]==';' ) {
 										md1[slen-1]= 0;
 
@@ -590,7 +625,7 @@ int convert_include(char *filename)
 										sp[0]= type;
 										sp[1]= name;
 
-										if ((debugSDNA>1) && (names[name] != 0 )) printf("%s |", names[name]);
+										if ((debugSDNA>1) && (names[name] != NULL)) printf("%s |", names[name]);
 
 										structpoin[1]++;
 										sp+= 2;
@@ -605,7 +640,7 @@ int convert_include(char *filename)
 
 									sp[0]= type;
 									sp[1]= name;
-									if ((debugSDNA > 1) && (names[name] != 0 )) printf("%s ||", names[name]);
+									if ((debugSDNA > 1) && (names[name] != NULL)) printf("%s ||", names[name]);
 
 									structpoin[1]++;
 									sp+= 2;
@@ -632,10 +667,10 @@ int convert_include(char *filename)
 	return 0;
 }
 
-int arraysize(char *astr, int len)
+static int arraysize(char *astr, int len)
 {
 	int a, mul=1;
-	char str[100], *cp=0;
+	char str[100], *cp=NULL;
 
 	memcpy(str, astr, len+1);
 	
@@ -645,6 +680,8 @@ int arraysize(char *astr, int len)
 		}
 		else if( str[a]==']' && cp) {
 			str[a]= 0;
+			/* if 'cp' is a preprocessor definition, it will evaluate to 0,
+			 * the caller needs to check for this case and throw an error */
 			mul*= atoi(cp);
 		}
 	}
@@ -652,7 +689,7 @@ int arraysize(char *astr, int len)
 	return mul;
 }
 
-int calculate_structlens(void)
+static int calculate_structlens(int firststruct)
 {
 	int a, b, len, alphalen, unknown= nr_structs, lastunknown, structtype, type, mul, namelen;
 	short *sp, *structpoin;
@@ -680,47 +717,71 @@ int calculate_structlens(void)
 				for(b=0; b<structpoin[1]; b++, sp+=2) {
 					type= sp[0];
 					cp= names[sp[1]];
-					
-					namelen= strlen(cp);
+
+					namelen= (int) strlen(cp);
 					/* is it a pointer or function pointer? */
 					if(cp[0]=='*' || cp[1]=='*') {
 						has_pointer = 1;
 						/* has the name an extra length? (array) */
 						mul= 1;
 						if( cp[namelen-1]==']') mul= arraysize(cp, namelen);
-						
+
+						if (mul == 0) {
+							printf("Zero array size found or could not parse %s: '%.*s'\n", types[structtype], namelen + 1, cp);
+							dna_error = 1;
+						}
+
 						/* 4-8 aligned/ */
 						if(sizeof(void *) == 4) {
 							if (len % 4) {
-								printf("Align pointer error in struct: %s %s\n", types[structtype], cp);
+								printf("Align pointer error in struct (len4): %s %s\n", types[structtype], cp);
 								dna_error = 1;
 							}
 						} else {
 							if (len % 8) {
-								printf("Align pointer error in struct: %s %s\n", types[structtype], cp);
+								printf("Align pointer error in struct (len8): %s %s\n", types[structtype], cp);
 								dna_error = 1;
 							}
 						}
 
 						if (alphalen % 8) {
-							printf("Align pointer error in struct: %s %s\n", types[structtype],cp);
+							printf("Align pointer error in struct (alphalen8): %s %s\n", types[structtype],cp);
 							dna_error = 1;
 						}
 
 						len += sizeof(void *) * mul;
 						alphalen += 8 * mul;
 
+					} else if(cp[0]=='[') {
+						/* parsing can cause names "var" and "[3]" to be found for "float var [3]" ... */
+						printf("Parse error in struct, invalid member name: %s %s\n", types[structtype], cp);
+						dna_error = 1;
 					} else if( typelens[type] ) {
 						/* has the name an extra length? (array) */
 						mul= 1;
 						if( cp[namelen-1]==']') mul= arraysize(cp, namelen);
+
+						if (mul == 0) {
+							printf("Zero array size found or could not parse %s: '%.*s'\n", types[structtype], namelen + 1, cp);
+							dna_error = 1;
+						}
+
+						/* struct alignment */
+						if(type >= firststruct) {
+							if(sizeof(void *)==8 && (len % 8) ) {
+								printf("Align struct error: %s %s\n", types[structtype],cp);
+								dna_error = 1;
+							}
+						}
 						
 						/* 2-4 aligned/ */
 						if(typelens[type]>3 && (len % 4) ) {
-							printf("Align 4 error in struct: %s %s\n", types[structtype], cp);
+							printf("Align 4 error in struct: %s %s (add %d padding bytes)\n", types[structtype], cp, len%4);
+							dna_error = 1;
 						}
 						else if(typelens[type]==2 && (len % 2) ) {
-							printf("Align 2 error in struct: %s %s\n", types[structtype], cp);
+							printf("Align 2 error in struct: %s %s (add %d padding bytes)\n", types[structtype], cp, len%2);
+							dna_error = 1;
 						}
 
 						len += mul*typelens[type];
@@ -742,10 +803,16 @@ int calculate_structlens(void)
 					// has_pointer is set or alphalen != len
 					if (has_pointer || alphalen != len) {
 						if (alphalen % 8) {
-							printf("Sizeerror in struct: %s\n", types[structtype]);
+							printf("Sizeerror 8 in struct: %s (add %d bytes)\n", types[structtype], alphalen%8);
 							dna_error = 1;
 						}
 					}
+					
+					if(len % 4) {
+						printf("Sizeerror 4 in struct: %s (add %d bytes)\n", types[structtype], len%4);
+						dna_error = 1;
+					}
+					
 				}
 			}
 		}
@@ -810,12 +877,13 @@ void dna_write(FILE *file, void *pntr, int size)
 
 void printStructLenghts(void)
 {
-	int a, unknown= nr_structs, lastunknown, structtype;
+	int a, unknown= nr_structs, structtype;
+	/*int lastunknown;*/ /*UNUSED*/
 	short *structpoin;
 	printf("\n\n*** All detected structs:\n");
 
 	while(unknown) {
-		lastunknown= unknown;
+		/*lastunknown= unknown;*/ /*UNUSED*/
 		unknown= 0;
 		
 		/* check all structs... */
@@ -831,7 +899,7 @@ void printStructLenghts(void)
 }
 
 
-int make_structDNA(FILE *file)
+static int make_structDNA(char *baseDirectory, FILE *file)
 {
 	int len, i;
 	short *sp;
@@ -865,7 +933,7 @@ int make_structDNA(FILE *file)
 	add_type("short", 2);	/* 2 */
 	add_type("ushort", 2);	/* 3 */
 	add_type("int", 4);		/* 4 */
-	add_type("long", 4);	/* 5 */
+	add_type("long", 4);	/* 5 */		/* should it be 8 on 64 bits? */
 	add_type("ulong", 4);	/* 6 */
 	add_type("float", 4);	/* 7 */
 	add_type("double", 8);	/* 8 */
@@ -874,31 +942,21 @@ int make_structDNA(FILE *file)
 	// the defines above shouldn't be output in the padding file...
 	firststruct = nr_types;
 	
-#ifndef BASE_HEADER
-#define BASE_HEADER "../"
-#endif
-
 	/* add all include files defined in the global array                     */
 	/* Since the internal file+path name buffer has limited length, I do a   */
 	/* little test first...                                                  */
 	/* Mind the breaking condition here!                                     */
 	if (debugSDNA) printf("\tStart of header scan:\n"); 
 	for (i = 0; strlen(includefiles[i]); i++) {
-  		if (debugSDNA) printf("\t|-- Converting %s%s\n", BASE_HEADER, includefiles[i]); 
-		if (strlen(includefiles[i]) > SDNA_MAX_FILENAME_LENGTH) {
-			/* this would cause coredumps*/
-			printf("*** \tError in makesdna: the specified filenames is too long "
-				   "for parsing.\n\tFile: %s%s\n", BASE_HEADER, includefiles[i]);			
-		} else {
-			sprintf(str, "%s%s", BASE_HEADER, includefiles[i]);
-			if (convert_include(str)) {
-				return (1);
-			}
+		sprintf(str, "%s%s", baseDirectory, includefiles[i]);
+		  if (debugSDNA) printf("\t|-- Converting %s\n", str); 
+		if (convert_include(str)) {
+			return (1);
 		}
 	}
 	if (debugSDNA) printf("\tFinished scanning %d headers.\n", i); 
 
-	if (calculate_structlens()) {
+	if (calculate_structlens(firststruct)) {
 		// error
 		return(1);
 	}
@@ -952,7 +1010,7 @@ int make_structDNA(FILE *file)
 		/* calculate size of datablock with strings */
 		cp= names[nr_names-1];
 		cp+= strlen(names[nr_names-1]) + 1;			/* +1: null-terminator */
-		len= (long)cp - (long)(names[0]);
+		len= (intptr_t) (cp - (char*) names[0]);
 		len= (len+3) & ~3;
 		dna_write(file, names[0], len);
 		
@@ -965,7 +1023,7 @@ int make_structDNA(FILE *file)
 		/* calculate datablock size */
 		cp= types[nr_types-1];
 		cp+= strlen(types[nr_types-1]) + 1;		/* +1: null-terminator */
-		len= (long)cp - (long)(types[0]);
+		len= (intptr_t) (cp - (char*) types[0]);
 		len= (len+3) & ~3;
 		
 		dna_write(file, types[0], len);
@@ -987,13 +1045,12 @@ int make_structDNA(FILE *file)
 		/* calc datablock size */
 		sp= structs[nr_structs-1];
 		sp+= 2+ 2*( sp[1] );
-		len= (long)sp - (long)(structs[0]);
+		len= (intptr_t) ((char*) sp - (char*) structs[0]);
 		len= (len+3) & ~3;
 		
 		dna_write(file, structs[0], len);
 	
 		/* a simple dna padding test */
-
 		if (0) {
 			FILE *fp;
 			int a;
@@ -1004,16 +1061,18 @@ int make_structDNA(FILE *file)
 
 				// add all include files defined in the global array
 				for (i = 0; strlen(includefiles[i]); i++) {
-					fprintf(fp, "#include \"%s\"\n", includefiles[i]);
+					fprintf(fp, "#include \"%s%s\"\n", baseDirectory, includefiles[i]);
 				}
 
 				fprintf(fp, "main(){\n");
 				sp = typelens;
 				sp += firststruct;
 				for(a=firststruct; a<nr_types; a++, sp++) { 
-					fprintf(fp, "\tprintf(\" ");
-					fprintf(fp, "%%d %s %d ", types[a], *sp);
-					fprintf(fp, "\\n\",  sizeof(struct %s) - %d);\n", types[a], *sp);
+					if(*sp) {
+						fprintf(fp, "\tif(sizeof(struct %s) - %d) printf(\"ALIGN ERROR:", types[a], *sp);
+						fprintf(fp, "%%d %s %d ", types[a], *sp);
+						fprintf(fp, "\\n\",  sizeof(struct %s) - %d);\n", types[a], *sp);
+					}
 				}
 				fprintf(fp, "}\n");
 				fclose(fp);
@@ -1029,6 +1088,7 @@ int make_structDNA(FILE *file)
 	MEM_freeN(names);
 	MEM_freeN(types);
 	MEM_freeN(typelens);
+	MEM_freeN(alphalens);
 	MEM_freeN(structs);
 
 	if (debugSDNA > -1) printf("done.\n");
@@ -1038,20 +1098,24 @@ int make_structDNA(FILE *file)
 
 /* ************************* END MAKE DNA ********************** */
 
-void make_bad_file(char *file)
+static void make_bad_file(const char *file, int line)
 {
 	FILE *fp= fopen(file, "w");
-	fprintf(fp, "ERROR! Cannot make correct DNA.c file, STUPID!\n");
+	fprintf(fp, "#error \"Error! can't make correct DNA.c file from %s:%d, STUPID!\"\n", __FILE__, line);
 	fclose(fp);
 }
 
+#ifndef BASE_HEADER
+#define BASE_HEADER "../"
+#endif
+
 int main(int argc, char ** argv)
 {
 	FILE *file;
 	int return_status = 0;
 
-	if (argc != 2) {
-		printf("Usage: %s outfile.c\n", argv[0]);
+	if (argc!=2 && argc!=3) {
+		printf("Usage: %s outfile.c [base directory]\n", argv[0]);
 		return_status = 1;
 	} else {
 		file = fopen(argv[1], "w");
@@ -1059,11 +1123,19 @@ int main(int argc, char ** argv)
 			printf ("Unable to open file: %s\n", argv[1]);
 			return_status = 1;
 		} else {
+			char baseDirectory[256];
+
+			if (argc==3) {
+				strcpy(baseDirectory, argv[2]);
+			} else {
+				strcpy(baseDirectory, BASE_HEADER);
+			}
+
 			fprintf (file, "unsigned char DNAstr[]= {\n");
-			if (make_structDNA(file)) {
+			if (make_structDNA(baseDirectory, file)) {
 				// error
 				fclose(file);
-				make_bad_file(argv[1]);
+				make_bad_file(argv[1], __LINE__);
 				return_status = 1;
 			} else {
 				fprintf(file, "};\n");
@@ -1084,23 +1156,24 @@ int main(int argc, char ** argv)
 #include "DNA_ID.h"
 #include "DNA_ipo_types.h"
 #include "DNA_key_types.h"
-#include "DNA_scriptlink_types.h"
 #include "DNA_text_types.h"
 #include "DNA_packedFile_types.h"
 #include "DNA_camera_types.h"
 #include "DNA_image_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_lamp_types.h"
-#include "DNA_wave_types.h"
 #include "DNA_material_types.h"
 #include "DNA_vfont_types.h"
 #include "DNA_meta_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_meshdata_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_lattice_types.h"	
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
+#include "DNA_object_fluidsim.h"
 #include "DNA_world_types.h"
-#include "DNA_radio_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_view2d_types.h"	
@@ -1111,8 +1184,7 @@ int main(int argc, char ** argv)
 #include "DNA_fileglobal_types.h"
 #include "DNA_sequence_types.h"
 #include "DNA_effect_types.h"
-#include "DNA_ika_types.h"
-#include "DNA_oops_types.h"
+#include "DNA_outliner_types.h"
 #include "DNA_property_types.h"
 #include "DNA_sensor_types.h"
 #include "DNA_controller_types.h"
@@ -1123,4 +1195,19 @@ int main(int argc, char ** argv)
 #include "DNA_action_types.h"
 #include "DNA_constraint_types.h"
 #include "DNA_nla_types.h"
+#include "DNA_node_types.h"
+#include "DNA_color_types.h"
+#include "DNA_brush_types.h"
+#include "DNA_customdata_types.h"
+#include "DNA_particle_types.h"
+#include "DNA_cloth_types.h"
+#include "DNA_gpencil_types.h"
+#include "DNA_windowmanager_types.h"
+#include "DNA_anim_types.h"
+#include "DNA_boid_types.h"
+#include "DNA_smoke_types.h"
+#include "DNA_speaker_types.h"
+#include "DNA_movieclip_types.h"
+#include "DNA_tracking_types.h"
+#include "DNA_dynamicpaint_types.h"
 /* end of list */
