@@ -1,18 +1,14 @@
-/**
- * $Id: quicktime_import.c,v 1.8 2004/04/09 14:09:56 phase Exp $
+/*
  *
  * quicktime_import.c
  *
  * Code to use Quicktime to load images/movies as texture.
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,22 +17,30 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *
  * The Original Code is written by Rob Haarsma (phase)
  *
  * Contributor(s): Stefan Gartner (sgefant)
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
+
+/** \file blender/quicktime/apple/quicktime_import.c
+ *  \ingroup quicktime
+ */
+
 #ifdef WITH_QUICKTIME
 
 #if defined(_WIN32) || defined(__APPLE__)
+#ifndef USE_QTKIT
 
+#include "MEM_guardedalloc.h"
 #include "IMB_anim.h"
 #include "BLO_sys_types.h"
 #include "BKE_global.h"
+#include "BLI_dynstr.h"
 
 #ifdef __APPLE__
 #include <QuickTime/Movies.h>
@@ -48,30 +52,63 @@
 #include <QTML.h>
 #include <TextUtils.h>
 #include <QuickTimeComponents.h>
+#include <QTLoadLibraryUtils.h>
 #endif /* _WIN32 */
 
+
 #include "quicktime_import.h"
 #include "quicktime_export.h"
 
-
 #define	RECT_WIDTH(r)	(r.right-r.left)
 #define	RECT_HEIGHT(r)	(r.bottom-r.top)
 
 #define QTIME_DEBUG 0
 
+typedef struct _QuicktimeMovie {
+
+	GWorldPtr	offscreenGWorld;
+	PixMapHandle	offscreenPixMap;
+	Movie		movie;
+	Rect		movieBounds;
+	short		movieRefNum;
+	short		movieResId;
+	int			movWidth, movHeight;
+
+	
+	int			framecount;
+	
+	
+	ImBuf		*ibuf;
+	
+
+	TimeValue	*frameIndex;
+	Media		theMedia;
+	Track		theTrack;
+	long		trackIndex;
+	short		depth;
+	
+	int			have_gw;	//ugly
+} QuicktimeMovie;
+
+
 
 void quicktime_init(void)
 {
+	OSErr nerr;
 #ifdef _WIN32
-	if (InitializeQTML(0) != noErr)
+	QTLoadLibrary("QTCF.dll");
+	nerr = InitializeQTML(0);
+	if (nerr != noErr) {
 		G.have_quicktime = FALSE;
+	}
 	else
 		G.have_quicktime = TRUE;
 #endif /* _WIN32 */
 
 	/* Initialize QuickTime */
 #if defined(_WIN32) || defined (__APPLE__)
-	if (EnterMovies() != noErr)
+	nerr = EnterMovies();
+	if (nerr != noErr)
 		G.have_quicktime = FALSE;
 	else
 #endif /* _WIN32 || __APPLE__ */
@@ -88,7 +125,7 @@ void quicktime_exit(void)
 #if defined(_WIN32) || defined(__APPLE__)
 #ifdef WITH_QUICKTIME
 	if(G.have_quicktime) {
-		free_qtcodecdataExt();
+		free_qtcomponentdata();
 		ExitMovies();
 #ifdef _WIN32
 		TerminateQTML();
@@ -99,7 +136,44 @@ void quicktime_exit(void)
 }
 
 
-int anim_is_quicktime (char *name)
+#ifdef _WIN32
+char *get_valid_qtname(char *name)
+{
+	TCHAR Buffer[MAX_PATH];
+	DWORD dwRet;
+	char *qtname;
+	DynStr *ds= BLI_dynstr_new();
+
+	dwRet = GetCurrentDirectory(MAX_PATH, Buffer);
+
+	if(name[1] != ':') {
+		char drive[2];
+
+		if(name[0] == '/' || name[0] == '\\') {
+			drive[0] = Buffer[0];
+			drive[1] = '\0';
+
+			BLI_dynstr_append(ds, drive);
+			BLI_dynstr_append(ds, ":");
+			BLI_dynstr_append(ds, name);
+		} else {
+			BLI_dynstr_append(ds, Buffer);
+			BLI_dynstr_append(ds, "/");
+			BLI_dynstr_append(ds, name);
+		}
+	} else {
+		BLI_dynstr_append(ds, name);
+	}
+
+	qtname= BLI_dynstr_get_cstring(ds);
+	BLI_dynstr_free(ds);
+
+	return qtname;
+}
+#endif /* _WIN32 */
+
+
+int anim_is_quicktime (const char *name)
 {
 	FSSpec	theFSSpec;
 	char	theFullPath[255];
@@ -111,6 +185,7 @@ int anim_is_quicktime (char *name)
 	FInfo						myFinderInfo;
 	FSRef						myRef;
 #else
+	char *qtname;
 	Str255  dst;
 #endif
 	OSErr						err = noErr;
@@ -122,18 +197,24 @@ int anim_is_quicktime (char *name)
 		BLI_testextensie(name, ".avi") ||	// wouldnt be appropriate ;)
 		BLI_testextensie(name, ".tga") ||
 		BLI_testextensie(name, ".png") ||
+		BLI_testextensie(name, ".bmp") ||
 		BLI_testextensie(name, ".jpg") ||
 		BLI_testextensie(name, ".wav") ||
 		BLI_testextensie(name, ".zip") ||
 		BLI_testextensie(name, ".mp3")) return 0;
 
-	if(QTIME_DEBUG) printf("qt: checking as movie\n");
+	if(QTIME_DEBUG) printf("qt: checking as movie: %s\n", name);
 
-	sprintf(theFullPath, "%s", name);
 #ifdef __APPLE__
+	sprintf(theFullPath, "%s", name);
+
 	err = FSPathMakeRef(theFullPath, &myRef, 0);
 	err = FSGetCatalogInfo(&myRef, kFSCatInfoNone, NULL, NULL, &theFSSpec, NULL);
 #else
+	qtname = get_valid_qtname(name);
+	sprintf(theFullPath, "%s", qtname);
+	MEM_freeN(qtname);
+
 	CopyCStringToPascal(theFullPath, dst);
 	err = FSMakeFSSpec(0, 0L, dst, &theFSSpec);
 #endif
@@ -191,7 +272,7 @@ void free_anim_quicktime (struct anim *anim) {
 }
 
 
-OSErr QT_get_frameIndexes(struct anim *anim)
+static OSErr QT_get_frameIndexes(struct anim *anim)
 {
 	int i;
 	OSErr	anErr = noErr;
@@ -199,6 +280,7 @@ OSErr QT_get_frameIndexes(struct anim *anim)
 	TimeValue nextTime = 0;
 	TimeValue	startPoint;
 	TimeValue	tmpstartPoint;
+	long sampleCount = 0;
 
 	startPoint = -1;
 
@@ -209,12 +291,12 @@ OSErr QT_get_frameIndexes(struct anim *anim)
 
 	anim->qtime->framecount = 0;
 
-	while(tmpstartPoint != -1) {
-		nextTime = 0;
-		GetMovieNextInterestingTime(anim->qtime->movie, nextTimeMediaSample, 1, &media, tmpstartPoint, 0, &nextTime, NULL);
-		tmpstartPoint = nextTime;
-		anim->qtime->framecount ++;
-	}
+	sampleCount = GetMediaSampleCount(anim->qtime->theMedia);
+	anErr = GetMoviesError();
+	if (anErr != noErr) return anErr;
+
+	anim->qtime->framecount = sampleCount;
+
 	anim->qtime->frameIndex = (TimeValue *) MEM_callocN(sizeof(TimeValue) * anim->qtime->framecount, "qtframeindex");
 
 	//rewind
@@ -246,6 +328,9 @@ ImBuf * qtime_fetchibuf (struct anim *anim, int position)
 
 	ImBuf *ibuf = NULL;
 	unsigned int *rect;
+#ifdef __APPLE__
+	unsigned char *from, *to;
+#endif
 #ifdef _WIN32
 	unsigned char *crect;
 #endif
@@ -254,7 +339,7 @@ ImBuf * qtime_fetchibuf (struct anim *anim, int position)
 		return (NULL);
 	}
 
-	ibuf = IMB_allocImBuf (anim->x, anim->y, 32, IB_rect, 0);
+	ibuf = IMB_allocImBuf (anim->x, anim->y, 32, IB_rect);
 	rect = ibuf->rect;
 
 	SetMovieTimeValue(anim->qtime->movie, anim->qtime->frameIndex[position]);
@@ -276,11 +361,16 @@ ImBuf * qtime_fetchibuf (struct anim *anim, int position)
 	changePos = (uint32_t *) rect; //textureIMBuf *THE* data pointerrr
 
 #ifdef __APPLE__
-	// Swap alpha byte to the end, so ARGB become RGBA; note this is
- // big endian-centric.
-	for( index = 0; index < boxsize; index++, changePos++, readPos++ )
-		*( changePos ) = ( ( *readPos & 0xFFFFFF ) << 8 ) |
-			( ( *readPos >> 24 ) & 0xFF );
+	// Swap alpha byte to the end, so ARGB become RGBA;
+	from= (unsigned char *)readPos;
+	to= (unsigned char *)changePos;
+	
+	for( index = 0; index < boxsize; index++, from+=4, to+=4 ) {
+		to[3] = from[0];
+		to[0] = from[1];
+		to[1] = from[2];
+		to[2] = from[3];
+	}
 #endif
 
 #ifdef _WIN32
@@ -291,11 +381,14 @@ ImBuf * qtime_fetchibuf (struct anim *anim, int position)
 		//add alpha to ibuf
 		boxsize = anim->x * anim->y * 4;
 		crect = (unsigned char *) rect;
-		for( index = 0; index < boxsize; index+=4, crect+=4 )
-			 crect[3] = 0xFF;
+		for( index = 0; index < boxsize; index+=4, crect+=4 ) {
+			crect[3] = 0xFF;
+		}
 	}
 #endif
 
+	ibuf->profile = IB_PROFILE_SRGB;
+	
 	IMB_flipy(ibuf);
 	return ibuf;
 }
@@ -303,7 +396,7 @@ ImBuf * qtime_fetchibuf (struct anim *anim, int position)
 
 // following two functions only here to get movie pixeldepth
 
-int GetFirstVideoMedia(struct anim *anim)
+static int GetFirstVideoMedia(struct anim *anim)
 {
 	long    numTracks;
 	OSType  mediaType;
@@ -325,10 +418,10 @@ int GetFirstVideoMedia(struct anim *anim)
 	return 0;      // went through all tracks and no video
 }
 
-short GetFirstVideoTrackPixelDepth(struct anim *anim)
+static short GetFirstVideoTrackPixelDepth(struct anim *anim)
 {
 	SampleDescriptionHandle imageDescH =	(SampleDescriptionHandle)NewHandle(sizeof(Handle));
-	long	trackIndex = 0;
+//	long	trackIndex = 0; /*unused*/
 	
 	if(!GetFirstVideoMedia(anim))
 		return -1;
@@ -349,8 +442,10 @@ int startquicktime (struct anim *anim)
 #ifdef __APPLE__
 	FSRef		myRef;
 #else
+	char		*qtname;
 	Str255		dst;
 #endif
+	short depth = 0;
 
 	anim->qtime = MEM_callocN (sizeof(QuicktimeMovie),"animqt");
 	anim->qtime->have_gw = FALSE;
@@ -361,12 +456,17 @@ int startquicktime (struct anim *anim)
 	}
 
 	if(QTIME_DEBUG) printf("qt: attempting to load as movie %s\n", anim->name);
-	sprintf(theFullPath, "%s", anim->name);
 	
 #ifdef __APPLE__
+	sprintf(theFullPath, "%s", anim->name);
+
 	err = FSPathMakeRef(theFullPath, &myRef, 0);
 	err = FSGetCatalogInfo(&myRef, kFSCatInfoNone, NULL, NULL, &theFSSpec, NULL);
 #else
+	qtname = get_valid_qtname(anim->name);
+	sprintf(theFullPath, "%s", qtname);
+	MEM_freeN(qtname);
+
 	CopyCStringToPascal(theFullPath, dst);
 	FSMakeFSSpec(0, 0L, dst, &theFSSpec);
 #endif
@@ -401,7 +501,7 @@ int startquicktime (struct anim *anim)
 		return -1;
 	}
 
-	anim->qtime->ibuf = IMB_allocImBuf (anim->x, anim->y, 32, IB_rect, 0);
+	anim->qtime->ibuf = IMB_allocImBuf (anim->x, anim->y, 32, IB_rect);
 
 #ifdef _WIN32
 	err = NewGWorldFromPtr(&anim->qtime->offscreenGWorld,
@@ -423,8 +523,12 @@ int startquicktime (struct anim *anim)
 		anim->qtime->have_gw = TRUE;
 
 		SetMovieGWorld(anim->qtime->movie,
-				 anim->qtime->offscreenGWorld,
-				 GetGWorldDevice(anim->qtime->offscreenGWorld));
+		               anim->qtime->offscreenGWorld,
+		               GetGWorldDevice(anim->qtime->offscreenGWorld));
+		SetMoviePlayHints(anim->qtime->movie, hintsHighQuality, hintsHighQuality);
+		
+		// sets Media and Track!
+		depth = GetFirstVideoTrackPixelDepth(anim);
 
 		QT_get_frameIndexes(anim);
 	}
@@ -433,7 +537,7 @@ int startquicktime (struct anim *anim)
 	LockPixels(anim->qtime->offscreenPixMap);
 
 	//fill blender's anim struct
-	anim->qtime->depth = GetFirstVideoTrackPixelDepth(anim);
+	anim->qtime->depth = depth;
 	
 	anim->duration = anim->qtime->framecount;
 	anim->params = 0;
@@ -455,18 +559,22 @@ int imb_is_a_quicktime (char *name)
 	GraphicsImportComponent		theImporter = NULL;
 
 	FSSpec	theFSSpec;
-	Str255  dst;
+#ifdef _WIN32
+	Str255  dst; /*unused*/
+#endif
 	char	theFullPath[255];
 
-	Boolean						isMovieFile = false;
-	AliasHandle					myAlias = NULL;
-	Component					myImporter = NULL;
+//	Boolean						isMovieFile = false; /*unused*/
+//	AliasHandle					myAlias = NULL; /*unused*/
+//	Component					myImporter = NULL; /*unused*/
 #ifdef __APPLE__
-	FInfo						myFinderInfo;
+//	FInfo						myFinderInfo; /*unused*/
 	FSRef						myRef;
 #endif
 	OSErr						err = noErr;
 
+	if(!G.have_quicktime) return 0;
+
 	if(QTIME_DEBUG) printf("qt: checking as image %s\n", name);
 
 	// dont let quicktime image import handle these
@@ -507,12 +615,12 @@ ImBuf  *imb_quicktime_decode(unsigned char *mem, int size, int flags)
 	ImageDescriptionHandle		desc;
 
 	ComponentInstance			dataHandler;
-	PointerDataRef dataref = (PointerDataRef)NewHandle(sizeof(PointerDataRefRecord));
+	PointerDataRef dataref;
 
 	int x, y, depth;
 	int have_gw = FALSE;
 	ImBuf *ibuf = NULL;
-	ImBuf *imbuf = NULL;
+//	ImBuf *imbuf = NULL; /*unused*/
 	GWorldPtr	offGWorld;
 	PixMapHandle		myPixMap = NULL;
 
@@ -527,13 +635,15 @@ ImBuf  *imb_quicktime_decode(unsigned char *mem, int size, int flags)
 
 	ImBuf *wbuf = NULL;
 	unsigned int *rect;
+	unsigned char *from, *to;
 #endif
 
-	if (mem == NULL)
+	if (mem == NULL || !G.have_quicktime)
 		goto bail;
 	
 	if(QTIME_DEBUG) printf("qt: attempt to load mem as image\n");
 
+	dataref= (PointerDataRef)NewHandle(sizeof(PointerDataRefRecord));
 	(**dataref).data = mem;
 	(**dataref).dataLength = size;
 
@@ -572,7 +682,7 @@ ImBuf  *imb_quicktime_decode(unsigned char *mem, int size, int flags)
 	depth = (**desc).depth;
 
 	if (flags & IB_test) {
-		ibuf = IMB_allocImBuf(x, y, depth, 0, 0);
+		ibuf = IMB_allocImBuf(x, y, depth, 0);
 		ibuf->ftype = QUICKTIME;
 		DisposeHandle((Handle)dataref);
 		if (gImporter != NULL)	CloseComponent(gImporter);
@@ -580,8 +690,8 @@ ImBuf  *imb_quicktime_decode(unsigned char *mem, int size, int flags)
 	}
 
 #ifdef __APPLE__
-	ibuf = IMB_allocImBuf (x, y, 32, IB_rect, 0);
-	wbuf = IMB_allocImBuf (x, y, 32, IB_rect, 0);
+	ibuf = IMB_allocImBuf (x, y, 32, IB_rect);
+	wbuf = IMB_allocImBuf (x, y, 32, IB_rect);
 
 	err = NewGWorldFromPtr(&offGWorld,
 						k32ARGBPixelFormat,
@@ -589,7 +699,7 @@ ImBuf  *imb_quicktime_decode(unsigned char *mem, int size, int flags)
 						(unsigned char *)wbuf->rect, x * 4);
 #else
 
-	ibuf = IMB_allocImBuf (x, y, 32, IB_rect, 0);	
+	ibuf = IMB_allocImBuf (x, y, 32, IB_rect);	
 
 	err = NewGWorldFromPtr(&offGWorld,
 							k32RGBAPixelFormat,
@@ -624,9 +734,16 @@ ImBuf  *imb_quicktime_decode(unsigned char *mem, int size, int flags)
 	readPos = (uint32_t *) myPtr;
 	changePos = (uint32_t *) rect;
 
-	for( index = 0; index < boxsize; index++, changePos++, readPos++ )
-		*( changePos ) = ( ( *readPos & 0xFFFFFF ) << 8 ) |
-			( ( *readPos >> 24 ) & 0xFF );
+	// Swap alpha byte to the end, so ARGB become RGBA;
+	from= (unsigned char *)readPos;
+	to= (unsigned char *)changePos;
+	
+	for( index = 0; index < boxsize; index++, from+=4, to+=4 ) {
+		to[3] = from[0];
+		to[0] = from[1];
+		to[1] = from[2];
+		to[2] = from[3];
+	}
 #endif
 
 bail:
@@ -664,7 +781,7 @@ bail:
 
 		if( depth < 32 && (**desc).cType != kGIFCodecType) {
 			for(i = 0; i < box; i++, arect+=4)
-				 arect[3] = 0xFF;
+				arect[3] = 0xFF;
 		}
 #endif
 
@@ -674,6 +791,7 @@ bail:
 	return ibuf;
 }
 
+#endif /* USE_QTKIT */
 #endif /* _WIN32 || __APPLE__ */
 
 #endif /* WITH_QUICKTIME */
@@ -682,25 +800,25 @@ bail:
 #if 0
 
 struct ImageDescription {
-     long         idSize;
-     CodecType    cType;
-     long         resvd1;
-     short        resvd2;
-     short        dataRefIndex;
-     short        version;
-     short        revisionLevel;
-     long         vendor;
-     CodecQ       temporalQuality;
-     CodecQ       spatialQuality;
-     short        width;
-     short        height;
-     Fixed        hRes;
-     Fixed        vRes;
-     long         dataSize;
-     short        frameCount;
-     Str31        name;
-     short        depth;
-     short        clutID;
+	long         idSize;
+	CodecType    cType;
+	long         resvd1;
+	short        resvd2;
+	short        dataRefIndex;
+	short        version;
+	short        revisionLevel;
+	long         vendor;
+	CodecQ       temporalQuality;
+	CodecQ       spatialQuality;
+	short        width;
+	short        height;
+	Fixed        hRes;
+	Fixed        vRes;
+	long         dataSize;
+	short        frameCount;
+	Str31        name;
+	short        depth;
+	short        clutID;
 };
 
 #endif // 0
