@@ -1,14 +1,10 @@
-/**
- * $Id: KX_BlenderCanvas.cpp,v 1.3 2002/11/25 15:29:39 mein Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,19 +22,26 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
-#include "BIF_scrarea.h"
+/** \file gameengine/BlenderRoutines/KX_BlenderCanvas.cpp
+ *  \ingroup blroutines
+ */
+
+
 #include "KX_BlenderCanvas.h"
+#include "DNA_screen_types.h"
+#include <stdio.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
-KX_BlenderCanvas::KX_BlenderCanvas(struct ScrArea *area) :
-m_area(area)
+KX_BlenderCanvas::KX_BlenderCanvas(struct wmWindow *win, RAS_Rect &rect, struct ARegion *ar) :
+m_win(win),
+m_frame_rect(rect)
 {
+	// area boundaries needed for mouse coordinates in Letterbox framing mode
+	m_area_left = ar->winrct.xmin;
+	m_area_top = ar->winrct.ymax;
 }
 
 KX_BlenderCanvas::~KX_BlenderCanvas()
@@ -53,7 +56,7 @@ void KX_BlenderCanvas::Init()
 
 void KX_BlenderCanvas::SwapBuffers()
 {
-	BL_SwapBuffers();
+	BL_SwapBuffers(m_win);
 }
 
 void KX_BlenderCanvas::BeginFrame()
@@ -96,24 +99,65 @@ void KX_BlenderCanvas::ClearBuffer(int type)
 
 int KX_BlenderCanvas::GetWidth(
 ) const {
-	return scrarea_get_win_width(m_area);
+	return m_frame_rect.GetWidth();
 }
 
 int KX_BlenderCanvas::GetHeight(
 ) const {
-	return scrarea_get_win_height(m_area);
+	return m_frame_rect.GetHeight();
 }
 
+int KX_BlenderCanvas::GetMouseX(int x)
+{
+	float left = GetWindowArea().GetLeft();
+	return float(x - (left - m_area_left));
+}
+
+int KX_BlenderCanvas::GetMouseY(int y)
+{
+	float top = GetWindowArea().GetTop();
+	return float(y - (m_area_top - top));
+}
+
+float KX_BlenderCanvas::GetMouseNormalizedX(int x)
+{
+	int can_x = GetMouseX(x);
+	return float(can_x)/this->GetWidth();
+}
+
+float KX_BlenderCanvas::GetMouseNormalizedY(int y)
+{
+	int can_y = GetMouseY(y);
+	return float(can_y)/this->GetHeight();
+}
+
+RAS_Rect &
+KX_BlenderCanvas::
+GetWindowArea(
+){
+	return m_area_rect;
+}	
+
 	void
 KX_BlenderCanvas::
 SetViewPort(
 	int x1, int y1,
 	int x2, int y2
 ){
+	/*	x1 and y1 are the min pixel coordinate (e.g. 0)
+		x2 and y2 are the max pixel coordinate
+		the width,height is calculated including both pixels
+		therefore: max - min + 1
+	*/
 	int vp_width = (x2 - x1) + 1;
 	int vp_height = (y2 - y1) + 1;
-	int minx = scrarea_get_win_x(m_area);
-	int miny = scrarea_get_win_y(m_area);
+	int minx = m_frame_rect.GetLeft();
+	int miny = m_frame_rect.GetBottom();
+
+	m_area_rect.SetLeft(minx + x1);
+	m_area_rect.SetBottom(miny + y1);
+	m_area_rect.SetRight(minx + x2);
+	m_area_rect.SetTop(miny + y2);
 
 	glViewport(minx + x1, miny + y1, vp_width, vp_height);
 	glScissor(minx + x1, miny + y1, vp_width, vp_height);
@@ -122,21 +166,23 @@ SetViewPort(
 
 void KX_BlenderCanvas::SetMouseState(RAS_MouseState mousestate)
 {
+	m_mousestate = mousestate;
+
 	switch (mousestate)
 	{
 	case MOUSE_INVISIBLE:
 		{
-			BL_HideMouse();
+			BL_HideMouse(m_win);
 			break;
 		}
 	case MOUSE_WAIT:
 		{
-			BL_WaitMouse();
+			BL_WaitMouse(m_win);
 			break;
 		}
 	case MOUSE_NORMAL:
 		{
-			BL_NormalMouse();
+			BL_NormalMouse(m_win);
 			break;
 		}
 	default:
@@ -150,16 +196,22 @@ void KX_BlenderCanvas::SetMouseState(RAS_MouseState mousestate)
 //	(0,0) is top left, (width,height) is bottom right
 void KX_BlenderCanvas::SetMousePosition(int x,int y)
 {
-	int winX = scrarea_get_win_x(m_area);
-	int winY = scrarea_get_win_y(m_area);
-	int winH = scrarea_get_win_height(m_area);
+	int winX = m_frame_rect.GetLeft();
+	int winY = m_frame_rect.GetBottom();
+	int winH = m_frame_rect.GetHeight();
 	
-	BL_warp_pointer(winX + x, winY + (winH-y-1));
+	BL_warp_pointer(m_win, winX + x, winY + (winH-y));
 }
 
 
 
 void KX_BlenderCanvas::MakeScreenShot(const char* filename)
 {
-	BL_MakeScreenShot(m_area, filename);
+	ScrArea area_dummy= {0};
+	area_dummy.totrct.xmin = m_frame_rect.GetLeft();
+	area_dummy.totrct.xmax = m_frame_rect.GetRight();
+	area_dummy.totrct.ymin = m_frame_rect.GetBottom();
+	area_dummy.totrct.ymax = m_frame_rect.GetTop();
+
+	BL_MakeScreenShot(&area_dummy, filename);
 }
