@@ -1,14 +1,10 @@
-/**
- * $Id: KX_BlenderKeyboardDevice.cpp,v 1.4 2004/03/22 22:01:23 jesterking Exp $
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -26,14 +22,15 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+/** \file gameengine/BlenderRoutines/KX_BlenderKeyboardDevice.cpp
+ *  \ingroup blroutines
+ */
 
-#ifdef WIN32
+
+#if defined(WIN32) && !defined(FREE_WINDOWS)
 // annoying warnings about truncated STL debug info
 #pragma warning (disable :4786)
 #endif 
@@ -41,6 +38,7 @@
 #include "KX_BlenderKeyboardDevice.h"
 
 KX_BlenderKeyboardDevice::KX_BlenderKeyboardDevice()
+	: m_hookesc(false)
 {
 
 }
@@ -76,7 +74,7 @@ void	KX_BlenderKeyboardDevice::NextFrame()
 	
 	// now convert justpressed keyevents into regular (active) keyevents
 	int previousTable = 1-m_currentTable;
-	for (int keyevent= KX_BEGINKEY; keyevent< KX_ENDKEY;keyevent++)
+	for (int keyevent= KX_BEGINKEY; keyevent<= KX_ENDKEY;keyevent++)
 	{
 		SCA_InputEvent& oldevent = m_eventStatusTables[previousTable][keyevent];
 		if (oldevent.m_status == SCA_InputEvent::KX_JUSTACTIVATED ||
@@ -102,13 +100,15 @@ bool	KX_BlenderKeyboardDevice::ConvertBlenderEvent(unsigned short incode,short v
 	KX_EnumInputs kxevent = this->ToNative(incode);
 
 	// only process it, if it's a key
-	if (kxevent >= KX_BEGINKEY && kxevent < KX_ENDKEY)
+	if (kxevent >= KX_BEGINKEY && kxevent <= KX_ENDKEY)
 	{
 		int previousTable = 1-m_currentTable;
 
-		if (val > 0)
+		if (val == KM_PRESS)
 		{
-			if (kxevent == SCA_IInputDevice::KX_ESCKEY && val != 0)
+			if (kxevent == KX_ESCKEY && val != 0 && !m_hookesc)
+				result = true;
+			if (kxevent == KX_PAUSEKEY && val && (IsPressed(KX_LEFTCTRLKEY) || IsPressed(KX_RIGHTCTRLKEY)))
 				result = true;
 
 			// todo: convert val ??
@@ -138,7 +138,7 @@ bool	KX_BlenderKeyboardDevice::ConvertBlenderEvent(unsigned short incode,short v
 				}
 			}
 			
-		} else
+		} else if (val == KM_RELEASE)
 		{
 			// blender eventval == 0
 			switch (m_eventStatusTables[previousTable][kxevent].m_status)
@@ -162,3 +162,8 @@ bool	KX_BlenderKeyboardDevice::ConvertBlenderEvent(unsigned short incode,short v
 	}
 	return result;
 }
+
+void KX_BlenderKeyboardDevice::HookEscape()
+{
+	m_hookesc = true;
+}
