@@ -1,15 +1,10 @@
-/**
- * $Id: KX_BlenderKeyboardDevice.h,v 1.6 2004/03/22 22:01:23 jesterking Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,15 +22,25 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
+
+/** \file KX_BlenderKeyboardDevice.h
+ *  \ingroup blroutines
+ */
+
 #ifndef __KX_BLENDERKEYBOARDDEVICE
 #define __KX_BLENDERKEYBOARDDEVICE
 
 #include "KX_BlenderInputDevice.h"
 
+#ifdef WITH_CXX_GUARDEDALLOC
+#include "MEM_guardedalloc.h"
+#endif
+
 class KX_BlenderKeyboardDevice : public BL_BlenderInputDevice
 {
+	bool	m_hookesc;
 public:
 	KX_BlenderKeyboardDevice();
 	virtual ~KX_BlenderKeyboardDevice();
@@ -44,6 +49,14 @@ public:
 //	virtual const SCA_InputEvent&	GetEventValue(SCA_IInputDevice::KX_EnumInputs inputcode);
 	virtual bool	ConvertBlenderEvent(unsigned short incode,short val);
 	virtual void	NextFrame();
+	virtual void	HookEscape();
+
+
+#ifdef WITH_CXX_GUARDEDALLOC
+public:
+	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:KX_BlenderKeyboardDevice"); }
+	void operator delete(void *mem) { MEM_freeN(mem); }
+#endif
 };
 
 #endif //__KX_BLENDERKEYBOARDDEVICE
