@@ -1,15 +1,10 @@
-/**
-* $Id: KX_ConvertActuators.cpp,v 1.6 2004/10/16 11:41:47 kester Exp $
-*
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,44 +22,55 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
-* Convert Blender actuators for use in the GameEngine
-*/
+ * ***** END GPL LICENSE BLOCK *****
+ * Convert Blender actuators for use in the GameEngine
+ */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+/** \file gameengine/Converter/KX_ConvertActuators.cpp
+ *  \ingroup bgeconv
+ */
 
-#ifdef WIN32
+
+#if defined(WIN32) && !defined(FREE_WINDOWS)
 #pragma warning (disable : 4786) 
 #endif //WIN32
 
-#define BLENDER_HACK_DTIME 0.02
+#include <math.h>
+
+#include "MEM_guardedalloc.h"
 
 #include "KX_BlenderSceneConverter.h"
 #include "KX_ConvertActuators.h"
 
+#ifdef WITH_AUDASPACE
+#  include "AUD_C-API.h"
+#  include "AUD_ChannelMapperFactory.h"
+#endif
+
 // Actuators
 //SCA logiclibrary native logicbricks
 #include "SCA_PropertyActuator.h"
 #include "SCA_LogicManager.h"
 #include "SCA_RandomActuator.h"
-
+#include "SCA_2DFilterActuator.h"
 
 // Ketsji specific logicbricks
 #include "KX_SceneActuator.h"
 #include "KX_IpoActuator.h"
 #include "KX_SoundActuator.h"
-#include "KX_CDActuator.h"
 #include "KX_ObjectActuator.h"
 #include "KX_TrackToActuator.h"
 #include "KX_ConstraintActuator.h"
 #include "KX_CameraActuator.h"
 #include "KX_GameActuator.h"
+#include "KX_StateActuator.h"
 #include "KX_VisibilityActuator.h"
 #include "KX_SCA_AddObjectActuator.h"
 #include "KX_SCA_EndObjectActuator.h"
 #include "KX_SCA_ReplaceMeshActuator.h"
+#include "KX_ParentActuator.h"
+#include "KX_SCA_DynamicActuator.h"
+#include "KX_SteeringActuator.h"
 
 #include "KX_Scene.h"
 #include "KX_KetsjiEngine.h"
@@ -73,8 +79,11 @@
 #include "KX_GameObject.h"
 
 /* This little block needed for linking to Blender... */
-
+#include "BKE_text.h"
 #include "BLI_blenlib.h"
+#include "BLI_math_base.h"
+
+#define FILE_MAX 240 // repeated here to avoid dependency from BKE_utildefines.h
 
 #include "KX_NetworkMessageActuator.h"
 
@@ -88,6 +97,10 @@
 #include "DNA_actuator_types.h"
 #include "DNA_packedFile_types.h"
 #include "BL_ActionActuator.h"
+#include "BL_ShapeActionActuator.h"
+#include "BL_ArmatureActuator.h"
+#include "RNA_access.h"
+#include "BL_Action.h"
 /* end of blender include block */
 
 #include "BL_BlenderDataConversion.h"
@@ -98,13 +111,12 @@ KX_BLENDERTRUNC needed to round 'almost' zero values to zero, else velocities et
 
 #define KX_BLENDERTRUNC(x)  (( x < 0.0001 && x > -0.0001 )  ? 0.0 : x)
 
-void BL_ConvertActuators(char* maggiename,
+void BL_ConvertActuators(const char* maggiename,
 						 struct Object* blenderobject,
 						 KX_GameObject* gameobj,
 						 SCA_LogicManager* logicmgr,
 						 KX_Scene* scene,
 						 KX_KetsjiEngine* ketsjiEngine,
-						 int & executePriority, 
 						 int activeLayerBitInfo,
 						 bool isInActiveLayer,
 						 RAS_IRenderTools* rendertools,
@@ -113,11 +125,20 @@ void BL_ConvertActuators(char* maggiename,
 {
 	
 	int uniqueint = 0;
+	int actcount = 0;
+	int executePriority = 0;
 	bActuator* bact = (bActuator*) blenderobject->actuators.first;
+	while (bact)
+	{
+		actcount++;
+		bact = bact->next;
+	}
+	gameobj->ReserveActuator(actcount);
+	bact = (bActuator*) blenderobject->actuators.first;
 	while(bact)
 	{
 		STR_String uniquename = bact->name;
-		STR_String objectname = gameobj->GetName();
+		STR_String& objectname = gameobj->GetName();
 		
 		SCA_IActuator* baseact = NULL;
 		switch (bact->type)
@@ -125,6 +146,7 @@ void BL_ConvertActuators(char* maggiename,
 		case ACT_OBJECT:
 			{
 				bObjectActuator* obact = (bObjectActuator*) bact->data;
+				KX_GameObject* obref = NULL;
 				MT_Vector3 forcevec(KX_BLENDERTRUNC(obact->forceloc[0]),
 					KX_BLENDERTRUNC(obact->forceloc[1]),
 					KX_BLENDERTRUNC(obact->forceloc[2]));
@@ -139,13 +161,7 @@ void BL_ConvertActuators(char* maggiename,
 				MT_Vector3 angvelvec ( KX_BLENDERTRUNC(obact->angularvelocity[0]),
 					KX_BLENDERTRUNC(obact->angularvelocity[1]),
 					KX_BLENDERTRUNC(obact->angularvelocity[2]));
-				
-				drotvec /=		BLENDER_HACK_DTIME;
-				//drotvec /=		BLENDER_HACK_DTIME;
-				drotvec *= MT_2_PI/360.0;
-				//dlocvec /= BLENDER_HACK_DTIME;
-				//linvelvec /=	BLENDER_HACK_DTIME;
-				//angvelvec /=	BLENDER_HACK_DTIME;
+				short damping = obact->damping;
 				
 				/* Blender uses a bit vector internally for the local-flags. In */
 				/* KX, we have four bools. The compiler should be smart enough  */
@@ -159,17 +175,22 @@ void BL_ConvertActuators(char* maggiename,
 				bitLocalFlag.DRot = bool((obact->flag & ACT_DROT_LOCAL)!=0);
 				bitLocalFlag.LinearVelocity = bool((obact->flag & ACT_LIN_VEL_LOCAL)!=0);
 				bitLocalFlag.AngularVelocity = bool((obact->flag & ACT_ANG_VEL_LOCAL)!=0);
-				
+				bitLocalFlag.ServoControl = bool(obact->type == ACT_OBJECT_SERVO);
 				bitLocalFlag.AddOrSetLinV = bool((obact->flag & ACT_ADD_LIN_VEL)!=0);
-				
+				if (obact->reference && bitLocalFlag.ServoControl)
+				{
+					obref = converter->FindGameObject(obact->reference);
+				}
 				
 				KX_ObjectActuator* tmpbaseact = new KX_ObjectActuator(gameobj,
+					obref,
 					forcevec.getValue(),
 					torquevec.getValue(),
 					dlocvec.getValue(),
 					drotvec.getValue(),
 					linvelvec.getValue(),
 					angvelvec.getValue(),
+					damping,
 					bitLocalFlag
 					);
 				baseact = tmpbaseact;
@@ -177,13 +198,49 @@ void BL_ConvertActuators(char* maggiename,
 			}
 		case ACT_ACTION:
 			{
-				if (blenderobject->type==OB_ARMATURE){
+				bActionActuator* actact = (bActionActuator*) bact->data;
+				STR_String propname = (actact->name ? actact->name : "");
+				STR_String propframe = (actact->frameProp ? actact->frameProp : "");
+
+				short ipo_flags = 0;
+
+				// Convert flags
+				if (actact->flag & ACT_IPOFORCE) ipo_flags |= BL_Action::ACT_IPOFLAG_FORCE;
+				if (actact->flag & ACT_IPOLOCAL) ipo_flags |= BL_Action::ACT_IPOFLAG_LOCAL;
+				if (actact->flag & ACT_IPOADD) ipo_flags |= BL_Action::ACT_IPOFLAG_ADD;
+				if (actact->flag & ACT_IPOCHILD) ipo_flags |= BL_Action::ACT_IPOFLAG_CHILD;
+					
+				BL_ActionActuator* tmpbaseact = new BL_ActionActuator(
+					gameobj,
+					propname,
+					propframe,
+					actact->sta,
+					actact->end,
+					actact->act,
+					actact->type, // + 1, because Blender starts to count at zero,
+					actact->blendin,
+					actact->priority,
+					actact->layer,
+					actact->layer_weight,
+					ipo_flags,
+					actact->end_reset,
+					actact->stridelength
+					// Ketsji at 1, because zero is reserved for "NoDef"
+					);
+				baseact= tmpbaseact;
+				break;
+			}
+		case ACT_SHAPEACTION:
+			{
+				if (blenderobject->type==OB_MESH){
 					bActionActuator* actact = (bActionActuator*) bact->data;
 					STR_String propname = (actact->name ? actact->name : "");
+					STR_String propframe = (actact->frameProp ? actact->frameProp : "");
 					
-					BL_ActionActuator* tmpbaseact = new BL_ActionActuator(
+					BL_ShapeActionActuator* tmpbaseact = new BL_ShapeActionActuator(
 						gameobj,
 						propname,
+						propframe,
 						actact->sta,
 						actact->end,
 						actact->act,
@@ -197,28 +254,31 @@ void BL_ConvertActuators(char* maggiename,
 					break;
 				}
 				else
-					printf ("Discarded action actuator from non-armature object [%s]\n", blenderobject->id.name+2);
+					printf ("Discarded shape action actuator from non-mesh object [%s]\n", blenderobject->id.name+2);
 			}
 		case ACT_IPO:
 			{
 				bIpoActuator* ipoact = (bIpoActuator*) bact->data;
 				bool ipochild = (ipoact->flag & ACT_IPOCHILD) !=0;
-				STR_String propname = ( ipoact->name ? ipoact->name : "");
+				STR_String propname = ipoact->name;
+				STR_String frameProp = ipoact->frameProp;
 				// first bit?
 				bool ipo_as_force = (ipoact->flag & ACT_IPOFORCE);
-				bool force_local = (ipoact->flag & ACT_IPOFORCE_LOCAL);
+				bool local = (ipoact->flag & ACT_IPOLOCAL);
+				bool ipo_add = (ipoact->flag & ACT_IPOADD);
 				
 				KX_IpoActuator* tmpbaseact = new KX_IpoActuator(
 					gameobj,
 					propname ,
+					frameProp,
 					ipoact->sta,
 					ipoact->end,
 					ipochild,
 					ipoact->type + 1, // + 1, because Blender starts to count at zero,
 					// Ketsji at 1, because zero is reserved for "NoDef"
 					ipo_as_force,
-					force_local
-					);
+					ipo_add,
+					local);
 				baseact = tmpbaseact;
 				break;
 			}
@@ -240,7 +300,8 @@ void BL_ConvertActuators(char* maggiename,
 						camact->height,
 						camact->min,
 						camact->max,
-						camact->axis=='x');
+						camact->axis=='x',
+						camact->damping);
 					baseact = tmpcamact;
 				}
 				break;
@@ -256,15 +317,16 @@ void BL_ConvertActuators(char* maggiename,
 				STR_String toPropName = (msgAct->toPropName
 					? (char*) msgAct->toPropName
 					: "");
-					/**
-					* Get the Message Subject to send.
+				
+				/**
+				* Get the Message Subject to send.
 				*/
 				STR_String subject = (msgAct->subject
 					? (char*) msgAct->subject
 					: "");
 				
-					/**
-					* Get the bodyType
+				/**
+				* Get the bodyType
 				*/
 				int bodyType = msgAct->bodyType;
 				
@@ -295,8 +357,7 @@ void BL_ConvertActuators(char* maggiename,
 			{
 				bSoundActuator* soundact = (bSoundActuator*) bact->data;
 				/* get type, and possibly a start and end frame */
-				short startFrame = soundact->sta, stopFrame = soundact->end;
-				KX_SoundActuator::KX_SOUNDACT_TYPE 
+				KX_SoundActuator::KX_SOUNDACT_TYPE
 					soundActuatorType = KX_SoundActuator::KX_SOUNDACT_NODEF;
 				
 				switch(soundact->type) {
@@ -326,156 +387,68 @@ void BL_ConvertActuators(char* maggiename,
 				
 				if (soundActuatorType != KX_SoundActuator::KX_SOUNDACT_NODEF) 
 				{
-					SND_SoundObject* sndobj = NULL;
-					
-					if (soundact->sound)
+					bSound* sound = soundact->sound;
+					bool is3d = soundact->flag & ACT_SND_3D_SOUND ? true : false;
+					AUD_Reference<AUD_IFactory> snd_sound;
+					KX_3DSoundSettings settings;
+					settings.cone_inner_angle = soundact->sound3D.cone_inner_angle;
+					settings.cone_outer_angle = soundact->sound3D.cone_outer_angle;
+					settings.cone_outer_gain = soundact->sound3D.cone_outer_gain;
+					settings.max_distance = soundact->sound3D.max_distance;
+					settings.max_gain = soundact->sound3D.max_gain;
+					settings.min_gain = soundact->sound3D.min_gain;
+					settings.reference_distance = soundact->sound3D.reference_distance;
+					settings.rolloff_factor = soundact->sound3D.rolloff_factor;
+
+					if(!sound)
 					{
-						SND_Scene* soundscene = scene->GetSoundScene();
-						STR_String samplename = soundact->sound->name;
-						
-						bool sampleisloaded = false;
-						
-						/* let's see if the sample was already loaded */
-						if (soundscene->IsSampleLoaded(samplename))
+						std::cout <<	"WARNING: Sound actuator \"" << bact->name <<
+										"\" from object \"" <<  blenderobject->id.name+2 <<
+										"\" has no sound datablock." << std::endl;
+					}
+					else
+					{
+						snd_sound = *reinterpret_cast<AUD_Reference<AUD_IFactory>*>(sound->playback_handle);
+
+						// if sound shall be 3D but isn't mono, we have to make it mono!
+						if(is3d)
 						{
-							sampleisloaded = true;
-						}
-						else
-						{
-							/* if not, make it so */
-							PackedFile* pf = soundact->sound->newpackedfile;
-							
-							/* but we need a packed file then */
-							if (pf)
+							try
 							{
-								if (soundscene->LoadSample(samplename, pf->data, pf->size) > -1)
-									sampleisloaded = true;
+								AUD_Reference<AUD_IReader> reader = snd_sound->createReader();
+								if(reader->getSpecs().channels != AUD_CHANNELS_MONO)
+								{
+									AUD_DeviceSpecs specs;
+									specs.channels = AUD_CHANNELS_MONO;
+									specs.rate = AUD_RATE_INVALID;
+									specs.format = AUD_FORMAT_INVALID;
+									snd_sound = new AUD_ChannelMapperFactory(snd_sound, specs);
+								}
 							}
-							/* or else load it from disk */
-							else
+							catch(AUD_Exception&)
 							{
-								/* but we need to convert the samplename into absolute pathname first */
-								BLI_convertstringcode(soundact->sound->name, maggiename, 0);
-								samplename = soundact->sound->name;
-								
-								/* and now we can load it */
-								if (soundscene->LoadSample(samplename, NULL, 0) > -1)
-									sampleisloaded = true;
+								// sound cannot be played... ignore
 							}
 						}
-						
-						if (sampleisloaded)
-						{
-							sndobj = new SND_SoundObject();
-							sndobj->SetSampleName(samplename.Ptr());
-							sndobj->SetObjectName(bact->name);
-							sndobj->SetRollOffFactor(soundact->sound->attenuation);
-							sndobj->SetGain(soundact->sound->volume);
-							sndobj->SetPitch(exp((soundact->sound->pitch / 12.0) * log(2.0)));
-							//							sndobj->SetLoopStart(soundact->sound->loopstart);
-							//							sndobj->SetLoopStart(soundact->sound->loopend);
-							if (soundact->sound->flags & SOUND_FLAGS_LOOP)
-							{
-								if (soundact->sound->flags & SOUND_FLAGS_BIDIRECTIONAL_LOOP)
-									sndobj->SetLoopMode(SND_LOOP_BIDIRECTIONAL);
-								else
-									sndobj->SetLoopMode(SND_LOOP_NORMAL);
-							}
-							else
-								sndobj->SetLoopMode(SND_LOOP_OFF);
-							
-							if (soundact->sound->flags & SOUND_FLAGS_PRIORITY)
-								sndobj->SetHighPriority(true);
-							else
-								sndobj->SetHighPriority(false);
-							
-							if (soundact->sound->flags & SOUND_FLAGS_3D)
-								sndobj->Set3D(true);
-							else
-								sndobj->Set3D(false);
-							
-							KX_SoundActuator* tmpsoundact = 
-								new KX_SoundActuator(gameobj, 
-								sndobj,
-								scene->GetSoundScene(), // needed for replication!
-								soundActuatorType,
-								startFrame,
-								stopFrame);
-							
-							tmpsoundact->SetName(bact->name);
-							baseact = tmpsoundact;
-							soundscene->AddObject(sndobj);
-						} else {
-							std::cout << "WARNING: Sound actuator " << bact->name << " failed to load sample." << std::endl;
-						}
-					}
-				}
-				break;
-			}
-		case ACT_CD:
-			{
-				bCDActuator* cdact = (bCDActuator*) bact->data;
-				/* get type, and possibly a start and end frame */
-				short startFrame = cdact->sta, stopFrame = cdact->end;
-				KX_CDActuator::KX_CDACT_TYPE 
-					cdActuatorType = KX_CDActuator::KX_CDACT_NODEF;
-				
-				switch(cdact->type)
-				{
-				case ACT_CD_PLAY_ALL:
-					cdActuatorType = KX_CDActuator::KX_CDACT_PLAY_ALL;
-					break;
-				case ACT_CD_PLAY_TRACK:
-					cdActuatorType = KX_CDActuator::KX_CDACT_PLAY_TRACK;
-					break;
-				case ACT_CD_LOOP_TRACK:
-					cdActuatorType = KX_CDActuator::KX_CDACT_LOOP_TRACK;
-					break;
-				case ACT_CD_VOLUME:
-					cdActuatorType = KX_CDActuator::KX_CDACT_VOLUME;
-					break;
-				case ACT_CD_STOP:
-					cdActuatorType = KX_CDActuator::KX_CDACT_STOP;
-					break;
-				case ACT_CD_PAUSE:
-					cdActuatorType = KX_CDActuator::KX_CDACT_PAUSE;
-					break;
-				case ACT_CD_RESUME:
-					cdActuatorType = KX_CDActuator::KX_CDACT_RESUME;
-					break;
-					
-				default:
-					/* This is an error!!! */
-					cdActuatorType = KX_CDActuator::KX_CDACT_NODEF;
-				}
-				
-				if (cdActuatorType != KX_CDActuator::KX_CDACT_NODEF) 
-				{
-					SND_Scene* soundscene = scene->GetSoundScene();
-					SND_CDObject* pCD = SND_CDObject::Instance();
-					
-					if (pCD)
-					{
-						pCD->SetGain(cdact->volume);
-						
-						KX_CDActuator* tmpcdact = 
-							new KX_CDActuator(gameobj, 
-							scene->GetSoundScene(), // needed for replication!
-							cdActuatorType,
-							cdact->track,
-							startFrame,
-							stopFrame);
-						
-						tmpcdact->SetName(bact->name);
-						baseact = tmpcdact;
 					}
+					KX_SoundActuator* tmpsoundact =
+						new KX_SoundActuator(gameobj,
+						snd_sound,
+						soundact->volume,
+						(float)(exp((soundact->pitch / 12.0) * M_LN2)),
+						is3d,
+						settings,
+						soundActuatorType);
+
+					tmpsoundact->SetName(bact->name);
+					baseact = tmpsoundact;
 				}
 				break;
 			}
 		case ACT_PROPERTY:
 			{
 				bPropertyActuator* propact = (bPropertyActuator*) bact->data;
-				CValue* destinationObj = NULL;
+				SCA_IObject* destinationObj = NULL;
 				
 				/*
 				here the destinationobject is searched. problem with multiple scenes: other scenes
@@ -511,22 +484,28 @@ void BL_ConvertActuators(char* maggiename,
 						
 						// does the 'original' for replication exists, and 
 						// is it in a non-active layer ?
-						CValue* originalval = NULL;
-						if (editobact->ob && !(editobact->ob->lay & activeLayerBitInfo))
-							originalval = converter->FindGameObject(editobact->ob);
+						SCA_IObject* originalval = NULL;
+						if (editobact->ob)
+						{
+							if (editobact->ob->lay & activeLayerBitInfo)
+							{
+								fprintf(stderr, "Warning, object \"%s\" from AddObject actuator \"%s\" is not in a hidden layer.\n", objectname.Ptr(), uniquename.Ptr());
+							}
+							else {
+								originalval = converter->FindGameObject(editobact->ob);
+							}
+						}
 						
-						MT_Vector3 linvelvec ( KX_BLENDERTRUNC(editobact->linVelocity[0]),
-							KX_BLENDERTRUNC(editobact->linVelocity[1]),
-							KX_BLENDERTRUNC(editobact->linVelocity[2]));
-							
 						KX_SCA_AddObjectActuator* tmpaddact = 
 							new KX_SCA_AddObjectActuator(
 								gameobj, 
 								originalval,
 								editobact->time,
 								scene,
-								linvelvec.getValue(),
-								editobact->localflag!=0
+								editobact->linVelocity,
+								(editobact->localflag & ACT_EDOB_LOCAL_LINV)!=0,
+								editobact->angVelocity,
+								(editobact->localflag & ACT_EDOB_LOCAL_ANGV)!=0
 								);
 								
 								//editobact->ob to gameobj
@@ -547,7 +526,6 @@ void BL_ConvertActuators(char* maggiename,
 							tmpmesh = BL_ConvertMesh(
 								editobact->me,
 								blenderobject,
-								rendertools,
 								scene,
 								converter
 								);
@@ -556,7 +534,10 @@ void BL_ConvertActuators(char* maggiename,
 							= new KX_SCA_ReplaceMeshActuator(
 								gameobj,
 								tmpmesh,
-								scene
+								scene,
+								(editobact->flag & ACT_EDOB_REPLACE_MESH_NOGFX)==0,
+								(editobact->flag & ACT_EDOB_REPLACE_MESH_PHYS)!=0
+								
 								);
 							
 							baseact = tmpreplaceact;
@@ -577,6 +558,16 @@ void BL_ConvertActuators(char* maggiename,
 								blenderobject->upflag
 								);
 							baseact = tmptrackact;
+						break;
+					}
+				case ACT_EDOB_DYNAMICS:
+					{
+						KX_SCA_DynamicActuator* tmpdynact 
+							= new KX_SCA_DynamicActuator(gameobj, 
+								editobact->dyn_operation,
+								editobact->mass
+								);
+							baseact = tmpdynact;
 					}
 				}
 				break;
@@ -584,51 +575,141 @@ void BL_ConvertActuators(char* maggiename,
 		case ACT_CONSTRAINT:
 			{
 				float min = 0.0, max = 0.0;
+				char *prop = NULL;
 				KX_ConstraintActuator::KX_CONSTRAINTTYPE locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_NODEF;
 				bConstraintActuator *conact 
 					= (bConstraintActuator*) bact->data;
 				/* convert settings... degrees in the ui become radians  */ 
 				/* internally                                            */ 
-				switch (conact->flag) {
-				case ACT_CONST_LOCX:
-					locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_LOCX; 
-					min = conact->minloc[0];
-					max = conact->maxloc[0];
-					break;
-				case ACT_CONST_LOCY:
-					locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_LOCY; 
-					min = conact->minloc[1];
-					max = conact->maxloc[1];
-					break;
-				case ACT_CONST_LOCZ:
-					locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_LOCZ;
-					min = conact->minloc[2];
-					max = conact->maxloc[2];
-					break;
-				case ACT_CONST_ROTX:
-					locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ROTX;
-					min = MT_2_PI * conact->minrot[0] / 360.0;
-					max = MT_2_PI * conact->maxrot[0] / 360.0;
-					break;
-				case ACT_CONST_ROTY:
-					locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ROTY;
-					min = MT_2_PI * conact->minrot[1] / 360.0;
-					max = MT_2_PI * conact->maxrot[1] / 360.0;
-					break;
-				case ACT_CONST_ROTZ:
-					locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ROTZ;
-					min = MT_2_PI * conact->minrot[2] / 360.0;
-					max = MT_2_PI * conact->maxrot[2] / 360.0;
-					break;
-				default:
-					; /* error */ 
+				if (conact->type == ACT_CONST_TYPE_ORI) {
+					min = (float)((MT_2_PI * conact->minloc[0])/360.0);
+					max = (float)((MT_2_PI * conact->maxloc[0])/360.0);
+					switch (conact->mode) {
+					case ACT_CONST_DIRPX:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ORIX;
+						break;
+					case ACT_CONST_DIRPY:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ORIY;
+						break;
+					case ACT_CONST_DIRPZ:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ORIZ;
+						break;
+					}
+				} else if (conact->type == ACT_CONST_TYPE_DIST) {
+					switch (conact->mode) {
+					case ACT_CONST_DIRPX:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_DIRPX;
+						min = conact->minloc[0];
+						max = conact->maxloc[0];
+						break;
+					case ACT_CONST_DIRPY:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_DIRPY;
+						min = conact->minloc[1];
+						max = conact->maxloc[1];
+						break;
+					case ACT_CONST_DIRPZ:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_DIRPZ;
+						min = conact->minloc[2];
+						max = conact->maxloc[2];
+						break;
+					case ACT_CONST_DIRNX:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_DIRNX;
+						min = conact->minloc[0];
+						max = conact->maxloc[0];
+						break;
+					case ACT_CONST_DIRNY:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_DIRNY;
+						min = conact->minloc[1];
+						max = conact->maxloc[1];
+						break;
+					case ACT_CONST_DIRNZ:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_DIRNZ;
+						min = conact->minloc[2];
+						max = conact->maxloc[2];
+						break;
+					}
+					prop = conact->matprop;
+				} else if (conact->type == ACT_CONST_TYPE_FH) {
+					switch (conact->mode) {
+					case ACT_CONST_DIRPX:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_FHPX;
+						min = conact->minloc[0];
+						max = conact->maxloc[0];
+						break;
+					case ACT_CONST_DIRPY:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_FHPY;
+						min = conact->minloc[1];
+						max = conact->maxloc[1];
+						break;
+					case ACT_CONST_DIRPZ:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_FHPZ;
+						min = conact->minloc[2];
+						max = conact->maxloc[2];
+						break;
+					case ACT_CONST_DIRNX:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_FHNX;
+						min = conact->minloc[0];
+						max = conact->maxloc[0];
+						break;
+					case ACT_CONST_DIRNY:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_FHNY;
+						min = conact->minloc[1];
+						max = conact->maxloc[1];
+						break;
+					case ACT_CONST_DIRNZ:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_FHNZ;
+						min = conact->minloc[2];
+						max = conact->maxloc[2];
+						break;
+					}
+					prop = conact->matprop;
+				} else {
+					switch (conact->flag) {
+					case ACT_CONST_LOCX:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_LOCX; 
+						min = conact->minloc[0];
+						max = conact->maxloc[0];
+						break;
+					case ACT_CONST_LOCY:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_LOCY; 
+						min = conact->minloc[1];
+						max = conact->maxloc[1];
+						break;
+					case ACT_CONST_LOCZ:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_LOCZ;
+						min = conact->minloc[2];
+						max = conact->maxloc[2];
+						break;
+					case ACT_CONST_ROTX:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ROTX;
+						min = conact->minrot[0] * (float)MT_RADS_PER_DEG;
+						max = conact->maxrot[0] * (float)MT_RADS_PER_DEG;
+						break;
+					case ACT_CONST_ROTY:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ROTY;
+						min = conact->minrot[1] * (float)MT_RADS_PER_DEG;
+						max = conact->maxrot[1] * (float)MT_RADS_PER_DEG;
+						break;
+					case ACT_CONST_ROTZ:
+						locrot = KX_ConstraintActuator::KX_ACT_CONSTRAINT_ROTZ;
+						min = conact->minrot[2] * (float)MT_RADS_PER_DEG;
+						max = conact->maxrot[2] * (float)MT_RADS_PER_DEG;
+						break;
+					default:
+						; /* error */ 
+					}
 				}
 				KX_ConstraintActuator *tmpconact 
 					= new KX_ConstraintActuator(gameobj,
-					conact->damp,
-					min,
-					max,
-					locrot);
+						conact->damp,
+						conact->rotdamp,
+						min,
+						max,
+						conact->maxrot,
+						locrot,
+						conact->time,
+						conact->flag,
+						prop);
 				baseact = tmpconact;
 				break;
 			}
@@ -686,15 +767,11 @@ void BL_ConvertActuators(char* maggiename,
 						break;
 					}
 				case ACT_SCENE_CAMERA:
+					mode = KX_SceneActuator::KX_SCENE_SET_CAMERA;
 					if (sceneact->camera)
 					{
-						mode = KX_SceneActuator::KX_SCENE_SET_CAMERA;
 						cam = (KX_Camera*) converter->FindGameObject(sceneact->camera);
 					}
-					else
-					{
-						// TODO:warn user
-					}
 					break;
 				case ACT_SCENE_RESTART:
 					{
@@ -747,6 +824,16 @@ void BL_ConvertActuators(char* maggiename,
 						mode = KX_GameActuator::KX_GAME_QUIT;
 						break;
 					}
+				case ACT_GAME_SAVECFG:
+					{
+						mode = KX_GameActuator::KX_GAME_SAVECFG;
+						break;
+					}
+				case ACT_GAME_LOADCFG:
+					{
+						mode = KX_GameActuator::KX_GAME_LOADCFG;
+						break;
+					}
 				default:
 					; /* flag error */
 				}
@@ -766,6 +853,11 @@ void BL_ConvertActuators(char* maggiename,
 					= (bRandomActuator *) bact->data;
 				
 				unsigned long seedArg = randAct->seed;
+				if (seedArg == 0)
+				{
+					seedArg = (int)(ketsjiEngine->GetRealTime()*100000.0);
+					seedArg ^= (intptr_t)randAct;
+				}
 				SCA_RandomActuator::KX_RANDOMACT_MODE modeArg 
 					= SCA_RandomActuator::KX_RANDOMACT_NODEF;
 				SCA_RandomActuator *tmprandomact;
@@ -833,15 +925,184 @@ void BL_ConvertActuators(char* maggiename,
 			bVisibilityActuator *vis_act = (bVisibilityActuator *) bact->data;
 			KX_VisibilityActuator * tmp_vis_act = NULL;
 			bool v = ((vis_act->flag & ACT_VISIBILITY_INVISIBLE) != 0);
+			bool o = ((vis_act->flag & ACT_VISIBILITY_OCCLUSION) != 0);
+			bool recursive = ((vis_act->flag & ACT_VISIBILITY_RECURSIVE) != 0);
 
-			tmp_vis_act = 
-				new KX_VisibilityActuator(gameobj,
-							  !v);
+			tmp_vis_act = new KX_VisibilityActuator(gameobj, !v, o, recursive);
 			
 			baseact = tmp_vis_act;
 		}
 		break;
 
+		case ACT_STATE:
+		{
+			bStateActuator *sta_act = (bStateActuator *) bact->data;
+			KX_StateActuator * tmp_sta_act = NULL;
+
+			tmp_sta_act = 
+				new KX_StateActuator(gameobj, sta_act->type, sta_act->mask);
+			
+			baseact = tmp_sta_act;
+		}
+		break;
+
+		case ACT_2DFILTER:
+		{
+			bTwoDFilterActuator *_2dfilter = (bTwoDFilterActuator*) bact->data;
+			SCA_2DFilterActuator *tmp = NULL;
+
+			RAS_2DFilterManager::RAS_2DFILTER_MODE filtermode;
+			switch(_2dfilter->type)
+			{
+				case ACT_2DFILTER_MOTIONBLUR:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_MOTIONBLUR;
+					break;
+				case ACT_2DFILTER_BLUR:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_BLUR;
+					break;
+				case ACT_2DFILTER_SHARPEN:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_SHARPEN;
+					break;
+				case ACT_2DFILTER_DILATION:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_DILATION;
+					break;
+				case ACT_2DFILTER_EROSION:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_EROSION;
+					break;
+				case ACT_2DFILTER_LAPLACIAN:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_LAPLACIAN;
+					break;
+				case ACT_2DFILTER_SOBEL:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_SOBEL;
+					break;
+				case ACT_2DFILTER_PREWITT:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_PREWITT;
+					break;
+				case ACT_2DFILTER_GRAYSCALE:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_GRAYSCALE;
+					break;
+				case ACT_2DFILTER_SEPIA:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_SEPIA;
+					break;
+				case ACT_2DFILTER_INVERT:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_INVERT;
+					break;
+				case ACT_2DFILTER_CUSTOMFILTER:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_CUSTOMFILTER;
+					break;
+				case ACT_2DFILTER_NOFILTER:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_NOFILTER;
+					break;
+				case ACT_2DFILTER_DISABLED:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_DISABLED;
+					break;
+				case ACT_2DFILTER_ENABLED:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_ENABLED;
+					break;
+				default:
+					filtermode = RAS_2DFilterManager::RAS_2DFILTER_NOFILTER;
+					break;
+			}
+
+			tmp = new SCA_2DFilterActuator(gameobj, filtermode, _2dfilter->flag,
+				_2dfilter->float_arg,_2dfilter->int_arg,ketsjiEngine->GetRasterizer(),scene);
+
+			if (_2dfilter->text)
+			{
+				char *buf;
+				// this is some blender specific code
+				buf = txt_to_buf(_2dfilter->text);
+				if (buf)
+				{
+					tmp->SetShaderText(buf);
+					MEM_freeN(buf);
+				}
+			}
+
+			baseact = tmp;
+
+		}
+		break;
+		case ACT_PARENT:
+			{
+				bParentActuator *parAct = (bParentActuator *) bact->data;
+				int mode = KX_ParentActuator::KX_PARENT_NODEF;
+				bool addToCompound = true;
+				bool ghost = true;
+				KX_GameObject *tmpgob = NULL;
+
+				switch(parAct->type)
+				{
+					case ACT_PARENT_SET:
+						mode = KX_ParentActuator::KX_PARENT_SET;
+						tmpgob = converter->FindGameObject(parAct->ob);
+						addToCompound = !(parAct->flag & ACT_PARENT_COMPOUND);
+						ghost = !(parAct->flag & ACT_PARENT_GHOST);
+						break;
+					case ACT_PARENT_REMOVE:
+						mode = KX_ParentActuator::KX_PARENT_REMOVE;
+						tmpgob = NULL;
+						break;
+				}
+	
+				KX_ParentActuator *tmpparact
+					= new KX_ParentActuator(gameobj,
+					mode,
+					addToCompound,
+					ghost,
+					tmpgob);
+				baseact = tmpparact;
+				break;
+			}
+		
+		case ACT_ARMATURE:
+			{
+				bArmatureActuator* armAct = (bArmatureActuator*) bact->data;
+				KX_GameObject *tmpgob = converter->FindGameObject(armAct->target);
+				KX_GameObject *subgob = converter->FindGameObject(armAct->subtarget);
+				BL_ArmatureActuator* tmparmact = new BL_ArmatureActuator(gameobj, armAct->type, armAct->posechannel, armAct->constraint, tmpgob, subgob, armAct->weight);
+				baseact = tmparmact;
+				break;
+			}
+		case ACT_STEERING:
+			{
+				bSteeringActuator *stAct = (bSteeringActuator *) bact->data;
+				KX_GameObject *navmeshob = NULL;
+				if (stAct->navmesh)
+				{
+					PointerRNA settings_ptr;
+					RNA_pointer_create((ID *)stAct->navmesh, &RNA_GameObjectSettings, stAct->navmesh, &settings_ptr);
+					if (RNA_enum_get(&settings_ptr, "physics_type") == OB_BODY_TYPE_NAVMESH)
+						navmeshob = converter->FindGameObject(stAct->navmesh);
+				}
+				KX_GameObject *targetob = converter->FindGameObject(stAct->target);
+
+				int mode = KX_SteeringActuator::KX_STEERING_NODEF;
+				switch(stAct->type)
+				{
+				case ACT_STEERING_SEEK:
+					mode = KX_SteeringActuator::KX_STEERING_SEEK;
+					break;
+				case ACT_STEERING_FLEE:
+					mode = KX_SteeringActuator::KX_STEERING_FLEE;
+					break;
+				case ACT_STEERING_PATHFOLLOWING:
+					mode = KX_SteeringActuator::KX_STEERING_PATHFOLLOWING;
+					break;
+				}
+
+				bool selfTerminated = (stAct->flag & ACT_STEERING_SELFTERMINATED) !=0;
+				bool enableVisualization = (stAct->flag & ACT_STEERING_ENABLEVISUALIZATION) !=0;
+				short facingMode = (stAct->flag & ACT_STEERING_AUTOMATICFACING) ? stAct->facingaxis : 0;
+				bool normalup = (stAct->flag & ACT_STEERING_NORMALUP) !=0;
+				KX_SteeringActuator *tmpstact
+					= new KX_SteeringActuator(gameobj, mode, targetob, navmeshob,stAct->dist, 
+					stAct->velocity, stAct->acceleration, stAct->turnspeed, 
+					selfTerminated, stAct->updateTime,
+					scene->GetObstacleSimulation(), facingMode, normalup, enableVisualization);
+				baseact = tmpstact;
+				break;
+			}
 		default:
 			; /* generate some error */
 		}
@@ -854,14 +1115,17 @@ void BL_ConvertActuators(char* maggiename,
 			CIntValue* uniqueval = new CIntValue(uniqueint);
 			uniquename += uniqueval->GetText();
 			uniqueval->Release();
-			baseact->SetName(STR_String(bact->name));
+			baseact->SetName(bact->name);
 			//gameobj->SetProperty(uniquename,baseact);
 			gameobj->AddActuator(baseact);
 			
 			converter->RegisterGameActuator(baseact, bact);
+			// done with baseact, release it
+			baseact->Release();
 		}
 		
 		bact = bact->next;
 	}
 }
 
+
