@@ -1,3 +1,6 @@
+/** \file gameengine/Expressions/Value.cpp
+ *  \ingroup expressions
+ */
 // Value.cpp: implementation of the CValue class.
 // developed at Eindhoven University of Technology, 1997
 // by the OOPS team
@@ -14,7 +17,6 @@
  * purpose.  It is provided "as is" without express or implied warranty.
  *
  */
-
 #include "Value.h"
 #include "FloatValue.h"
 #include "IntValue.h"
@@ -24,160 +26,71 @@
 #include "ErrorValue.h"
 #include "ListValue.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
 //////////////////////////////////////////////////////////////////////
 
 double CValue::m_sZeroVec[3] = {0.0,0.0,0.0};
 
-#ifndef NO_EXP_PYTHON_EMBEDDING
-
-PyObject* cvalue_add(PyObject*v, PyObject*w)
-{
-	return  ((CValue*)v)->Calc(VALUE_ADD_OPERATOR,(CValue*)w);
-}
-PyObject* cvalue_sub(PyObject*v, PyObject*w)
-{
-	return  ((CValue*)v)->Calc(VALUE_SUB_OPERATOR,(CValue*)w);
-}
-PyObject* cvalue_mul(PyObject*v, PyObject*w)
-{
-	return  ((CValue*)v)->Calc(VALUE_MUL_OPERATOR,(CValue*)w);
-}
-PyObject* cvalue_div(PyObject*v, PyObject*w)
-{
-	return  ((CValue*)v)->Calc(VALUE_DIV_OPERATOR,(CValue*)w);
-}
-PyObject* cvalue_neg(PyObject*v)
-{
-	return  ((CValue*)v)->Calc(VALUE_NEG_OPERATOR,(CValue*)v);
-}
-PyObject* cvalue_pos(PyObject*v)
-{
-	return  ((CValue*)v)->Calc(VALUE_POS_OPERATOR,(CValue*)v);
-}
-
-
-int MyPyCompare (PyObject* v,PyObject* w)
-{
-	CValue* eqval =  ((CValue*)v)->Calc(VALUE_EQL_OPERATOR,(CValue*)w);
-	STR_String txt = eqval->GetText();
-	eqval->Release();
-	if (txt=="TRUE")
-		return 0;
-	CValue* lessval =  ((CValue*)v)->Calc(VALUE_LES_OPERATOR,(CValue*)w);
-	txt = lessval->GetText();
-	lessval->Release();
-	if (txt=="TRUE")
-		return -1;
-
-	return 1;
-}
-
-
-int cvalue_coerce(PyObject** pv,PyObject** pw)
-{
-	if (PyInt_Check(*pw)) {
-		double db  = (double)PyInt_AsLong(*pw);
-		*pw = new CIntValue(db);
-		Py_INCREF(*pv);
-		return 0;
-	}
-	else if (PyLong_Check(*pw)) {
-		double db = PyLong_AsDouble(*pw);
-		*pw = new CFloatValue(db);
-		Py_INCREF(*pv);
-		return 0;
-	}
-	else if (PyFloat_Check(*pw)) {
-		double db = PyFloat_AsDouble(*pw);
-		*pw = new CFloatValue(db);
-		Py_INCREF(*pv);
-		return 0;
-	} else if (PyString_Check(*pw)) {
-		const STR_String str = PyString_AsString(*pw);
-		*pw = new CStringValue(str,"");
-		Py_INCREF(*pv);
-		return 0;
-	}
-	return 1; /* Can't do it */
-
-}
-static PyNumberMethods cvalue_as_number = {
-	(binaryfunc)cvalue_add, /*nb_add*/
-	(binaryfunc)cvalue_sub, /*nb_subtract*/
-	(binaryfunc)cvalue_mul, /*nb_multiply*/
-	(binaryfunc)cvalue_div, /*nb_divide*/
-	0,//(binaryfunc)cvalue_remainder,	/*nb_remainder*/
-	0,//(binaryfunc)cvalue_divmod,	/*nb_divmod*/
-	0,//0,//0,//0,//(ternaryfunc)cvalue_pow, /*nb_power*/
-	(unaryfunc)cvalue_neg, /*nb_negative*/
-	0,//(unaryfunc)cvalue_pos, /*nb_positive*/
-	0,//(unaryfunc)cvalue_abs, /*nb_absolute*/
-	0,//(inquiry)cvalue_nonzero, /*nb_nonzero*/
-	0,		/*nb_invert*/
-	0,		/*nb_lshift*/
-	0,		/*nb_rshift*/
-	0,		/*nb_and*/
-	0,		/*nb_xor*/
-	0,		/*nb_or*/
-	(coercion)cvalue_coerce, /*nb_coerce*/
-	0,//(unaryfunc)cvalue_int, /*nb_int*/
-	0,//(unaryfunc)cvalue_long, /*nb_long*/
-	0,//(unaryfunc)cvalue_float, /*nb_float*/
-	0,		/*nb_oct*/
-	0,		/*nb_hex*/
-};
-
+#ifdef WITH_PYTHON
 
 PyTypeObject CValue::Type = {
-	PyObject_HEAD_INIT(&PyType_Type)
-	0,
+	PyVarObject_HEAD_INIT(NULL, 0)
 	"CValue",
-	sizeof(CValue),
+	sizeof(PyObjectPlus_Proxy),
 	0,
-	PyDestructor,
-	0,
-	__getattr,
-	__setattr,
-	&MyPyCompare,
-	__repr,
-	&cvalue_as_number,
+	py_base_dealloc,
 	0,
 	0,
 	0,
-	0
-};
-
-PyParentObject CValue::Parents[] = {
-	&CValue::Type,
-		NULL
+	0,
+	py_base_repr,
+	0,
+	0,0,0,0,0,
+	NULL,
+	NULL,
+	0,
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+	0,0,0,0,0,0,0,
+	Methods,
+	0,
+	0,
+	&PyObjectPlus::Type,
+	0,0,0,0,0,0,
+	py_base_new
 };
 
 PyMethodDef CValue::Methods[] = {
-//  	{ "printHello", (PyCFunction) CValue::sPyPrintHello, Py_NEWARGS},
-	{ "getName", (PyCFunction) CValue::sPyGetName, Py_NEWARGS},
 	{NULL,NULL} //Sentinel
 };
+#endif // WITH_PYTHON
 
-PyObject* CValue::PyGetName(PyObject* self,PyObject* args,PyObject* kwds)
+
+/*#define CVALUE_DEBUG*/
+#ifdef CVALUE_DEBUG
+int gRefCount;
+struct SmartCValueRef 
 {
-	PyObject* pyname = PyString_FromString(this->GetName());
-	return pyname;
-}
+	CValue *m_ref;
+	int m_count;
+	SmartCValueRef(CValue *ref)
+	{
+		m_ref = ref;
+		m_count = gRefCount++;
+	}
+};
 
+#include <vector>
 
+std::vector<SmartCValueRef> gRefList;
+#endif
+
+#ifdef _DEBUG
+//int gRefCountValue;
+#endif
 
-CValue::CValue(PyTypeObject *T)
-		: PyObjectPlus(T),
-#else
 CValue::CValue()
-: 
-#endif //NO_EXP_PYTHON_EMBEDDING
+		: PyObjectPlus(),
 	
 m_pNamedPropertyArray(NULL),
 m_refcount(1)
@@ -187,6 +100,12 @@ effect: constucts a CValue
 */
 {
 	//debug(gRefCountValue++)	// debugging
+#ifdef _DEBUG
+	//gRefCountValue++;
+#ifdef CVALUE_DEBUG
+	gRefList.push_back(SmartCValueRef(this));
+#endif
+#endif
 }
 
 
@@ -200,6 +119,18 @@ effect: deletes the object
 	ClearProperties();
 
 	assertd (m_refcount==0);
+#ifdef CVALUE_DEBUG
+	std::vector<SmartCValueRef>::iterator it;
+	for (it=gRefList.begin(); it!=gRefList.end(); it++)
+	{
+		if (it->m_ref == this)
+		{
+			*it = gRefList.back();
+			gRefList.pop_back();
+			break;
+		}
+	}
+#endif
 }
 
 
@@ -219,6 +150,9 @@ STR_String CValue::op2str (VALUE_OPERATOR op)
 	
 	STR_String opmsg;
 	switch (op) {
+	case VALUE_MOD_OPERATOR:
+		opmsg = " % ";
+		break;
 	case VALUE_ADD_OPERATOR:
 		opmsg = " + ";
 		break;
@@ -275,73 +209,90 @@ STR_String CValue::op2str (VALUE_OPERATOR op)
 //
 void CValue::SetProperty(const STR_String & name,CValue* ioProperty)
 {
-	// Check if somebody is setting an empty property
 	if (ioProperty==NULL)
-	{
+	{	// Check if somebody is setting an empty property
 		trace("Warning:trying to set empty property!");
 		return;
 	}
 
-	// Make sure we have a property array
-	if (m_pNamedPropertyArray == NULL)
+	if (m_pNamedPropertyArray)
+	{	// Try to replace property (if so -> exit as soon as we replaced it)
+		CValue* oldval = (*m_pNamedPropertyArray)[name];
+		if (oldval)
+			oldval->Release();
+	}
+	else { // Make sure we have a property array
 		m_pNamedPropertyArray = new std::map<STR_String,CValue *>;
-
-	// Try to replace property (if so -> exit as soon as we replaced it)
-	CValue* oldval = (*m_pNamedPropertyArray)[name];
-	if (oldval)
-	{
-		oldval->Release();
 	}
 	
 	// Add property at end of array
-	(*m_pNamedPropertyArray)[name] = ioProperty;//->Add(ioProperty);
+	(*m_pNamedPropertyArray)[name] = ioProperty->AddRef();//->Add(ioProperty);
 }
 
+void CValue::SetProperty(const char* name,CValue* ioProperty)
+{
+	if (ioProperty==NULL)
+	{	// Check if somebody is setting an empty property
+		trace("Warning:trying to set empty property!");
+		return;
+	}
 
+	if (m_pNamedPropertyArray)
+	{	// Try to replace property (if so -> exit as soon as we replaced it)
+		CValue* oldval = (*m_pNamedPropertyArray)[name];
+		if (oldval)
+			oldval->Release();
+	}
+	else { // Make sure we have a property array
+		m_pNamedPropertyArray = new std::map<STR_String,CValue *>;
+	}
+	
+	// Add property at end of array
+	(*m_pNamedPropertyArray)[name] = ioProperty->AddRef();//->Add(ioProperty);
+}
 
 //
 // Get pointer to a property with name <inName>, returns NULL if there is no property named <inName>
 //
 CValue* CValue::GetProperty(const STR_String & inName)
 {
-	// Check properties, as soon as we found it -> Return a pointer to the property
-	CValue* result = NULL;
-	if (m_pNamedPropertyArray)
-	{
-		std::map<STR_String,CValue*>::iterator it = (*m_pNamedPropertyArray).find(inName);
-		if (!( it==m_pNamedPropertyArray->end()))
-		{
-			result = (*it).second;
-		}
-
+	if (m_pNamedPropertyArray) {
+		std::map<STR_String,CValue*>::iterator it = m_pNamedPropertyArray->find(inName);
+		if (it != m_pNamedPropertyArray->end())
+			return (*it).second;
 	}
-		//for (int i=0; i<m_pValuePropertyArray->size(); i++)
-		//	if ((*m_pValuePropertyArray)[i]->GetName() == inName)
-		//		return (*m_pValuePropertyArray)[i];
-	
-	// Did not find property with name <inName>, return NULL property pointer
-	return result;
+	return NULL;
 }
 
-
+CValue* CValue::GetProperty(const char *inName)
+{
+	if (m_pNamedPropertyArray) {
+		std::map<STR_String,CValue*>::iterator it = m_pNamedPropertyArray->find(inName);
+		if (it != m_pNamedPropertyArray->end())
+			return (*it).second;
+	}
+	return NULL;
+}
 
 //
 // Get text description of property with name <inName>, returns an empty string if there is no property named <inName>
 //
-STR_String CValue::GetPropertyText(const STR_String & inName,const STR_String& deftext)
+const STR_String& CValue::GetPropertyText(const STR_String & inName)
 {
+	const static STR_String sEmpty("");
+
 	CValue *property = GetProperty(inName);
 	if (property)
 		return property->GetText();
 	else
-		return deftext;//String::sEmpty;
+		return sEmpty;
 }
 
 float CValue::GetPropertyNumber(const STR_String& inName,float defnumber)
 {
 	CValue *property = GetProperty(inName);
 	if (property)
-		return property->GetNumber();
+		return property->GetNumber(); 
 	else
 		return defnumber;
 }
@@ -351,20 +302,39 @@ float CValue::GetPropertyNumber(const STR_String& inName,float defnumber)
 //
 // Remove the property named <inName>, returns true if the property was succesfully removed, false if property was not found or could not be removed
 //
-bool CValue::RemoveProperty(const STR_String & inName)
+bool CValue::RemoveProperty(const char *inName)
 {
 	// Check if there are properties at all which can be removed
-	if (m_pNamedPropertyArray == NULL)
-		return false;
-
-	// Scan all properties, as soon as we find one with <inName> -> Remove it
-//  	CValue* val = (*m_pNamedPropertyArray)[inName];
-	if (m_pNamedPropertyArray->erase(inName)) return true;
+	if (m_pNamedPropertyArray)
+	{
+		std::map<STR_String,CValue*>::iterator it = m_pNamedPropertyArray->find(inName);
+		if (it != m_pNamedPropertyArray->end())
+		{
+			((*it).second)->Release();
+			m_pNamedPropertyArray->erase(it);
+			return true;
+		}
+	}
 	
 	return false;
 }
 
-
+//
+// Get Property Names
+//
+vector<STR_String> CValue::GetPropertyNames()
+{
+	vector<STR_String> result;
+	if(!m_pNamedPropertyArray) return result;
+	result.reserve(m_pNamedPropertyArray->size());
+	
+	std::map<STR_String,CValue*>::iterator it;
+	for (it= m_pNamedPropertyArray->begin(); (it != m_pNamedPropertyArray->end()); it++)
+	{
+		result.push_back((*it).first);
+	}
+	return result;
+}
 
 //
 // Clear all properties
@@ -376,11 +346,11 @@ void CValue::ClearProperties()
 		return;
 
 	// Remove all properties
-	for ( std::map<STR_String,CValue*>::iterator it = m_pNamedPropertyArray->begin();
-	!(it == m_pNamedPropertyArray->end());it++)
+	std::map<STR_String,CValue*>::iterator it;
+	for (it= m_pNamedPropertyArray->begin();(it != m_pNamedPropertyArray->end()); it++)
 	{
 		CValue* tmpval = (*it).second;
-		STR_String name = (*it).first;
+		//STR_String name = (*it).first;
 		tmpval->Release();
 	}
 
@@ -396,9 +366,11 @@ void CValue::ClearProperties()
 //
 void CValue::SetPropertiesModified(bool inModified)
 {
-	int numprops = GetPropertyCount();
-	for (int i=0; i<numprops; i++)
-		GetProperty(i)->SetModified(inModified);
+	if(!m_pNamedPropertyArray) return;
+	std::map<STR_String,CValue*>::iterator it;
+	
+	for (it= m_pNamedPropertyArray->begin();(it != m_pNamedPropertyArray->end()); it++)
+		((*it).second)->SetModified(inModified);
 }
 
 
@@ -408,11 +380,13 @@ void CValue::SetPropertiesModified(bool inModified)
 //
 bool CValue::IsAnyPropertyModified()
 {
-	int numprops = GetPropertyCount();
-	for (int i=0;i<numprops;i++)
-		if (GetProperty(i)->IsModified())
+	if(!m_pNamedPropertyArray) return false;
+	std::map<STR_String,CValue*>::iterator it;
+	
+	for (it= m_pNamedPropertyArray->begin();(it != m_pNamedPropertyArray->end()); it++)
+		if (((*it).second)->IsModified())
 			return true;
-
+	
 	return false;
 }
 
@@ -421,7 +395,6 @@ bool CValue::IsAnyPropertyModified()
 //
 // Get property number <inIndex>
 //
-
 CValue* CValue::GetProperty(int inIndex)
 {
 
@@ -430,8 +403,8 @@ CValue* CValue::GetProperty(int inIndex)
 
 	if (m_pNamedPropertyArray)
 	{
-		for ( std::map<STR_String,CValue*>::iterator it = m_pNamedPropertyArray->begin();
-		!(it == m_pNamedPropertyArray->end());it++)
+		std::map<STR_String,CValue*>::iterator it;
+		for (it= m_pNamedPropertyArray->begin(); (it != m_pNamedPropertyArray->end()); it++)
 		{
 			if (count++==inIndex)
 			{
@@ -458,31 +431,6 @@ int CValue::GetPropertyCount()
 }
 
 
-
-
-
-void CValue::CloneProperties(CValue *replica)
-{
-	
-	if (m_pNamedPropertyArray)
-	{
-		replica->m_pNamedPropertyArray=NULL;
-		for ( std::map<STR_String,CValue*>::iterator it = m_pNamedPropertyArray->begin();
-		!(it == m_pNamedPropertyArray->end());it++)
-		{
-			
-			replica->SetProperty((*it).first,(*it).second->GetReplica());
-		}
-	}
-
-	
-}
-
-
-
-
-
-
 double*		CValue::GetVector3(bool bGetTransformedVec)
 {
 	assertd(false); // don;t get vector from me
@@ -490,54 +438,15 @@ double*		CValue::GetVector3(bool bGetTransformedVec)
 }
 
 
-
-
-
-
 /*---------------------------------------------------------------------------------------------------------------------
 	Reference Counting
 ---------------------------------------------------------------------------------------------------------------------*/
 
 
 
-//
-// Add a reference to this value
-//
-CValue *CValue::AddRef()
-{
-	// Increase global reference count, used to see at the end of the program
-	// if all CValue-derived classes have been dereferenced to 0
-	//debug(gRefCountValue++);
-	m_refcount++; 
-	return this;
-}
-
-
-
 //
 // Release a reference to this value (when reference count reaches 0, the value is removed from the heap)
 //
-int	CValue::Release()
-{
-	// Decrease global reference count, used to see at the end of the program
-	// if all CValue-derived classes have been dereferenced to 0
-	//debug(gRefCountValue--);
-
-	// Decrease local reference count, if it reaches 0 the object should be freed
-	if (--m_refcount > 0)
-	{
-		// Reference count normal, return new reference count
-		return m_refcount;
-	}
-	else
-	{
-		// Reference count reached 0, delete ourselves and return 0
-//		assert(m_refcount==0, "Reference count reached sub-zero, object released too much");
-		delete this;
-		return 0;
-	}
-
-}
 
 
 
@@ -550,27 +459,40 @@ void CValue::DisableRefCount()
 	m_refcount--;
 
 	//debug(gRefCountValue--);
+#ifdef _DEBUG
+	//gRefCountValue--;
+#endif
 	m_ValFlags.RefCountDisabled=true;
 }
 
 
 
-void CValue::AddDataToReplica(CValue *replica)
+void CValue::ProcessReplica() /* was AddDataToReplica in 2.48 */
 {
-	replica->m_refcount = 1;
+	m_refcount = 1;
+	
 #ifdef _DEBUG
 	//gRefCountValue++;
 #endif
-	replica->m_ValFlags.RefCountDisabled = false;
+	PyObjectPlus::ProcessReplica();
 
-	replica->ReplicaSetName(GetName());
+	m_ValFlags.RefCountDisabled = false;
 
-	//copy all props
-	CloneProperties(replica);
+	/* copy all props */
+	if (m_pNamedPropertyArray)
+	{
+		std::map<STR_String,CValue*> *pOldArray = m_pNamedPropertyArray;
+		m_pNamedPropertyArray=NULL;
+		std::map<STR_String,CValue*>::iterator it;
+		for (it= pOldArray->begin(); (it != pOldArray->end()); it++)
+		{
+			CValue *val = (*it).second->GetReplica();
+			SetProperty((*it).first,val);
+			val->Release();
+		}
+	}
 }
 
-
-
 CValue*	CValue::FindIdentifier(const STR_String& identifiername)
 {
 
@@ -590,71 +512,46 @@ CValue*	CValue::FindIdentifier(const STR_String& identifiername)
 	} else
 	{
 		result = GetProperty(identifiername);
+		if (result)
+			return result->AddRef();
+	}
+	if (!result)
+	{
+		// warning here !!!
+		result = new CErrorValue(identifiername+" not found");
 	}
-	if (result)
-		return result->AddRef();
-	// warning here !!!
-	result = new CErrorValue(identifiername+" not found");
 	return result;
 }
 
+#ifdef WITH_PYTHON
 
-#ifndef NO_EXP_PYTHON_EMBEDDING
-
-
-static PyMethodDef	CValueMethods[] = 
-{
-	//{ "new", CValue::PyMake , Py_NEWARGS},
-	{ NULL,NULL}	// Sentinel
+PyAttributeDef CValue::Attributes[] = {
+	KX_PYATTRIBUTE_RO_FUNCTION("name",	CValue, pyattr_get_name),
+	{ NULL }	//Sentinel
 };
 
-
-PyObject*	CValue::_getattr(const STR_String& attr)
-{
-	CValue* resultattr = FindIdentifier(attr);
-	STR_String text;
-	if (resultattr)
-	{
-		if (resultattr->IsError())
-		{
-			resultattr->Release();
-		} else
-		{
-			// to avoid some compare problems, return a real pythonthing
-			PyObject* pyconvert = resultattr->ConvertValueToPython();
-			if (pyconvert)
-			{
-				resultattr->Release();
-				return pyconvert;
-			} else
-			{
-				// also check if it's already in pythoninterpreter!
-				return resultattr;
-			}
-			
-		}
-	}
-	_getattr_up(PyObjectPlus);
+PyObject * CValue::pyattr_get_name(void * self_v, const KX_PYATTRIBUTE_DEF * attrdef) {
+	CValue * self = static_cast<CValue *> (self_v);
+	return PyUnicode_From_STR_String(self->GetName());
 }
 
-CValue* CValue::ConvertPythonToValue(PyObject* pyobj)
+CValue* CValue::ConvertPythonToValue(PyObject* pyobj, const char *error_prefix)
 {
 
 	CValue* vallie = NULL;
-
-	PyTypeObject* type = pyobj->ob_type;
-
-	if (type == &PyList_Type)
+	/* refcounting is broking here! - this crashes anyway, just store a python list for KX_GameObject */
+#if 0
+	if (PyList_Check(pyobj))
 	{
 		CListValue* listval = new CListValue();
 		bool error = false;
 
-		int i;
-		int numitems = PyList_Size(pyobj);
+		Py_ssize_t i;
+		Py_ssize_t numitems = PyList_GET_SIZE(pyobj);
 		for (i=0;i<numitems;i++)
 		{
 			PyObject* listitem = PyList_GetItem(pyobj,i); /* borrowed ref */
-			CValue* listitemval = ConvertPythonToValue(listitem);
+			CValue* listitemval = ConvertPythonToValue(listitem, error_prefix);
 			if (listitemval)
 			{
 				listval->Add(listitemval);
@@ -674,77 +571,54 @@ CValue* CValue::ConvertPythonToValue(PyObject* pyobj)
 		}
 
 	} else
-	if (type == &PyFloat_Type)
+#endif
+	if (PyFloat_Check(pyobj))
 	{
-		float fl;
-		PyArg_Parse(pyobj,"f",&fl);
-		vallie = new CFloatValue(fl);
+		vallie = new CFloatValue( (float)PyFloat_AsDouble(pyobj) );
 	} else
-	if (type==&PyInt_Type)
+	if (PyLong_Check(pyobj))
 	{
-		int innie;
-		PyArg_Parse(pyobj,"i",&innie);
-		vallie = new CIntValue(innie);
+		vallie = new CIntValue( (cInt)PyLong_AsLongLong(pyobj) );
 	} else
-	
-	if (type==&PyString_Type)
+	if (PyUnicode_Check(pyobj))
 	{
-		vallie = new CStringValue(PyString_AsString(pyobj),"");
+		vallie = new CStringValue(_PyUnicode_AsString(pyobj),"");
 	} else
-	if (type==&CValue::Type || type==&CListValue::Type)
+	if (PyObject_TypeCheck(pyobj, &CValue::Type)) /* Note, dont let these get assigned to GameObject props, must check elsewhere */
 	{
-		vallie = ((CValue*) pyobj)->AddRef();
+		vallie = (static_cast<CValue *>(BGE_PROXY_REF(pyobj)))->AddRef();
+	} else
+	{
+		/* return an error value from the caller */
+		PyErr_Format(PyExc_TypeError, "%scould convert python value to a game engine property", error_prefix);
 	}
 	return vallie;
 
 }
-int	CValue::_setattr(const STR_String& attr,PyObject* pyobj)
-{
-	
-	if (!pyobj) { // member deletion
-		RemoveProperty(attr);
-		return 0;
-	}
 
-	CValue* vallie = ConvertPythonToValue(pyobj);
-	if (vallie)
+PyObject* CValue::ConvertKeysToPython(void)
+{
+	if (m_pNamedPropertyArray)
 	{
-		CValue* oldprop = GetProperty(attr);
-		
-		if (oldprop)
+		PyObject *pylist= PyList_New(m_pNamedPropertyArray->size());
+		Py_ssize_t i= 0;
+
+		std::map<STR_String,CValue*>::iterator it;
+		for (it= m_pNamedPropertyArray->begin(); (it != m_pNamedPropertyArray->end()); it++)
 		{
-			oldprop->SetValue(vallie);
-		} else
-		{
-			SetProperty(attr,vallie->AddRef());
+			PyList_SET_ITEM(pylist, i++, PyUnicode_From_STR_String((*it).first));
 		}
-		vallie->Release();
+
+		return pylist;
 	}
-	
-	//PyObjectPlus::_setattr(attr,value);
-	return 0;
-};
-/*
-PyObject*	CValue::PyMake(PyObject* ignored,PyObject* args)
-{
-
-	//Py_Try(PyArg_ParseTuple(args,"s",&name));
-	Py_INCREF(Py_None);
-	return Py_None;//new CValue();
-}
-*/
-
-extern "C" {
-	void initCValue(void)
-	{
-		Py_InitModule("CValue",CValueMethods);
+	else {
+		return PyList_New(0);
 	}
 }
 
+#endif // WITH_PYTHON
 
 
-#endif //NO_EXP_PYTHON_EMBEDDING
-
 ///////////////////////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////////////////////
 /* These implementations were moved out of the header */
@@ -763,6 +637,3 @@ void CValue::SetValue(CValue* newval)
 	// no one should get here
 	assertd(newval->GetNumber() == 10121969);	
 }
-///////////////////////////////////////////////////////////////////////////////////////////////
-///////////////////////////////////////////////////////////////////////////////////////////////
-
