@@ -1,15 +1,10 @@
-/**
- * $Id: SCA_KeyboardSensor.h,v 1.6 2004/03/22 22:01:33 jesterking Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,8 +22,12 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
- * Sensor for keyboard input
+ * ***** END GPL LICENSE BLOCK *****
+ */
+
+/** \file SCA_KeyboardSensor.h
+ *  \ingroup gamelogic
+ *  \brief Sensor for keyboard input
  */
 
 #ifndef __KX_KEYBOARDSENSOR
@@ -45,9 +44,7 @@
  * exclusive.  */
 class SCA_KeyboardSensor : public SCA_ISensor
 {
-	Py_Header;
-	class SCA_KeyboardManager*	m_pKeyboardMgr;
-
+	Py_Header
 
 	/**
 	 * the key this sensor is sensing for
@@ -70,7 +67,7 @@ class SCA_KeyboardSensor : public SCA_ISensor
 	 * The property that indicates whether or not to log text when in
 	 * loggin mode. If the property equals 0, no loggin is done. For
 	 * all other values, logging is active. Logging can only become
-	 * active if there is a property to log to. Logging is independant
+	 * active if there is a property to log to. Logging is independent
 	 * from hotkey settings. */
 	STR_String	m_toggleprop;
 
@@ -84,22 +81,6 @@ class SCA_KeyboardSensor : public SCA_ISensor
 	 */
 	void AddToTargetProp(int keyIndex);
 
-	/**
-	 * Determine whether this character can be printed. We cannot use
-	 * the library functions here, because we need to test our own
-	 * keycodes. */
-	bool IsPrintable(int keyIndex);
-
-	/**
-	 * Transform keycodes to something printable.
-	 */
-	char ToCharacter(int keyIndex, bool shifted);
-
-	/**
-	 * Tests whether this is a delete key.
-	 */	
-	bool IsDelete(int keyIndex);
-
 	/**
 	 * Tests whether shift is pressed.
 	 */
@@ -113,47 +94,50 @@ public:
 					   bool bAllKeys,
 					   const STR_String& targetProp,
 					   const STR_String& toggleProp,
-					   SCA_IObject* gameobj,
-					   PyTypeObject* T=&Type );
+					   SCA_IObject* gameobj);
 	virtual ~SCA_KeyboardSensor();
 	virtual CValue* GetReplica();
+	virtual void Init();
+
 
 	short int GetHotkey();
-	virtual bool Evaluate(CValue* event);
+	virtual bool Evaluate();
 	virtual bool IsPositiveTrigger();
 	bool	TriggerOnAllKeys();
 
+#ifdef WITH_PYTHON
 	/* --------------------------------------------------------------------- */
 	/* Python interface ---------------------------------------------------- */
 	/* --------------------------------------------------------------------- */
 
-	virtual PyObject*  _getattr(char *attr);
-
-
-	PyObject* PySetAllMode(PyObject* self, 
-			       PyObject* args, 
-			       PyObject* kwds);
-	static PyObject* sPySetAllMode(PyObject* self, 
-				       PyObject* args, 
-				       PyObject* kwds);
+	// KeyEvents: 
+	KX_PYMETHOD_DOC_NOARGS(SCA_KeyboardSensor,getEventList); 
+	// KeyStatus: 
+	KX_PYMETHOD_DOC_O(SCA_KeyboardSensor,getKeyStatus);
 	
-	/** 1. GetKey : check which key this sensor looks at */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,GetKey);
-	/** 2. SetKey: change the key to look at */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,SetKey);
-	/** 3. GetHold1 : set the first bucky bit */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,GetHold1);
-	/** 4. SetHold1: change the first bucky bit */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,SetHold1);
-	/** 5. GetHold2 : set the second bucky bit */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,GetHold2);
-	/** 6. SetHold2: change the second bucky bit */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,SetHold2);
-	/** 9. GetPressedKeys: */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,GetPressedKeys); 
-	/** 9. GetCurrrentlyPressedKeys: */
-	KX_PYMETHOD_DOC(SCA_KeyboardSensor,GetCurrentlyPressedKeys); 
+	static PyObject*	pyattr_get_events(void* self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+#endif
 };
 
+
+/**
+ * Transform keycodes to something printable.
+ */
+char ToCharacter(int keyIndex, bool shifted);
+
+/**
+ * Determine whether this character can be printed. We cannot use
+ * the library functions here, because we need to test our own
+ * keycodes. */
+bool IsPrintable(int keyIndex);
+
+/**
+ * Tests whether this is a delete key.
+ */	
+bool IsDelete(int keyIndex);
+
+
 #endif //__KX_KEYBOARDSENSOR
 
+
+
