@@ -1,17 +1,13 @@
-/**
+/*
  * Property sensor
  *
- * $Id: SCA_PropertySensor.cpp,v 1.4 2004/03/22 22:01:33 jesterking Exp $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -29,9 +25,16 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
+/** \file gameengine/GameLogic/SCA_PropertySensor.cpp
+ *  \ingroup gamelogic
+ */
+
+
+#include <stddef.h>
+
 #include <iostream>
 #include "SCA_PropertySensor.h"
 #include "Operator2Expr.h"
@@ -40,64 +43,80 @@
 #include "StringValue.h"
 #include "SCA_EventManager.h"
 #include "SCA_LogicManager.h"
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "BoolValue.h"
+#include "FloatValue.h"
+#include <stdio.h>
 
 SCA_PropertySensor::SCA_PropertySensor(SCA_EventManager* eventmgr,
 									 SCA_IObject* gameobj,
 									 const STR_String& propname,
 									 const STR_String& propval,
 									 const STR_String& propmaxval,
-									 KX_PROPSENSOR_TYPE checktype,
-									 PyTypeObject* T )
-	: SCA_ISensor(gameobj,eventmgr,T),
+									 KX_PROPSENSOR_TYPE checktype)
+	: SCA_ISensor(gameobj,eventmgr),
 	  m_checktype(checktype),
 	  m_checkpropval(propval),
 	  m_checkpropmaxval(propmaxval),
 	  m_checkpropname(propname),
-	  m_lastresult(false),
 	  m_range_expr(NULL)
 {
-	m_recentresult=false;
 	//CParser pars;
 	//pars.SetContext(this->AddRef());
 	//CValue* resultval = m_rightexpr->Calculate();
 
 	CValue* orgprop = GetParent()->FindIdentifier(m_checkpropname);
-	if (orgprop)
+	if (!orgprop->IsError())
 	{
 		m_previoustext = orgprop->GetText();
-		orgprop->Release();
 	}
+	orgprop->Release();
 
 	if (m_checktype==KX_PROPSENSOR_INTERVAL)
 	{
 		PrecalculateRangeExpression();
 	}
+	Init();
+}
 
+void SCA_PropertySensor::Init()
+{
+	m_recentresult = false;
+	m_lastresult = m_invert?true:false;
+	m_reset = true;
 }
 
 void SCA_PropertySensor::PrecalculateRangeExpression()
 {
 		CParser pars;
+		//The context is needed to retrieve the property at runtime but it creates
+		//loop of references
 		pars.SetContext(this->AddRef());
 		STR_String checkstr = "(" + m_checkpropval + " <= " 
 							+ m_checkpropname + ") && ( " 
 							+ m_checkpropname + " <= " 
-							+ m_checkpropmaxval;
+							+ m_checkpropmaxval + ")";
 
 		m_range_expr = pars.ProcessText(checkstr);
 }
 
-
+// Forced deletion of precalculated range expression to break reference loop
+// Use this function when you know that you won't use the sensor anymore
+void SCA_PropertySensor::Delete()
+{
+	if (m_range_expr)
+	{
+		m_range_expr->Release();
+		m_range_expr = NULL;
+	}
+	Release();
+}
 
 CValue* SCA_PropertySensor::GetReplica()
 {
 	SCA_PropertySensor* replica = new SCA_PropertySensor(*this);
 	// m_range_expr must be recalculated on replica!
-	CValue::AddDataToReplica(replica);
+	replica->ProcessReplica();
+	replica->Init();
 
 	replica->m_range_expr = NULL;
 	if (replica->m_checktype==KX_PROPSENSOR_INTERVAL)
@@ -137,17 +156,18 @@ SCA_PropertySensor::~SCA_PropertySensor()
 
 
 
-bool SCA_PropertySensor::Evaluate(CValue* event)
+bool SCA_PropertySensor::Evaluate()
 {
 	bool result = CheckPropertyCondition();
+	bool reset = m_reset && m_level;
 	
+	m_reset = false;
 	if (m_lastresult!=result)
 	{
 		m_lastresult = result;
 		return true;
 	}
-
-	return false;
+	return (reset) ? true : false;
 }
 
 
@@ -164,22 +184,34 @@ bool	SCA_PropertySensor::CheckPropertyCondition()
 	case KX_PROPSENSOR_EQUAL:
 		{
 			CValue* orgprop = GetParent()->FindIdentifier(m_checkpropname);
-			if (orgprop)
+			if (!orgprop->IsError())
 			{
-				STR_String testprop = orgprop->GetText();
+				const STR_String& testprop = orgprop->GetText();
 				// Force strings to upper case, to avoid confusion in
 				// bool tests. It's stupid the prop's identity is lost
 				// on the way here...
-				if ((testprop == "TRUE") || (testprop == "FALSE")) {
-					STR_String checkprop = m_checkpropval;
-					checkprop.Upper();
-					result = (testprop == checkprop);
-				} else {
-					result = (orgprop->GetText() == m_checkpropval);
+				if ((&testprop == &CBoolValue::sTrueString) || (&testprop == &CBoolValue::sFalseString)) {
+					m_checkpropval.Upper();
 				}
-				orgprop->Release();
-
+				result = (testprop == m_checkpropval);
+				
+				/* Patch: floating point values cant use strings usefully since you can have "0.0" == "0.0000"
+				 * this could be made into a generic Value class function for comparing values with a string.
+				 */
+				if(result==false && dynamic_cast<CFloatValue *>(orgprop) != NULL) {
+					float f;
+					
+					if(EOF == sscanf(m_checkpropval.ReadPtr(), "%f", &f))
+					{
+						//error
+					} 
+					else {
+						result = (f == ((CFloatValue *)orgprop)->GetFloat());
+					}
+				}
+				/* end patch */
 			}
+			orgprop->Release();
 
 			if (reverse)
 				result = !result;
@@ -217,8 +249,8 @@ bool	SCA_PropertySensor::CheckPropertyCondition()
 					CValue* vallie = m_range_expr->Calculate();
 					if (vallie)
 					{
-						STR_String errtext = vallie->GetText();
-						if (errtext == "TRUE")
+						const STR_String& errtext = vallie->GetText();
+						if (&errtext == &CBoolValue::sTrueString)
 						{
 							result = true;
 						} else
@@ -244,15 +276,15 @@ bool	SCA_PropertySensor::CheckPropertyCondition()
 		{
 			CValue* orgprop = GetParent()->FindIdentifier(m_checkpropname);
 				
-			if (orgprop)
+			if (!orgprop->IsError())
 			{
 				if (m_previoustext != orgprop->GetText())
 				{
 					m_previoustext = orgprop->GetText();
 					result = true;
 				}
-				orgprop->Release();
 			}
+			orgprop->Release();
 
 			//cout << " \nSens:Prop:changed!"; /* need implementation here!!! */
 			break;
@@ -260,7 +292,16 @@ bool	SCA_PropertySensor::CheckPropertyCondition()
 	default:
 		; /* error */
 	}
-	m_recentresult=result;
+
+	//the concept of Edge and Level triggering has unwanted effect for KX_PROPSENSOR_CHANGED
+	//see Game Engine bugtracker [ #3809 ]
+	if (m_checktype != KX_PROPSENSOR_CHANGED)
+	{
+		m_recentresult=result;
+	} else
+	{
+		m_recentresult=result;//true;
+	}
 	return result;
 }
 
@@ -269,156 +310,80 @@ CValue* SCA_PropertySensor::FindIdentifier(const STR_String& identifiername)
 	return  GetParent()->FindIdentifier(identifiername);
 }
 
-bool SCA_PropertySensor::validValueForProperty(char *val, STR_String &prop)
-{
-	bool result = true;
-	/*  There is no type checking at this moment, unfortunately...           */
-	return result;
-}
+#ifdef WITH_PYTHON
 
 /* ------------------------------------------------------------------------- */
 /* Python functions                                                          */
 /* ------------------------------------------------------------------------- */
 
+int SCA_PropertySensor::validValueForProperty(void *self, const PyAttributeDef*)
+{
+	/*  If someone actually do type checking please make sure the 'max' and 'min'
+		are checked as well (currently they are calling the PrecalculateRangeExpression
+		function directly	*/
+
+	/*  There is no type checking at this moment, unfortunately...           */
+	return 0;
+}
+
+int SCA_PropertySensor::validValueForIntervalProperty(void *self, const PyAttributeDef*)
+{
+	SCA_PropertySensor*	sensor = reinterpret_cast<SCA_PropertySensor*>(self);
+
+	if (sensor->m_checktype==KX_PROPSENSOR_INTERVAL)
+	{
+		sensor->PrecalculateRangeExpression();
+	}
+	return 0;
+}
+
+int SCA_PropertySensor::modeChange(void *self, const PyAttributeDef* attrdef)
+{
+	SCA_PropertySensor*	sensor = reinterpret_cast<SCA_PropertySensor*>(self);
+
+	if (sensor->m_checktype==KX_PROPSENSOR_INTERVAL)
+	{
+		sensor->PrecalculateRangeExpression();
+	}
+	return 0;
+}
+
 /* Integration hooks ------------------------------------------------------- */
 PyTypeObject SCA_PropertySensor::Type = {
-	PyObject_HEAD_INIT(&PyType_Type)
-	0,
+	PyVarObject_HEAD_INIT(NULL, 0)
 	"SCA_PropertySensor",
-	sizeof(SCA_PropertySensor),
+	sizeof(PyObjectPlus_Proxy),
 	0,
-	PyDestructor,
-	0,
-	__getattr,
-	__setattr,
-	0, //&MyPyCompare,
-	__repr,
-	0, //&cvalue_as_number,
+	py_base_dealloc,
 	0,
 	0,
 	0,
-	0
-};
-
-PyParentObject SCA_PropertySensor::Parents[] = {
-	&SCA_PropertySensor::Type,
+	0,
+	py_base_repr,
+	0,0,0,0,0,0,0,0,0,
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+	0,0,0,0,0,0,0,
+	Methods,
+	0,
+	0,
 	&SCA_ISensor::Type,
-	&SCA_ILogicBrick::Type,
-	&CValue::Type,
-	NULL
+	0,0,0,0,0,0,
+	py_base_new
 };
 
 PyMethodDef SCA_PropertySensor::Methods[] = {
-	{"getType", (PyCFunction) SCA_PropertySensor::sPyGetType, METH_VARARGS, GetType_doc},
-	{"setType", (PyCFunction) SCA_PropertySensor::sPySetType, METH_VARARGS, SetType_doc},
-	{"getProperty", (PyCFunction) SCA_PropertySensor::sPyGetProperty, METH_VARARGS, GetProperty_doc},
-	{"setProperty", (PyCFunction) SCA_PropertySensor::sPySetProperty, METH_VARARGS, SetProperty_doc},
-	{"getValue", (PyCFunction) SCA_PropertySensor::sPyGetValue, METH_VARARGS, GetValue_doc},
-	{"setValue", (PyCFunction) SCA_PropertySensor::sPySetValue, METH_VARARGS, SetValue_doc},
 	{NULL,NULL} //Sentinel
 };
 
-PyObject* SCA_PropertySensor::_getattr(char* attr) {
-	_getattr_up(SCA_ISensor); /* implicit return! */
-}
+PyAttributeDef SCA_PropertySensor::Attributes[] = {
+	KX_PYATTRIBUTE_INT_RW_CHECK("mode",KX_PROPSENSOR_NODEF,KX_PROPSENSOR_MAX-1,false,SCA_PropertySensor,m_checktype,modeChange),
+	KX_PYATTRIBUTE_STRING_RW_CHECK("propName",0,100,false,SCA_PropertySensor,m_checkpropname,CheckProperty),
+	KX_PYATTRIBUTE_STRING_RW_CHECK("value",0,100,false,SCA_PropertySensor,m_checkpropval,validValueForProperty),
+	KX_PYATTRIBUTE_STRING_RW_CHECK("min",0,100,false,SCA_PropertySensor,m_checkpropval,validValueForIntervalProperty),
+	KX_PYATTRIBUTE_STRING_RW_CHECK("max",0,100,false,SCA_PropertySensor,m_checkpropmaxval,validValueForIntervalProperty),
+	{ NULL }	//Sentinel
+};
 
-/* 1. getType */
-char SCA_PropertySensor::GetType_doc[] = 
-"getType()\n"
-"\tReturns the type of check this sensor performs.\n";
-PyObject* SCA_PropertySensor::PyGetType(PyObject* self, PyObject* args, PyObject* kwds)
-{
-	return PyInt_FromLong(m_checktype);
-}
-
-/* 2. setType */
-char SCA_PropertySensor::SetType_doc[] = 
-"setType(type)\n"
-"\t- type: KX_PROPSENSOR_EQUAL, KX_PROPSENSOR_NOTEQUAL,\n"
-"\t        KX_PROPSENSOR_INTERVAL, KX_PROPSENSOR_CHANGED,\n"
-"\t        or KX_PROPSENSOR_EXPRESSION.\n"
-"\tSet the type of check to perform.\n";
-PyObject* SCA_PropertySensor::PySetType(PyObject* self, PyObject* args, PyObject* kwds) 
-{
-	int typeArg;
-	
-	if (!PyArg_ParseTuple(args, "i", &typeArg)) {
-		return NULL;
-	}
-	
-	if ( (typeArg > KX_PROPSENSOR_NODEF) 
-		 && (typeArg < KX_PROPSENSOR_MAX) ) {
-		m_checktype =  typeArg;
-	}
-	
-	Py_Return;
-}
-
-/* 3. getProperty */
-char SCA_PropertySensor::GetProperty_doc[] = 
-"getProperty()\n"
-"\tReturn the property with which the sensor operates.\n";
-PyObject* SCA_PropertySensor::PyGetProperty(PyObject* self, PyObject* args, PyObject* kwds) 
-{
-	return PyString_FromString(m_checkpropname);
-}
-
-/* 4. setProperty */
-char SCA_PropertySensor::SetProperty_doc[] = 
-"setProperty(name)\n"
-"\t- name: string\n"
-"\tSets the property with which to operate. If there is no property\n"
-"\tof this name, the call is ignored.\n";
-PyObject* SCA_PropertySensor::PySetProperty(PyObject* self, PyObject* args, PyObject* kwds) 
-{
-	/* We should query whether the name exists. Or should we create a prop   */
-	/* on the fly?                                                           */
-	char *propNameArg = NULL;
-
-	if (!PyArg_ParseTuple(args, "s", &propNameArg)) {
-		return NULL;
-	}
-
-	if (FindIdentifier(STR_String(propNameArg))) {
-		m_checkpropname = propNameArg;
-	} else {
-		; /* error: bad property name */
-	}
-
-	Py_Return;
-}
-
-/* 5. getValue */
-char SCA_PropertySensor::GetValue_doc[] = 
-"getValue()\n"
-"\tReturns the value with which the sensor operates.\n";
-PyObject* SCA_PropertySensor::PyGetValue(PyObject* self, PyObject* args, PyObject* kwds) 
-{
-	return PyString_FromString(m_checkpropval);
-}
-
-/* 6. setValue */
-char SCA_PropertySensor::SetValue_doc[] = 
-"setValue(value)\n"
-"\t- value: string\n"
-"\tSet the value with which the sensor operates. If the value\n"
-"\tis not compatible with the type of the property, the subsequent\n"
-"\t action is ignored.\n";
-PyObject* SCA_PropertySensor::PySetValue(PyObject* self, PyObject* args, PyObject* kwds) 
-{
-	/* Here, we need to check whether the value is 'valid' for this property.*/
-	/* We know that the property exists, or is NULL.                         */
-	char *propValArg = NULL;
-
-	if(!PyArg_ParseTuple(args, "s", &propValArg)) {
-		return NULL;
-	}
-
-	if (validValueForProperty(propValArg, m_checkpropname)) {
-		m_checkpropval = propValArg;
-	}	
-
-	Py_Return;
-}
+#endif // WITH_PYTHON
 
 /* eof */
