@@ -1,15 +1,10 @@
-/**
- * $Id: GPG_Application.cpp,v 1.12 2004/08/10 11:34:11 kester Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,31 +22,28 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  * GHOST Blender Player application implementation file.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+/** \file gameengine/GamePlayer/ghost/GPG_Application.cpp
+ *  \ingroup player
+ */
+
 
 #ifdef WIN32
 	#pragma warning (disable:4786) // suppress stl-MSVC debug info warning
 	#include <windows.h>
 #endif
 
-#ifdef __APPLE__
-#include <OpenGL/gl.h>
-#include <OpenGL/glu.h>
-#else
-#include <GL/gl.h>
-#include <GL/glu.h>
-#endif
+#include "GL/glew.h"
+#include "GPU_extensions.h"
 
 #include "GPG_Application.h"
 
 #include <iostream>
-#include <assert.h>
+#include <MT_assert.h>
+#include <stdlib.h>
 
 /**********************************
  * Begin Blender include block
@@ -62,6 +54,11 @@ extern "C"
 #endif  // __cplusplus
 #include "BLI_blenlib.h"
 #include "BLO_readfile.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
+#include "BKE_sound.h"
+#include "IMB_imbuf.h"
+#include "DNA_scene_types.h"
 #ifdef __cplusplus
 }
 #endif // __cplusplus
@@ -70,22 +67,23 @@ extern "C"
  **********************************/
 
 
-#include "SYS_System.h"
+#include "BL_System.h"
 #include "KX_KetsjiEngine.h"
 
 // include files needed by "KX_BlenderSceneConverter.h"
-#include "GEN_Map.h"
+#include "CTR_Map.h"
 #include "SCA_IActuator.h"
 #include "RAS_MeshObject.h"
 #include "RAS_OpenGLRasterizer.h"
 #include "RAS_VAOpenGLRasterizer.h"
+#include "RAS_ListRasterizer.h"
 #include "RAS_GLExtensionManager.h"
 #include "KX_PythonInit.h"
 #include "KX_PyConstraintBinding.h"
+#include "BL_Material.h" // MAXTEX
 
 #include "KX_BlenderSceneConverter.h"
 #include "NG_LoopBackNetworkDeviceInterface.h"
-#include "SND_DeviceManager.h"
 
 #include "GPC_MouseDevice.h"
 #include "GPC_RenderTools.h"
@@ -101,15 +99,15 @@ extern "C"
 #include "GHOST_IWindow.h"
 #include "GHOST_Rect.h"
 
-
 static void frameTimerProc(GHOST_ITimerTask* task, GHOST_TUns64 time);
 
 static GHOST_ISystem* fSystem = 0;
 static const int kTimerFreq = 10;
 
-GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR_String startSceneName)
-	: m_startSceneName(startSceneName), 
-	  m_maggie(maggie), 
+GPG_Application::GPG_Application(GHOST_ISystem* system)
+	: m_startSceneName(""), 
+	  m_startScene(0),
+	  m_maggie(0),
 	  m_exitRequested(0),
 	  m_system(system), 
 	  m_mainWindow(0), 
@@ -117,6 +115,7 @@ GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR
 	  m_cursor(GHOST_kStandardCursorFirstCursor),
 	  m_engineInitialized(0), 
 	  m_engineRunning(0), 
+	  m_isEmbedded(false),
 	  m_ketsjiengine(0),
 	  m_kxsystem(0), 
 	  m_keyboard(0), 
@@ -125,8 +124,11 @@ GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR
 	  m_rendertools(0), 
 	  m_rasterizer(0), 
 	  m_sceneconverter(0),
-	  m_networkdevice(0), 
-	  m_audiodevice(0)
+	  m_networkdevice(0),
+	  m_blendermat(0),
+	  m_blenderglslmat(0),
+	  m_pyGlobalDictString(0),
+	  m_pyGlobalDictString_Length(0)
 {
 	fSystem = system;
 }
@@ -135,27 +137,176 @@ GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR
 
 GPG_Application::~GPG_Application(void)
 {
+	if(m_pyGlobalDictString) {
+		delete [] m_pyGlobalDictString;
+		m_pyGlobalDictString = 0;
+		m_pyGlobalDictString_Length = 0;
+	}
+
 	exitEngine();
 	fSystem->disposeWindow(m_mainWindow);
 }
 
 
 
-bool GPG_Application::SetGameEngineData(struct Main *maggie, STR_String startSceneName)
+bool GPG_Application::SetGameEngineData(struct Main* maggie, Scene *scene, GlobalSettings *gs, int argc, char **argv)
 {
 	bool result = false;
 
-	if (maggie != NULL && startSceneName != "")
+	if (maggie != NULL && scene != NULL)
 	{
+// XXX		G.scene = scene;
 		m_maggie = maggie;
-		m_startSceneName = startSceneName;
+		m_startSceneName = scene->id.name+2;
+		m_startScene = scene;
 		result = true;
 	}
+	
+	/* Python needs these */
+	m_argc= argc;
+	m_argv= argv;
+
+	/* Global Settings */
+	m_globalSettings= gs;
 
 	return result;
 }
 
 
+#ifdef WIN32
+#define SCR_SAVE_MOUSE_MOVE_THRESHOLD 15
+
+static HWND found_ghost_window_hwnd;
+static GHOST_IWindow* ghost_window_to_find;
+static WNDPROC ghost_wnd_proc;
+static POINT scr_save_mouse_pos;
+
+static LRESULT CALLBACK screenSaverWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
+{
+	BOOL close = FALSE;
+	switch (uMsg)
+	{
+		case WM_MOUSEMOVE:
+		{ 
+			POINT pt; 
+			GetCursorPos(&pt);
+			LONG dx = scr_save_mouse_pos.x - pt.x;
+			LONG dy = scr_save_mouse_pos.y - pt.y;
+			if (abs(dx) > SCR_SAVE_MOUSE_MOVE_THRESHOLD
+			        || abs(dy) > SCR_SAVE_MOUSE_MOVE_THRESHOLD)
+			{
+				close = TRUE;
+			}
+			scr_save_mouse_pos = pt;
+			break;
+		}
+		case WM_LBUTTONDOWN: 
+		case WM_MBUTTONDOWN: 
+		case WM_RBUTTONDOWN: 
+		case WM_KEYDOWN:
+			close = TRUE;
+	}
+	if (close)
+		PostMessage(hwnd,WM_CLOSE,0,0);
+	return CallWindowProc(ghost_wnd_proc, hwnd, uMsg, wParam, lParam);
+}
+
+BOOL CALLBACK findGhostWindowHWNDProc(HWND hwnd, LPARAM lParam)
+{
+	GHOST_IWindow *p = (GHOST_IWindow*) GetWindowLongPtr(hwnd, GWLP_USERDATA);
+	BOOL ret = TRUE;
+	if (p == ghost_window_to_find)
+	{
+		found_ghost_window_hwnd = hwnd;
+		ret = FALSE;
+	}
+	return ret;
+}
+
+static HWND findGhostWindowHWND(GHOST_IWindow* window)
+{
+	found_ghost_window_hwnd = NULL;
+	ghost_window_to_find = window;
+	EnumWindows(findGhostWindowHWNDProc, NULL);
+	return found_ghost_window_hwnd;
+}
+
+bool GPG_Application::startScreenSaverPreview(
+	HWND parentWindow,
+	const bool stereoVisual,
+	const int stereoMode,
+	const GHOST_TUns16 samples)
+{
+	bool success = false;
+
+	RECT rc;
+	if (GetWindowRect(parentWindow, &rc))
+	{
+		int windowWidth = rc.right - rc.left;
+		int windowHeight = rc.bottom - rc.top;
+		STR_String title = "";
+							
+		m_mainWindow = fSystem->createWindow(title, 0, 0, windowWidth, windowHeight, GHOST_kWindowStateMinimized,
+			GHOST_kDrawingContextTypeOpenGL, stereoVisual, samples);
+		if (!m_mainWindow) {
+			printf("error: could not create main window\n");
+			exit(-1);
+		}
+
+		HWND ghost_hwnd = findGhostWindowHWND(m_mainWindow);
+		if (!ghost_hwnd) {
+			printf("error: could find main window\n");
+			exit(-1);
+		}
+
+		SetParent(ghost_hwnd, parentWindow);
+		LONG style = GetWindowLong(ghost_hwnd, GWL_STYLE);
+		LONG exstyle = GetWindowLong(ghost_hwnd, GWL_EXSTYLE);
+
+		RECT adjrc = { 0, 0, windowWidth, windowHeight };
+		AdjustWindowRectEx(&adjrc, style, FALSE, exstyle);
+
+		style = (style & (~(WS_POPUP|WS_OVERLAPPEDWINDOW|WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_THICKFRAME|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|WS_TILEDWINDOW ))) | WS_CHILD;
+		SetWindowLong(ghost_hwnd, GWL_STYLE, style);
+		SetWindowPos(ghost_hwnd, NULL, adjrc.left, adjrc.top, 0, 0, SWP_NOZORDER|SWP_NOSIZE|SWP_NOACTIVATE);
+
+		/* Check the size of the client rectangle of the window and resize the window
+		 * so that the client rectangle has the size requested.
+		 */
+		m_mainWindow->setClientSize(windowWidth, windowHeight);
+
+		success = initEngine(m_mainWindow, stereoMode);
+		if (success) {
+			success = startEngine();
+		}
+
+	}
+	return success;
+}
+
+bool GPG_Application::startScreenSaverFullScreen(
+		int width,
+		int height,
+		int bpp,int frequency,
+		const bool stereoVisual,
+		const int stereoMode,
+		const GHOST_TUns16 samples)
+{
+	bool ret = startFullScreen(width, height, bpp, frequency, stereoVisual, stereoMode, samples);
+	if (ret)
+	{
+		HWND ghost_hwnd = findGhostWindowHWND(m_mainWindow);
+		if (ghost_hwnd != NULL)
+		{
+			GetCursorPos(&scr_save_mouse_pos);
+			ghost_wnd_proc = (WNDPROC) GetWindowLongPtr(ghost_hwnd, GWLP_WNDPROC);
+			SetWindowLongPtr(ghost_hwnd,GWLP_WNDPROC, (uintptr_t) screenSaverWindowProc);
+		}
+	}
+	return ret;
+}
+
+#endif
 
 bool GPG_Application::startWindow(STR_String& title,
 	int windowLeft,
@@ -163,13 +314,14 @@ bool GPG_Application::startWindow(STR_String& title,
 	int windowWidth,
 	int windowHeight,
 	const bool stereoVisual,
-	const int stereoMode)
+	const int stereoMode,
+	const GHOST_TUns16 samples)
 {
 	bool success;
 	// Create the main window
 	//STR_String title ("Blender Player - GHOST");
 	m_mainWindow = fSystem->createWindow(title, windowLeft, windowTop, windowWidth, windowHeight, GHOST_kWindowStateNormal,
-		GHOST_kDrawingContextTypeOpenGL, stereoVisual);
+		GHOST_kDrawingContextTypeOpenGL, stereoVisual, samples);
 	if (!m_mainWindow) {
 		printf("error: could not create main window\n");
 		exit(-1);
@@ -188,6 +340,29 @@ bool GPG_Application::startWindow(STR_String& title,
 	return success;
 }
 
+bool GPG_Application::startEmbeddedWindow(STR_String& title,
+	const GHOST_TEmbedderWindowID parentWindow, 
+	const bool stereoVisual, 
+	const int stereoMode,
+	const GHOST_TUns16 samples) {
+	GHOST_TWindowState state = GHOST_kWindowStateNormal;
+	if (parentWindow != 0)
+		state = GHOST_kWindowStateEmbedded;
+	m_mainWindow = fSystem->createWindow(title, 0, 0, 0, 0, state,
+		GHOST_kDrawingContextTypeOpenGL, stereoVisual, samples, parentWindow);
+
+	if (!m_mainWindow) {
+		printf("error: could not create main window\n");
+		exit(-1);
+	}
+	m_isEmbedded = true;
+
+	bool success = initEngine(m_mainWindow, stereoMode);
+	if (success) {
+		success = startEngine();
+	}
+	return success;
+}
 
 
 bool GPG_Application::startFullScreen(
@@ -195,7 +370,8 @@ bool GPG_Application::startFullScreen(
 		int height,
 		int bpp,int frequency,
 		const bool stereoVisual,
-		const int stereoMode)
+		const int stereoMode,
+		const GHOST_TUns16 samples)
 {
 	bool success;
 	// Create the main window
@@ -207,6 +383,7 @@ bool GPG_Application::startFullScreen(
 
 	fSystem->beginFullScreen(setting, &m_mainWindow, stereoVisual);
 	m_mainWindow->setCursorVisibility(false);
+	m_mainWindow->setState(GHOST_kWindowStateFullScreen);
 
 	success = initEngine(m_mainWindow, stereoMode);
 	if (success) {
@@ -217,6 +394,7 @@ bool GPG_Application::startFullScreen(
 
 
 
+
 bool GPG_Application::StartGameEngine(int stereoMode)
 {
 	bool success = initEngine(m_mainWindow, stereoMode);
@@ -271,6 +449,7 @@ bool GPG_Application::processEvent(GHOST_IEvent* event)
 
 
 		case GHOST_kEventWindowClose:
+		case GHOST_kEventQuit:
 			m_exitRequested = KX_EXIT_REQUEST_OUTSIDE;
 			break;
 
@@ -295,10 +474,12 @@ bool GPG_Application::processEvent(GHOST_IEvent* event)
 					m_exitRequested = m_ketsjiengine->GetExitCode();
 					
 					// kick the engine
-					m_ketsjiengine->NextFrame();
-					
-					// render the frame
-					m_ketsjiengine->Render();
+					bool renderFrame = m_ketsjiengine->NextFrame();
+					if (renderFrame)
+					{
+						// render the frame
+						m_ketsjiengine->Render();
+					}
 				}
 				m_exitString = m_ketsjiengine->GetExitString();
 			}
@@ -331,6 +512,12 @@ int GPG_Application::getExitRequested(void)
 }
 
 
+GlobalSettings* GPG_Application::getGlobalSettings(void)
+{
+	return m_ketsjiengine->GetGlobalSettings();
+}
+
+
 
 const STR_String& GPG_Application::getExitString(void)
 {
@@ -343,7 +530,8 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 {
 	if (!m_engineInitialized)
 	{
-		bgl::InitExtensions(1);
+		GPU_extensions_init();
+		bgl::InitExtensions(true);
 
 		// get and set the preferences
 		SYS_SystemHandle syshandle = SYS_GetSystem();
@@ -352,25 +540,56 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 		
 		// SYS_WriteCommandLineInt(syshandle, "fixedtime", 0);
 		// SYS_WriteCommandLineInt(syshandle, "vertexarrays",1);		
+		GameData *gm= &m_startScene->gm;
 		bool properties	= (SYS_GetCommandLineInt(syshandle, "show_properties", 0) != 0);
 		bool profile = (SYS_GetCommandLineInt(syshandle, "show_profile", 0) != 0);
+		bool fixedFr = (gm->flag & GAME_ENABLE_ALL_FRAMES);
+
+		bool showPhysics = (gm->flag & GAME_SHOW_PHYSICS);
+		SYS_WriteCommandLineInt(syshandle, "show_physics", showPhysics);
+
+		bool fixed_framerate= (SYS_GetCommandLineInt(syshandle, "fixed_framerate", fixedFr) != 0);
 		bool frameRate = (SYS_GetCommandLineInt(syshandle, "show_framerate", 0) != 0);
-		bool useVertexArrays = SYS_GetCommandLineInt(syshandle,"vertexarrays",1) != 0;
+		bool useLists = (SYS_GetCommandLineInt(syshandle, "displaylists", gm->flag & GAME_DISPLAY_LISTS) != 0);
+		bool nodepwarnings = (SYS_GetCommandLineInt(syshandle, "ignore_deprecation_warnings", 1) != 0);
+		bool restrictAnimFPS = gm->flag & GAME_RESTRICT_ANIM_UPDATES;
+
+		if(GLEW_ARB_multitexture && GLEW_VERSION_1_1)
+			m_blendermat = (SYS_GetCommandLineInt(syshandle, "blender_material", 1) != 0);
+
+		if(GPU_glsl_support())
+			m_blenderglslmat = (SYS_GetCommandLineInt(syshandle, "blender_glsl_material", 1) != 0);
+		else if(m_globalSettings->matmode == GAME_MAT_GLSL)
+			m_blendermat = false;
+
 		// create the canvas, rasterizer and rendertools
 		m_canvas = new GPG_Canvas(window);
 		if (!m_canvas)
 			return false;
 				
-		m_canvas->Init();				
+		m_canvas->Init();
+		if (gm->flag & GAME_SHOW_MOUSE)
+			m_canvas->SetMouseState(RAS_ICanvas::MOUSE_NORMAL);				
+
 		m_rendertools = new GPC_RenderTools();
 		if (!m_rendertools)
 			goto initFailed;
-					
-		if (useVertexArrays && bgl::QueryVersion(1, 1))
+		
+		if(useLists) {
+			if(GLEW_VERSION_1_1)
+				m_rasterizer = new RAS_ListRasterizer(m_canvas, true);
+			else
+				m_rasterizer = new RAS_ListRasterizer(m_canvas);
+		}
+		else if (GLEW_VERSION_1_1)
 			m_rasterizer = new RAS_VAOpenGLRasterizer(m_canvas);
 		else
 			m_rasterizer = new RAS_OpenGLRasterizer(m_canvas);
+
+		/* Stereo parameters - Eye Separation from the UI - stereomode from the command-line/UI */
 		m_rasterizer->SetStereoMode((RAS_IRasterizer::StereoMode) stereoMode);
+		m_rasterizer->SetEyeSeparation(m_startScene->gm.eyeseparation);
+		
 		if (!m_rasterizer)
 			goto initFailed;
 						
@@ -388,13 +607,8 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 		if (!m_networkdevice)
 			goto initFailed;
 			
-		// get an audiodevice
-		SND_DeviceManager::Subscribe();
-		m_audiodevice = SND_DeviceManager::Instance();
-		if (!m_audiodevice)
-			goto initFailed;
-		m_audiodevice->UseCD();
-		
+		sound_init(m_maggie);
+
 		// create a ketsjisystem (only needed for timing and stuff)
 		m_kxsystem = new GPG_System (m_system);
 		if (!m_kxsystem)
@@ -410,20 +624,28 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 		m_ketsjiengine->SetCanvas(m_canvas);
 		m_ketsjiengine->SetRenderTools(m_rendertools);
 		m_ketsjiengine->SetRasterizer(m_rasterizer);
-		m_ketsjiengine->SetNetworkDevice(m_networkdevice);
-		m_ketsjiengine->SetAudioDevice(m_audiodevice);
-		m_ketsjiengine->SetTimingDisplay(frameRate, false, false);
 
-		m_ketsjiengine->SetUseFixedTime(false);
+		m_ketsjiengine->SetTimingDisplay(frameRate, false, false);
+#ifdef WITH_PYTHON
+		CValue::SetDeprecationWarnings(nodepwarnings);
+#else
+		(void)nodepwarnings;
+#endif
+
+		m_ketsjiengine->SetUseFixedTime(fixed_framerate);
 		m_ketsjiengine->SetTimingDisplay(frameRate, profile, properties);
+		m_ketsjiengine->SetRestrictAnimationFPS(restrictAnimFPS);
+
+		//set the global settings (carried over if restart/load new files)
+		m_ketsjiengine->SetGlobalSettings(m_globalSettings);
 
 		m_engineInitialized = true;
 	}
 
 	return m_engineInitialized;
 initFailed:
+	sound_exit();
 	delete m_kxsystem;
-	delete m_audiodevice;
 	delete m_networkdevice;
 	delete m_mouse;
 	delete m_keyboard;
@@ -436,7 +658,6 @@ initFailed:
 	m_keyboard = NULL;
 	m_mouse = NULL;
 	m_networkdevice = NULL;
-	m_audiodevice = NULL;
 	m_kxsystem = NULL;
 	return false;
 }
@@ -453,7 +674,7 @@ bool GPG_Application::startEngine(void)
 	/*
 	m_canvas->SetBannerDisplayEnabled(true);	
 	Camera* cam;
-	cam = (Camera*)G.scene->camera->data;
+	cam = (Camera*)scene->camera->data;
 	if (cam) {
 	if (((cam->flag) & 48)==48) {
 	m_canvas->SetBannerDisplayEnabled(false);
@@ -471,30 +692,38 @@ bool GPG_Application::startEngine(void)
 	{
 		STR_String startscenename = m_startSceneName.Ptr();
 		m_ketsjiengine->SetSceneConverter(m_sceneconverter);
-		
+
 		//	if (always_use_expand_framing)
 		//		sceneconverter->SetAlwaysUseExpandFraming(true);
-		
+		if(m_blendermat && (m_globalSettings->matmode != GAME_MAT_TEXFACE))
+			m_sceneconverter->SetMaterials(true);
+		if(m_blenderglslmat && (m_globalSettings->matmode == GAME_MAT_GLSL))
+			m_sceneconverter->SetGLSLMaterials(true);
 
 		KX_Scene* startscene = new KX_Scene(m_keyboard,
 			m_mouse,
 			m_networkdevice,
-			m_audiodevice,
-			startscenename);
-		
-		// some python things
-		PyObject* m_dictionaryobject = initGamePlayerPythonScripting("Ketsji", psl_Lowest);
-		m_ketsjiengine->SetPythonDictionary(m_dictionaryobject);
-		initRasterizer(m_rasterizer, m_canvas);
-		initGameLogic(startscene);
-		initGameKeys();
-		initPythonConstraintBinding();
-		
-		m_sceneconverter->ConvertScene(
 			startscenename,
+			m_startScene,
+			m_canvas);
+		
+#ifdef WITH_PYTHON
+			// some python things
+			PyObject *gameLogic, *gameLogic_keys;
+			setupGamePython(m_ketsjiengine, startscene, m_maggie, NULL, &gameLogic, &gameLogic_keys, m_argc, m_argv);
+#endif // WITH_PYTHON
+
+		//initialize Dome Settings
+		if(m_startScene->gm.stereoflag == STEREO_DOME)
+			m_ketsjiengine->InitDome(m_startScene->gm.dome.res, m_startScene->gm.dome.mode, m_startScene->gm.dome.angle, m_startScene->gm.dome.resbuf, m_startScene->gm.dome.tilt, m_startScene->gm.dome.warptext);
+
+#ifdef WITH_PYTHON
+		// Set the GameLogic.globalDict from marshal'd data, so we can
+		// load new blend files and keep data in GameLogic.globalDict
+		loadGamePythonConfig(m_pyGlobalDictString, m_pyGlobalDictString_Length);
+#endif		
+		m_sceneconverter->ConvertScene(
 			startscene,
-			m_dictionaryobject,
-			m_keyboard,
 			m_rendertools,
 			m_canvas);
 		m_ketsjiengine->AddScene(startscene);
@@ -504,9 +733,14 @@ bool GPG_Application::startEngine(void)
 			m_frameTimer = m_system->installTimer(0, kTimerFreq, frameTimerProc, m_mainWindow);
 		}
 		m_rasterizer->Init();
-		m_ketsjiengine->StartEngine();
+		m_ketsjiengine->StartEngine(true);
 		m_engineRunning = true;
 		
+		// Set the animation playback rate for ipo's and actions
+		// the framerate below should patch with FPS macro defined in blendef.h
+		// Could be in StartEngine set the framerate, we need the scene to do this
+		Scene *scene= startscene->GetBlenderScene(); // needed for macro
+		m_ketsjiengine->SetAnimFrameRate(FPS);
 	}
 	
 	if (!m_engineRunning)
@@ -520,8 +754,21 @@ bool GPG_Application::startEngine(void)
 
 void GPG_Application::stopEngine()
 {
+#ifdef WITH_PYTHON
+	// GameLogic.globalDict gets converted into a buffer, and sorted in
+	// m_pyGlobalDictString so we can restore after python has stopped
+	// and started between .blend file loads.
+	if(m_pyGlobalDictString) {
+		delete [] m_pyGlobalDictString;
+		m_pyGlobalDictString = 0;
+	}
+
+	m_pyGlobalDictString_Length = saveGamePythonConfig(&m_pyGlobalDictString);
+	
 	// when exiting the mainloop
 	exitGamePythonScripting();
+#endif
+	
 	m_ketsjiengine->StopEngine();
 	m_networkdevice->Disconnect();
 
@@ -539,6 +786,7 @@ void GPG_Application::stopEngine()
 
 void GPG_Application::exitEngine()
 {
+	sound_exit();
 	if (m_ketsjiengine)
 	{
 		stopEngine();
@@ -550,11 +798,6 @@ void GPG_Application::exitEngine()
 		delete m_kxsystem;
 		m_kxsystem = 0;
 	}
-	if (m_audiodevice)
-	{
-		SND_DeviceManager::Unsubscribe();
-		m_audiodevice = 0;
-	}
 	if (m_networkdevice)
 	{
 		delete m_networkdevice;
@@ -586,6 +829,8 @@ void GPG_Application::exitEngine()
 		m_canvas = 0;
 	}
 
+	GPU_extensions_exit();
+
 	m_exitRequested = 0;
 	m_engineInitialized = false;
 }
@@ -593,7 +838,7 @@ void GPG_Application::exitEngine()
 bool GPG_Application::handleWheel(GHOST_IEvent* event)
 {
 	bool handled = false;
-	assert(event);
+	MT_assert(event);
 	if (m_mouse) 
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
@@ -612,7 +857,7 @@ bool GPG_Application::handleWheel(GHOST_IEvent* event)
 bool GPG_Application::handleButton(GHOST_IEvent* event, bool isDown)
 {
 	bool handled = false;
-	assert(event);
+	MT_assert(event);
 	if (m_mouse) 
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
@@ -641,7 +886,7 @@ bool GPG_Application::handleButton(GHOST_IEvent* event, bool isDown)
 bool GPG_Application::handleCursorMove(GHOST_IEvent* event)
 {
 	bool handled = false;
-	assert(event);
+	MT_assert(event);
 	if (m_mouse && m_mainWindow)
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
@@ -658,16 +903,17 @@ bool GPG_Application::handleCursorMove(GHOST_IEvent* event)
 bool GPG_Application::handleKey(GHOST_IEvent* event, bool isDown)
 {
 	bool handled = false;
-	assert(event);
+	MT_assert(event);
 	if (m_keyboard)
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
 		GHOST_TEventKeyData* keyData = static_cast<GHOST_TEventKeyData*>(eventData);
-		if (fSystem->getFullScreen()) {
-			if (keyData->key == GHOST_kKeyEsc) {
+		//no need for this test
+		//if (fSystem->getFullScreen()) {
+			if (keyData->key == GHOST_kKeyEsc && !m_keyboard->m_hookesc && !m_isEmbedded) {
 				m_exitRequested = KX_EXIT_REQUEST_OUTSIDE;
 			}
-		}
+		//}
 		m_keyboard->ConvertEvent(keyData->key, isDown);
 		handled = true;
 	}
