@@ -1,15 +1,10 @@
-/**
- * $Id: GPG_Application.h,v 1.7 2004/05/02 23:45:03 kester Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,17 +22,26 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
- * GHOST Blender Player application declaration file.
+ * ***** END GPL LICENSE BLOCK *****
+ */
+
+/** \file GPG_Application.h
+ *  \ingroup player
+ *  \brief GHOST Blender Player application declaration file.
  */
 
 #include "GHOST_IEventConsumer.h"
 #include "STR_String.h"
 
+#ifdef WIN32
+#include <wtypes.h>
+#endif
+
+#include "KX_KetsjiEngine.h"
+
 class KX_KetsjiEngine;
 class KX_ISceneConverter;
 class NG_LoopBackNetworkDeviceInterface;
-class SND_IAudioDevice;
 class RAS_IRasterizer;
 class GHOST_IEvent;
 class GHOST_ISystem;
@@ -49,20 +53,28 @@ class GPG_Canvas;
 class GPG_KeyboardDevice;
 class GPG_System;
 struct Main;
+struct Scene;
 
 class GPG_Application : public GHOST_IEventConsumer
 {
 public:
-	GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR_String startSceneName);
+	GPG_Application(GHOST_ISystem* system);
 	~GPG_Application(void);
 
-			bool SetGameEngineData(struct Main *maggie, STR_String startSceneName);
+			bool SetGameEngineData(struct Main* maggie, struct Scene* scene, GlobalSettings* gs, int argc, char** argv);
 			bool startWindow(STR_String& title, int windowLeft, int windowTop, int windowWidth, int windowHeight,
-			const bool stereoVisual, const int stereoMode);
-			bool startFullScreen(int width, int height, int bpp, int frequency, const bool stereoVisual, const int stereoMode);
+			const bool stereoVisual, const int stereoMode, const GHOST_TUns16 samples=0);
+			bool startFullScreen(int width, int height, int bpp, int frequency, const bool stereoVisual, const int stereoMode, const GHOST_TUns16 samples=0);
+			bool startEmbeddedWindow(STR_String& title, const GHOST_TEmbedderWindowID parent_window, const bool stereoVisual, const int stereoMode, const GHOST_TUns16 samples=0);
+#ifdef WIN32
+			bool startScreenSaverFullScreen(int width, int height, int bpp, int frequency, const bool stereoVisual, const int stereoMode, const GHOST_TUns16 samples=0);
+			bool startScreenSaverPreview(HWND parentWindow,	const bool stereoVisual, const int stereoMode, const GHOST_TUns16 samples=0);
+#endif
+
 	virtual	bool processEvent(GHOST_IEvent* event);
 			int getExitRequested(void);
 			const STR_String& getExitString(void);
+			GlobalSettings* getGlobalSettings(void);
 			bool StartGameEngine(int stereoMode);
 			void StopGameEngine();
 
@@ -94,11 +106,14 @@ protected:
 
 	/* The game data */
 	STR_String				m_startSceneName;
+	struct Scene*			m_startScene;
 	struct Main*			m_maggie;
 
 	/* Exit state. */
 	int						m_exitRequested;
 	STR_String				m_exitString;
+	GlobalSettings*	m_globalSettings;
+
 	/* GHOST system abstraction. */
 	GHOST_ISystem*			m_system;
 	/* Main window. */
@@ -112,6 +127,8 @@ protected:
 	bool m_engineInitialized;
 	/** Engine state. */
 	bool m_engineRunning;
+	/** Running on embedded window */
+	bool m_isEmbedded;
 
 	/** the gameengine itself */
 	KX_KetsjiEngine* m_ketsjiengine;
@@ -131,7 +148,19 @@ protected:
 	KX_ISceneConverter* m_sceneconverter;
 	/** Network interface. */
 	NG_LoopBackNetworkDeviceInterface* m_networkdevice;
-	/** Sound device. */
-	SND_IAudioDevice* m_audiodevice;
+
+	bool m_blendermat;
+	bool m_blenderglslmat;
+	
+	/*
+	 * GameLogic.globalDict as a string so that loading new blend files can use the same dict.
+	 * Do this because python starts/stops when loading blend files.
+	 */
+	char* m_pyGlobalDictString;
+	int m_pyGlobalDictString_Length;
+	
+	/* argc and argv need to be passed on to python */
+	int		m_argc;
+	char**	m_argv;
 };
 
