@@ -1,17 +1,13 @@
-/**
+/*
  * KX_CameraActuator.cpp
  *
- * $Id: KX_CameraActuator.cpp,v 1.6 2004/10/16 11:41:48 kester Exp $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -29,21 +25,22 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  *
  */
 
+/** \file gameengine/Ketsji/KX_CameraActuator.cpp
+ *  \ingroup ketsji
+ */
+
+
 #include "KX_CameraActuator.h"
 #include <iostream>
 #include <math.h>
+#include <float.h>
 #include "KX_GameObject.h"
 
-STR_String KX_CameraActuator::X_AXIS_STRING = "x";
-STR_String KX_CameraActuator::Y_AXIS_STRING = "y";
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "PyObjectPlus.h" 
 
 
 /* ------------------------------------------------------------------------- */
@@ -52,26 +49,29 @@ STR_String KX_CameraActuator::Y_AXIS_STRING = "y";
 
 KX_CameraActuator::KX_CameraActuator(
 	SCA_IObject* gameobj, 
-	const CValue *obj,
-	MT_Scalar hght,
-	MT_Scalar minhght,
-	MT_Scalar maxhght,
+	SCA_IObject *obj,
+	float hght,
+	float minhght,
+	float maxhght,
 	bool  xytog,
-	PyTypeObject* T
+	float damping
 ): 
-	SCA_IActuator(gameobj, T),
+	SCA_IActuator(gameobj, KX_ACT_CAMERA),
 	m_ob (obj),
 	m_height (hght),
 	m_minHeight (minhght),
 	m_maxHeight (maxhght),
-	m_x (xytog)
+	m_x (xytog),
+	m_damping (damping)
 {
-	// nothing to do
+	if (m_ob)
+		m_ob->RegisterActuator(this);
 }
 
 KX_CameraActuator::~KX_CameraActuator()
 {
-	//nothing to do
+	if (m_ob)
+		m_ob->UnregisterActuator(this);
 }
 
 	CValue* 
@@ -80,22 +80,47 @@ GetReplica(
 ) {
 	KX_CameraActuator* replica = new KX_CameraActuator(*this);
 	replica->ProcessReplica();
-	// this will copy properties and so on...
-	CValue::AddDataToReplica(replica);
 	return replica;
 };
 
+void KX_CameraActuator::ProcessReplica()
+{
+	if (m_ob)
+		m_ob->RegisterActuator(this);
+	SCA_IActuator::ProcessReplica();
+}
+
+bool KX_CameraActuator::UnlinkObject(SCA_IObject* clientobj)
+{
+	if (clientobj == m_ob)
+	{
+		// this object is being deleted, we cannot continue to track it.
+		m_ob = NULL;
+		return true;
+	}
+	return false;
+}
 
 
+void KX_CameraActuator::Relink(CTR_Map<CTR_HashedPtr, void*> *obj_map)
+{
+	void **h_obj = (*obj_map)[m_ob];
+	if (h_obj) {
+		if (m_ob)
+			m_ob->UnregisterActuator(this);
+		m_ob = (SCA_IObject*)(*h_obj);
+		m_ob->RegisterActuator(this);
+	}
+}
 
 /* three functions copied from blender arith... don't know if there's an equivalent */
 
-float Normalise(float *n)
+static float Kx_Normalize(float *n)
 {
 	float d;
 	
 	d= n[0]*n[0]+n[1]*n[1]+n[2]*n[2];
-	/* FLT_EPSILON is too large! A larger value causes normalise errors in a scaled down utah teapot */
+	/* FLT_EPSILON is too large! A larger value causes normalize errors in a scaled down utah teapot */
 	if(d>0.0000000000001) {
 		d= sqrt(d);
 
@@ -109,7 +134,7 @@ float Normalise(float *n)
 	return d;
 }
 
-void Crossf(float *c, float *a, float *b)
+static void Kx_Crossf(float *c, float *a, float *b)
 {
 	c[0] = a[1] * b[2] - a[2] * b[1];
 	c[1] = a[2] * b[0] - a[0] * b[2];
@@ -117,7 +142,7 @@ void Crossf(float *c, float *a, float *b)
 }
 
 
-void VecUpMat3(float *vec, float mat[][3], short axis)
+static void Kx_VecUpMat3(float *vec, float mat[][3], short axis)
 {
 
 	// Construct a camera matrix s.t. the specified axis
@@ -159,23 +184,40 @@ void VecUpMat3(float *vec, float mat[][3], short axis)
 	mat[coz][0]= vec[0];
 	mat[coz][1]= vec[1];
 	mat[coz][2]= vec[2];
-	Normalise((float *)mat[coz]);
+	if (Kx_Normalize((float *)mat[coz]) == 0.f) {
+		/* this is a very abnormal situation: the camera has reach the object center exactly
+		   We will choose a completely arbitrary direction */
+		mat[coz][0] = 1.0f;
+		mat[coz][1] = 0.0f;
+		mat[coz][2] = 0.0f;
+	}
 	
 	inp= mat[coz][2];
 	mat[coy][0]= - inp*mat[coz][0];
 	mat[coy][1]= - inp*mat[coz][1];
 	mat[coy][2]= 1.0 - inp*mat[coz][2];
 
-	Normalise((float *)mat[coy]);
+	if (Kx_Normalize((float *)mat[coy]) == 0.f) {
+		/* the camera is vertical, chose the y axis arbitrary */
+		mat[coy][0] = 0.f;
+		mat[coy][1] = 1.f;
+		mat[coy][2] = 0.f;
+	}
 	
-	Crossf(mat[cox], mat[coy], mat[coz]);
+	Kx_Crossf(mat[cox], mat[coy], mat[coz]);
 	
 }
 
 bool KX_CameraActuator::Update(double curtime, bool frame)
 {
-	bool result = true;
+	/* wondering... is it really neccesary/desirable to suppress negative    */
+	/* events here?                                                          */
+	bool bNegativeEvent = IsNegativeEvent();
+	RemoveAllEvents();
 
+	if (bNegativeEvent || !m_ob) 
+		return false;
+	
 	KX_GameObject *obj = (KX_GameObject*) GetParent();
 	MT_Point3 from = obj->NodeGetWorldPosition();
 	MT_Matrix3x3 frommat = obj->NodeGetWorldOrientation();
@@ -188,13 +230,6 @@ bool KX_CameraActuator::Update(double curtime, bool frame)
 	float mindistsq, maxdistsq, distsq;
 	float mat[3][3];
 	
-	/* wondering... is it really neccesary/desirable to suppress negative    */
-	/* events here?                                                          */
-	bool bNegativeEvent = IsNegativeEvent();
-	RemoveAllEvents();
-
-	if (bNegativeEvent) return false;
-
 	/* The rules:                                                            */
 	/* CONSTRAINT 1: not implemented */
 	/* CONSTRAINT 2: can camera see actor?              */
@@ -217,7 +252,7 @@ bool KX_CameraActuator::Update(double curtime, bool frame)
 	/* ... set up some parameters ...                                        */
 	/* missing here: the 'floorloc' of the actor's shadow */
 
- 	mindistsq= m_minHeight*m_minHeight;
+	mindistsq= m_minHeight*m_minHeight;
 	maxdistsq= m_maxHeight*m_maxHeight;
 
 	/* C1: not checked... is a future option                                 */
@@ -249,7 +284,7 @@ bool KX_CameraActuator::Update(double curtime, bool frame)
 	}
 	
 	inp= fp1[0]*fp2[0] + fp1[1]*fp2[1] + fp1[2]*fp2[2];
-	fac= (-1.0 + inp)/32.0;
+	fac= (-1.0 + inp) * m_damping;
 
 	from[0]+= fac*fp1[0];
 	from[1]+= fac*fp1[1];
@@ -294,7 +329,7 @@ bool KX_CameraActuator::Update(double curtime, bool frame)
 	rc[0]= (lookat[0]-from[0]);
 	rc[1]= (lookat[1]-from[1]);
 	rc[2]= (lookat[2]-from[2]);
-	VecUpMat3(rc, mat, 3);	/* y up Track -z */
+	Kx_VecUpMat3(rc, mat, 3);	/* y up Track -z */
 	
 
 
@@ -308,23 +343,16 @@ bool KX_CameraActuator::Update(double curtime, bool frame)
 	actormat[2][0]= mat[0][2]; actormat[2][1]= mat[1][2]; actormat[2][2]= mat[2][2];
 	obj->NodeSetLocalOrientation(actormat);
 
-	return result;
+	return true;
 }
 
-CValue *KX_CameraActuator::findObject(char *obName) 
+CValue *KX_CameraActuator::findObject(const char *obName)
 {
 	/* hook to object system */
 	return NULL;
 }
 
-bool KX_CameraActuator::string2axischoice(const char *axisString) 
-{
-	bool res = true;
-
-	res = !(axisString == Y_AXIS_STRING);
-
-	return res;
-}
+#ifdef WITH_PYTHON
 
 /* ------------------------------------------------------------------------- */
 /* Python functions                                                          */
@@ -332,39 +360,67 @@ bool KX_CameraActuator::string2axischoice(const char *axisString)
 
 /* Integration hooks ------------------------------------------------------- */
 PyTypeObject KX_CameraActuator::Type = {
-	PyObject_HEAD_INIT(&PyType_Type)
-	0,
+	PyVarObject_HEAD_INIT(NULL, 0)
 	"KX_CameraActuator",
-	sizeof(KX_CameraActuator),
+	sizeof(PyObjectPlus_Proxy),
 	0,
-	PyDestructor,
-	0,
-	__getattr,
-	__setattr,
-	0, //&MyPyCompare,
-	__repr,
-	0, //&cvalue_as_number,
+	py_base_dealloc,
 	0,
 	0,
 	0,
-	0
-};
-
-PyParentObject KX_CameraActuator::Parents[] = {
-	&KX_CameraActuator::Type,
+	0,
+	py_base_repr,
+	0,0,0,0,0,0,0,0,0,
+	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+	0,0,0,0,0,0,0,
+	Methods,
+	0,
+	0,
 	&SCA_IActuator::Type,
-	&SCA_ILogicBrick::Type,
-	&CValue::Type,
-	NULL
+	0,0,0,0,0,0,
+	py_base_new
 };
 
 PyMethodDef KX_CameraActuator::Methods[] = {
-	{NULL,NULL,NULL,NULL} //Sentinel
+	{NULL, NULL} //Sentinel
 };
 
-PyObject* KX_CameraActuator::_getattr(const STR_String& attr) {
-	_getattr_up(SCA_IActuator);
+PyAttributeDef KX_CameraActuator::Attributes[] = {
+	KX_PYATTRIBUTE_FLOAT_RW("min",-FLT_MAX,FLT_MAX,KX_CameraActuator,m_minHeight),
+	KX_PYATTRIBUTE_FLOAT_RW("max",-FLT_MAX,FLT_MAX,KX_CameraActuator,m_maxHeight),
+	KX_PYATTRIBUTE_FLOAT_RW("height",-FLT_MAX,FLT_MAX,KX_CameraActuator,m_height),
+	KX_PYATTRIBUTE_BOOL_RW("useXY",KX_CameraActuator,m_x),
+	KX_PYATTRIBUTE_RW_FUNCTION("object", KX_CameraActuator, pyattr_get_object,	pyattr_set_object),
+	KX_PYATTRIBUTE_FLOAT_RW("damping",0.f,10.f,KX_CameraActuator,m_damping),
+	{NULL}
+};
+
+PyObject* KX_CameraActuator::pyattr_get_object(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef)
+{
+	KX_CameraActuator* self= static_cast<KX_CameraActuator*>(self_v);
+	if (self->m_ob==NULL)
+		Py_RETURN_NONE;
+	else
+		return self->m_ob->GetProxy();
 }
 
+int KX_CameraActuator::pyattr_set_object(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value)
+{
+	KX_CameraActuator* self= static_cast<KX_CameraActuator*>(self_v);
+	KX_GameObject *gameobj;
+	
+	if (!ConvertPythonToGameObject(value, &gameobj, true, "actuator.object = value: KX_CameraActuator"))
+		return PY_SET_ATTR_FAIL; // ConvertPythonToGameObject sets the error
+	
+	if (self->m_ob)
+		self->m_ob->UnregisterActuator(self);	
+
+	if ((self->m_ob = (SCA_IObject*)gameobj))
+		self->m_ob->RegisterActuator(self);
+	
+	return PY_SET_ATTR_SUCCESS;
+}
+
+#endif // WITH_PYTHON
 
 /* eof */
