@@ -1,17 +1,13 @@
-/**
+/*
  * KX_CameraActuator.h
  *
- * $Id: KX_CameraActuator.h,v 1.8 2004/06/04 03:00:13 kester Exp $
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -29,7 +25,11 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
+ */
+
+/** \file KX_CameraActuator.h
+ *  \ingroup ketsji
  */
 
 #ifndef __KX_CAMERAACTUATOR
@@ -37,6 +37,7 @@
 
 #include "SCA_IActuator.h"
 #include "MT_Scalar.h"
+#include "SCA_LogicManager.h"
 
 /**
  * The camera actuator does a Robbie Muller prespective for you. This is a 
@@ -48,25 +49,35 @@
 
 class KX_CameraActuator : public SCA_IActuator
 {
-	Py_Header;
+	Py_Header
 private :
 	/** Object that will be tracked. */
-	const CValue *m_ob;
+	SCA_IObject *m_ob;
 
 	/** height (float), */
-	const MT_Scalar m_height;
+	//const MT_Scalar m_height;
+	/** min (float), */
+	//const MT_Scalar m_minHeight;
+	/** max (float), */
+	//const MT_Scalar m_maxHeight;
+	
+	/** height (float), */
+	float m_height;
 	
 	/** min (float), */
-	const MT_Scalar m_minHeight;
+	float m_minHeight;
 	
 	/** max (float), */
-	const MT_Scalar m_maxHeight;
+	float m_maxHeight;
 	
 	/** xy toggle (pick one): true == x, false == y */
 	bool m_x;
+	
+	/** damping (float), */
+	float m_damping;
 
 	/* get the KX_IGameObject with this name */
-	CValue *findObject(char *obName);
+	CValue *findObject(const char *obName);
 
 	/* parse x or y to a toggle pick */
 	bool string2axischoice(const char *axisString);
@@ -81,13 +92,13 @@ private :
 	KX_CameraActuator(
 
 		SCA_IObject *gameobj,
-		const CValue *ob,
-		MT_Scalar hght,
-		MT_Scalar minhght,
-		MT_Scalar maxhght,
+		//const CValue *ob,
+		SCA_IObject *ob,
+		float hght,
+		float minhght,
+		float maxhght,
 		bool xytog,
-		PyTypeObject* T=&Type
-
+		float damping
 	);
 
 
@@ -96,30 +107,31 @@ private :
 
 
 	/** Methods Inherited from  CValue */
-
-
 	CValue* GetReplica();
+	virtual void ProcessReplica();
 	
 
 	/** Methods inherited from SCA_IActuator */
-
-
-	bool Update(
-
+	virtual bool Update(
 		double curtime,
-
-		double deltatime
-
+		bool frame
 	);
+	virtual bool	UnlinkObject(SCA_IObject* clientobj);
 
+	/** Methods inherited from SCA_ILogicBrick */
+	virtual void	Relink(CTR_Map<CTR_HashedPtr, void*> *obj_map);
+
+#ifdef WITH_PYTHON
 
 	/* --------------------------------------------------------------------- */
 	/* Python interface ---------------------------------------------------- */
 	/* --------------------------------------------------------------------- */
 
-	virtual PyObject* _getattr(const STR_String& attr);
+	/* set object to look at */
+	static PyObject*	pyattr_get_object(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static int			pyattr_set_object(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value);
 
-	
+#endif // WITH_PYTHON
 
 };
 
