@@ -1,15 +1,10 @@
 /*
- * $Id: KX_KetsjiEngine.h,v 1.8 2004/10/16 11:41:48 kester Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,9 +22,14 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  *
  */
+
+/** \file KX_KetsjiEngine.h
+ *  \ingroup ketsji
+ */
+
 #ifndef __KX_KETSJI_ENGINE
 #define __KX_KETSJI_ENGINE
 
@@ -60,6 +60,11 @@ enum KX_ExitRequestMode
 	KX_EXIT_REQUEST_MAX
 };
 
+typedef struct {
+	short matmode;
+	short glslflag;
+}	GlobalSettings;
+
 /**
  * KX_KetsjiEngine is the core game engine class.
  */
@@ -73,10 +78,13 @@ private:
 	class RAS_IRenderTools*				m_rendertools;
 	class KX_ISceneConverter*			m_sceneconverter;
 	class NG_NetworkDeviceInterface*		m_networkdevice;
-	class SND_IAudioDevice*				m_audiodevice;
+#ifdef WITH_PYTHON
+	/* borrowed from sys.modules["__main__"], dont manage ref's */
 	PyObject*					m_pythondictionary;
+#endif
 	class SCA_IInputDevice*				m_keyboarddevice;
 	class SCA_IInputDevice*				m_mousedevice;
+	class KX_Dome*						m_dome; // dome stereo mode
 
 	/** Lists of scenes scheduled to be removed at the end of the frame. */
 	std::set<STR_String> m_removingScenes;
@@ -96,11 +104,25 @@ private:
 	int					m_activecam;
 	bool				m_bFixedTime;
 	
-	bool				m_firstframe;
 	
-	double				m_previoustime;
-	double				m_deltatime;
+	bool				m_firstframe;
+	int					m_currentFrame;
+
+	double				m_frameTime;//discrete timestamp of the 'game logic frame'
+	double				m_clockTime;//current time
+	double				m_previousClockTime;//previous clock time
+	double				m_previousAnimTime; //the last time animations were updated
+	double				m_remainingTime;
+
+	static int				m_maxLogicFrame;	/* maximum number of consecutive logic frame */
+	static int				m_maxPhysicsFrame;	/* maximum number of consecutive physics frame */
 	static double			m_ticrate;
+	static double			m_anim_framerate; /* for animation playback only - ipo and action */
+
+	static bool				m_restrict_anim_fps;
+
+	static double			m_suspendedtime;
+	static double			m_suspendeddelta;
 
 	int					m_exitcode;
 	STR_String			m_exitstring;
@@ -118,6 +140,9 @@ private:
 	bool			m_overrideCamUseOrtho;
 	MT_CmMatrix4x4	m_overrideCamProjMat;
 	MT_CmMatrix4x4	m_overrideCamViewMat;
+	float			m_overrideCamNear;
+	float			m_overrideCamFar;
+	float			m_overrideCamLens;
 
 	bool m_stereo;
 	int m_curreye;
@@ -128,9 +153,9 @@ private:
 		tc_first = 0,
 		tc_physics = 0,
 		tc_logic,
+		tc_animations,
 		tc_network,
 		tc_scenegraph,
-		tc_sound,
 		tc_rasterizer,
 		tc_services,	// time spend in miscelaneous activities
 		tc_overhead,	// profile info drawing overhead
@@ -143,6 +168,8 @@ private:
 	
 	/** Labels for profiling display. */
 	static const char		m_profileLabels[tc_numCategories][15];
+	/** Last estimated framerate */
+	static double			m_average_framerate;
 	/** Show the framerate on the game display? */
 	bool					m_show_framerate;
 	/** Show profiling info on the game display? */
@@ -154,6 +181,9 @@ private:
 
 	bool					m_show_debug_properties;
 
+	/** record physics into keyframes */
+	bool					m_animation_record;
+
 	/** Hide cursor every frame? */
 	bool					m_hideCursor;
 
@@ -166,33 +196,52 @@ private:
 	/** Blue component of framing bar color. */
 	float					m_overrideFrameColorB;
 
-	void					SetupRenderFrame(KX_Scene *scene);
-	void					RenderFrame(KX_Scene* scene);
+	/** Settings that doesn't go away with Game Actuator */
+	GlobalSettings m_globalsettings;
+
+	void					RenderFrame(KX_Scene* scene, KX_Camera* cam);
+	void					PostRenderScene(KX_Scene* scene);
 	void					RenderDebugProperties();
+	void					RenderShadowBuffers(KX_Scene *scene);
 	void					SetBackGround(KX_WorldInfo* worldinfo);
-	void					SetWorldSettings(KX_WorldInfo* worldinfo);
-	void					DoSound(KX_Scene* scene);
+	void					RenderFonts(KX_Scene* scene);
 
 public:
-
 	KX_KetsjiEngine(class KX_ISystem* system);
 	virtual ~KX_KetsjiEngine();
 
 	// set the devices and stuff. the client must take care of creating these
+	void			SetWorldSettings(KX_WorldInfo* worldinfo);
 	void			SetKeyboardDevice(SCA_IInputDevice* keyboarddevice);
 	void			SetMouseDevice(SCA_IInputDevice* mousedevice);
 	void			SetNetworkDevice(NG_NetworkDeviceInterface* networkdevice);
-	void			SetAudioDevice(SND_IAudioDevice* audiodevice);
 	void			SetCanvas(RAS_ICanvas* canvas);
 	void			SetRenderTools(RAS_IRenderTools* rendertools);
 	void			SetRasterizer(RAS_IRasterizer* rasterizer);
-	void			SetPythonDictionary(PyObject* pythondictionary);
+#ifdef WITH_PYTHON
+	void			SetPyNamespace(PyObject* pythondictionary);
+	PyObject*		GetPyNamespace(){return m_pythondictionary;};
+#endif
 	void			SetSceneConverter(KX_ISceneConverter* sceneconverter);
+	void			SetAnimRecordMode(bool animation_record, int startFrame);
 
-	void			NextFrame();
+	RAS_IRasterizer*		GetRasterizer(){return m_rasterizer;};
+	RAS_ICanvas*		    GetCanvas(){return m_canvas;};
+	RAS_IRenderTools*	    GetRenderTools(){return m_rendertools;};
+	SCA_IInputDevice*		GetKeyboardDevice(){return m_keyboarddevice;};
+	SCA_IInputDevice*		GetMouseDevice(){return m_mousedevice;};
+
+	/// Dome functions
+	void			InitDome(short res, short mode, short angle, float resbuf, short tilt, struct Text* text); 
+	void			EndDome();
+	void			RenderDome();
+	bool			m_usedome;
+
+	///returns true if an update happened to indicate -> Render
+	bool			NextFrame();
 	void			Render();
 	
-	void			StartEngine();
+	void			StartEngine(bool clearIpo);
 	void			StopEngine();
 	void			Export(const STR_String& filename);
 
@@ -211,7 +260,11 @@ public:
 	void			SuspendScene(const STR_String& scenename);
 	void			ResumeScene(const STR_String& scenename);
 
+	void			GetSceneViewport(KX_Scene* scene, KX_Camera* cam, RAS_Rect& area, RAS_Rect& viewport);
+
 	void SetDrawType(int drawingtype);
+	int  GetDrawType(){return m_drawingmode;};
+
 	void SetCameraZoom(float camzoom);
 	
 	void EnableCameraOverride(const STR_String& forscene);
@@ -219,6 +272,8 @@ public:
 	void SetCameraOverrideUseOrtho(bool useOrtho);
 	void SetCameraOverrideProjectionMatrix(const MT_CmMatrix4x4& mat);
 	void SetCameraOverrideViewMatrix(const MT_CmMatrix4x4& mat);
+	void SetCameraOverrideClipping(float near, float far);
+	void SetCameraOverrideLens(float lens);
 	
 	/**
 	 * Sets display of all frames.
@@ -231,7 +286,23 @@ public:
 	 * @return Current setting for display all frames.
 	 */ 
 	bool GetUseFixedTime(void) const;
-	
+
+	/**
+	 * Returns current render frame clock time
+	 */
+	double GetClockTime(void) const;
+	/**
+	 * Returns current logic frame clock time
+	 */
+	double GetFrameTime(void) const;
+
+	double GetRealTime(void) const;
+	/**
+	 * Returns the difference between the local time of the scene (when it
+	 * was running and not suspended) and the "curtime"
+	 */
+	static double GetSuspendedDelta();
+
 	/**
 	 * Gets the number of logic updates per second.
 	 */
@@ -240,6 +311,46 @@ public:
 	 * Sets the number of logic updates per second.
 	 */
 	static void SetTicRate(double ticrate);
+	/**
+	 * Gets the maximum number of logic frame before render frame
+	 */
+	static int GetMaxLogicFrame();
+	/**
+	 * Sets the maximum number of logic frame before render frame
+	 */
+	static void SetMaxLogicFrame(int frame);
+	/**
+	 * Gets the maximum number of physics frame before render frame
+	 */
+	static int GetMaxPhysicsFrame();
+	/**
+	 * Sets the maximum number of physics frame before render frame
+	 */
+	static void SetMaxPhysicsFrame(int frame);
+
+	/**
+	 * Gets whether or not to lock animation updates to the animframerate
+	 */
+	static bool GetRestrictAnimationFPS();
+
+	/**
+	 * Sets whether or not to lock animation updates to the animframerate
+	 */
+	static void SetRestrictAnimationFPS(bool bRestrictAnimFPS);
+
+	/**
+	 * Gets the framerate for playing animations. (actions and ipos)
+	 */
+	static double GetAnimFrameRate();
+	/**
+	 * Sets the framerate for playing animations. (actions and ipos)
+	 */
+	static void SetAnimFrameRate(double framerate);
+
+	/**
+	 * Gets the last estimated average framerate
+	 */
+	static double GetAverageFrameRate();
 
 	/**
 	 * Activates or deactivates timing information display.
@@ -276,8 +387,7 @@ public:
 	void SetUseOverrideFrameColor(bool overrideFrameColor);
 
 	/** 
-	 * Enables/disables the use of the framing bar color of the Blender file's scenes.
-	 * @param useSceneFrameColor The new setting.
+	 * Check if the frame color is being overridden.
 	 */
 	bool GetUseOverrideFrameColor(void) const; 
 
@@ -297,6 +407,12 @@ public:
 	 */
 	void GetOverrideFrameColor(float& r, float& g, float& b) const;
 
+	KX_Scene*		CreateScene(const STR_String& scenename);
+	KX_Scene*		CreateScene(Scene *scene);
+
+	GlobalSettings*	GetGlobalSettings(void);	
+	void			SetGlobalSettings(GlobalSettings* gs);
+
 protected:
 	/**
 	 * Processes all scheduled scene activity.
@@ -315,11 +431,19 @@ protected:
 	void			AddScheduledScenes(void);
 	void			ReplaceScheduledScenes(void);
 	void			PostProcessScene(class KX_Scene* scene);
-	KX_Scene*		CreateScene(const STR_String& scenename);
 	
 	bool			BeginFrame();
+	void			ClearFrame();
 	void			EndFrame();
+	
+	
+#ifdef WITH_CXX_GUARDEDALLOC
+public:
+	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:KX_KetsjiEngine"); }
+	void operator delete( void *mem ) { MEM_freeN(mem); }
+#endif
 };
 
 #endif //__KX_KETSJI_ENGINE
 
+
