@@ -1,15 +1,10 @@
-/**
- * $Id: KX_Scene.h,v 1.10 2004/10/16 11:41:48 kester Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,37 +22,44 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
+
+/** \file KX_Scene.h
+ *  \ingroup ketsji
+ */
+
 #ifndef __KX_SCENE_H
 #define __KX_SCENE_H
 
 
 #include "KX_PhysicsEngineEnums.h"
 
-#include "MT_CmMatrix4x4.h"
-
 #include <vector>
 #include <set>
+#include <list>
 
-#include "GEN_Map.h"
-#include "GEN_HashedPtr.h"
+#include "CTR_Map.h"
+#include "CTR_HashedPtr.h"
 #include "SG_IObject.h"
 #include "SCA_IScene.h"
 #include "MT_Transform.h"
-#include "SND_Scene.h"
+
 #include "RAS_FramingManager.h"
 #include "RAS_Rect.h"
 
+
 #include "PyObjectPlus.h"
+#include "RAS_2DFilterManager.h"
 
 /**
  * @section Forward declarations
  */
 struct SM_MaterialProps;
 struct SM_ShapeProps;
+struct Scene;
 
-class GEN_HashedPtr;
+class CTR_HashedPtr;
 class CListValue;
 class CValue;
 class SCA_LogicManager;
@@ -66,8 +68,6 @@ class SCA_TimeEventManager;
 class SCA_MouseManager;
 class SCA_ISystem;
 class SCA_IInputDevice;
-class SND_Scene;
-class SND_IAudioDevice;
 class NG_NetworkDeviceInterface;
 class NG_NetworkScene;
 class SG_IObject;
@@ -78,11 +78,22 @@ class KX_Camera;
 class KX_GameObject;
 class KX_LightObject;
 class RAS_BucketManager;
-class RAS_BucketManager;
 class RAS_MaterialBucket;
 class RAS_IPolyMaterial;
 class RAS_IRasterizer;
 class RAS_IRenderTools;
+class SCA_JoystickManager;
+class btCollisionShape;
+class KX_BlenderSceneConverter;
+struct KX_ClientObjectInfo;
+class KX_ObstacleSimulation;
+
+#ifdef WITH_CXX_GUARDEDALLOC
+#include "MEM_guardedalloc.h"
+#endif
+
+/* for ID freeing */
+#define IS_TAGGED(_id) ((_id) && (((ID *)_id)->flag & LIB_DOIT))
 
 /**
  * The KX_Scene holds all data for an independent scene. It relates
@@ -90,7 +101,19 @@ class RAS_IRenderTools;
  * */
 class KX_Scene : public PyObjectPlus, public SCA_IScene
 {
-	Py_Header;
+	Py_Header
+
+#ifdef WITH_PYTHON
+	PyObject*	m_attr_dict;
+	PyObject*	m_draw_call_pre;
+	PyObject*	m_draw_call_post;
+#endif
+
+	struct CullingInfo {
+		int m_layer;
+		CullingInfo(int layer) : m_layer(layer) {}
+	};
+
 protected:
 	RAS_BucketManager*	m_bucketmanager;
 	CListValue*			m_tempObjectList;
@@ -105,17 +128,26 @@ protected:
 	CListValue*			m_objectlist;
 	CListValue*			m_parentlist; // all 'root' parents
 	CListValue*			m_lightlist;
+	CListValue*			m_inactivelist;	// all objects that are not in the active layer
+	CListValue*			m_animatedlist; // all animated objects
+	
+	SG_QList			m_sghead;		// list of nodes that needs scenegraph update
+										// the Dlist is not object that must be updated
+										// the Qlist is for objects that needs to be rescheduled
+										// for updates after udpate is over (slow parent, bone parent)
 
-	/**
-	 *  The tree of objects in the scene.
-	 */
-	SG_Tree*			m_objecttree;
 
 	/**
 	 * The set of cameras for this scene
 	 */
-	set<class KX_Camera*>       m_cameras;
-	
+	list<class KX_Camera*>       m_cameras;
+
+	/**
+	 * The set of fonts for this scene
+	 */
+	list<class KX_FontObject*>       m_fonts;
+
+
 	/**
 	 * Various SCA managers used by the scene
 	 */
@@ -124,11 +156,13 @@ protected:
 	SCA_MouseManager*		m_mousemgr;
 	SCA_TimeEventManager*	m_timemgr;
 
+	// Scene converter where many scene entities are registered
+	// Used to deregister objects that are deleted
+	class KX_BlenderSceneConverter*		m_sceneConverter;
 	/**
 	* physics engine abstraction
 	*/
-
-	e_PhysicsEngine m_physicsEngine;
+	//e_PhysicsEngine m_physicsEngine; //who needs this ?
 	class PHY_IPhysicsEnvironment*		m_physicsEnvironment;
 
 	/**
@@ -150,14 +184,7 @@ protected:
 	 * @section Different scenes, linked to ketsji scene
 	 */
 
-
 	/**
-	 * Sound scenes
-	 */
-	SND_Scene* m_soundScene;
-	SND_IAudioDevice* m_adi;
-
-	/** 
 	 * Network scene.
 	 */
 	NG_NetworkDeviceInterface*	m_networkDeviceInterface;
@@ -175,26 +202,12 @@ protected:
 	 */
 	KX_Camera* m_active_camera;
 
-	/** 
-	 * The projection and view matrices of this scene 
-	 * The projection matrix is computed externally by KX_Engine	
-	 * The view mat is stored as a side effect of GetViewMatrix()
-	 * and is totally unnessary.
-	 */
-	MT_CmMatrix4x4				m_projectionmat;
-	MT_CmMatrix4x4				m_viewmat;
-
-	/** Desired canvas width set at design time. */
-	unsigned int m_canvasDesignWidth;
-	/** Desired canvas height set at design time. */
-	unsigned int m_canvasDesignHeight;
-
 	/**
 	 * Another temporary variable outstaying its welcome
 	 * used in AddReplicaObject to map game objects to their
 	 * replicas so pointers can be updated.
 	 */
-	GEN_Map	<GEN_HashedPtr, void*> m_map_gameobject_to_replica;
+	CTR_Map	<CTR_HashedPtr, void*> m_map_gameobject_to_replica;
 
 	/**
 	 * Another temporary variable outstaying its welcome
@@ -204,6 +217,16 @@ protected:
 	 */
 	std::vector<KX_GameObject*>	m_logicHierarchicalGameObjects;
 	
+	/**
+	 * This temporary variable will contain the list of 
+	 * object that can be added during group instantiation.
+	 * objects outside this list will not be added (can 
+	 * happen with children that are outside the group).
+	 * Used in AddReplicaObject. If the list is empty, it
+	 * means don't care.
+	 */
+	std::set<CValue*>	m_groupGameObjects;
+	
 	/** 
 	 * Pointer to system variable passed in in constructor
 	 * only used in constructor so we do not need to keep it
@@ -233,6 +256,16 @@ protected:
 	 */
 	bool m_activity_culling;
 	
+	/**
+	 * Toggle to enable or disable culling via DBVT broadphase of Bullet.
+	 */
+	bool m_dbvt_culling;
+	
+	/**
+	 * Occlusion culling resolution
+	 */ 
+	int m_dbvt_occlusion_res;
+
 	/**
 	 * The framing settings used by this scene
 	 */
@@ -248,34 +281,49 @@ protected:
 	/**
 	 * Visibility testing functions.
 	 */
-	void MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty);
-	void MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible);
-	void MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj);
-	
-	/**
-	 * This stores anything from python
-	 */
-	PyObject* m_attrlist;
+	void MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty, KX_Camera*cam,int layer=0);
+	void MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible, KX_Camera*cam,int layer=0);
+	void MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj, KX_Camera*cam, int layer=0);
+	static void PhysicsCullingCallback(KX_ClientObjectInfo* objectInfo, void* cullingInfo);
 
-public:
+	double				m_suspendedtime;
+	double				m_suspendeddelta;
+
+	struct Scene* m_blenderScene;
+
+	RAS_2DFilterManager m_filtermanager;
+
+	KX_ObstacleSimulation* m_obstacleSimulation;
+
+public:	
 	KX_Scene(class SCA_IInputDevice* keyboarddevice,
 		class SCA_IInputDevice* mousedevice,
 		class NG_NetworkDeviceInterface* ndi,
-		class SND_IAudioDevice* adi,
-		const STR_String& scenename	);
+		const STR_String& scenename,
+		struct Scene* scene,
+		class RAS_ICanvas* canvas);
 
 	virtual	
 	~KX_Scene();
 
 	RAS_BucketManager* GetBucketManager();
-	RAS_MaterialBucket*	FindBucket(RAS_IPolyMaterial* polymat);
+	RAS_MaterialBucket*	FindBucket(RAS_IPolyMaterial* polymat, bool &bucketCreated);
 	void RenderBuckets(const MT_Transform& cameratransform,
 						RAS_IRasterizer* rasty,
 						RAS_IRenderTools* rendertools);
+
 	/**
 	 * Update all transforms according to the scenegraph.
 	 */
+	static bool KX_ScenegraphUpdateFunc(SG_IObject* node,void* gameobj,void* scene);
+	static bool KX_ScenegraphRescheduleFunc(SG_IObject* node,void* gameobj,void* scene);
 	void UpdateParents(double curtime);
+	void DupliGroupRecurse(CValue* gameobj, int level);
+	bool IsObjectInGroup(CValue* gameobj)
+	{ 
+		return (m_groupGameObjects.empty() || 
+				m_groupGameObjects.find(gameobj) != m_groupGameObjects.end());
+	}
 	SCA_IObject* AddReplicaObject(CValue* gameobj,
 								  CValue* locationobj,
 								  int lifespan=0);
@@ -285,22 +333,36 @@ public:
 								  CValue* gameobj);
 	void RemoveObject(CValue* gameobj);
 	void DelayedRemoveObject(CValue* gameobj);
-	void NewRemoveObject(CValue* gameobj);
+	
+	int NewRemoveObject(CValue* gameobj);
 	void ReplaceMesh(CValue* gameobj,
-					 void* meshobj);
+					 void* meshob, bool use_gfx, bool use_phys);
+
+	void AddAnimatedObject(CValue* gameobj);
+	void RemoveAnimatedObject(CValue* gameobj);
+
 	/**
 	 * @section Logic stuff
 	 * Initiate an update of the logic system.
 	 */
 	void LogicBeginFrame(double curtime);
 	void LogicUpdateFrame(double curtime, bool frame);
+	void UpdateAnimations(double curtime);
 
 		void						
 	LogicEndFrame(
 	);
 
 		CListValue*				
+	GetTempObjectList(
+	);
+
+		CListValue*
 	GetObjectList(
+	);
+
+		CListValue*				
+	GetInactiveList(
 	);
 
 		CListValue*				
@@ -319,6 +381,31 @@ public:
 	GetTimeEventManager(
 	);
 
+	/** Font Routines */
+		
+		list<class KX_FontObject*>*
+	GetFonts(
+	);
+
+	/** Find a font in the scene by pointer. */
+		KX_FontObject*              
+	FindFont(
+		KX_FontObject*
+	);
+
+	/** Add a camera to this scene. */
+		void                    
+	AddFont(
+		KX_FontObject*
+	);
+
+
+	/** Camera Routines */
+
+		list<class KX_Camera*>*
+	GetCameras(
+	);
+ 
 
 	/** Find a camera in the scene by pointer. */
 		KX_Camera*              
@@ -353,23 +440,13 @@ public:
 		class KX_Camera*
 	);
 
-	/** Return the viewmatrix as used by the last frame. */
-		MT_CmMatrix4x4&			
-	GetViewMatrix(
-	);
-
-	/** 
-	 * Return the projectionmatrix as used by the last frame. This is
-	 * set by hand :) 
+	/**
+	 * Move this camera to the end of the list so that it is rendered last.
+	 * If the camera is not on the list, it will be added
 	 */
-		MT_CmMatrix4x4&			
-	GetProjectionMatrix(
-	);
-
-	/** Sets the projection matrix. */
-		void					
-	SetProjectionMatrix(
-		MT_CmMatrix4x4& pmat
+		void
+	SetCameraOnTop(
+		class KX_Camera*
 	);
 
 	/**
@@ -447,12 +524,12 @@ public:
 	void SetNetworkScene(NG_NetworkScene *newScene);
 	void SetWorldInfo(class KX_WorldInfo* wi);
 	KX_WorldInfo* GetWorldInfo();
-	void CalculateVisibleMeshes(RAS_IRasterizer* rasty);
+	void CalculateVisibleMeshes(RAS_IRasterizer* rasty, KX_Camera *cam, int layer=0);
 	void UpdateMeshTransformations();
 	KX_Camera* GetpCamera();
-	SND_Scene* GetSoundScene();
 	NG_NetworkDeviceInterface* GetNetworkDeviceInterface();
 	NG_NetworkScene* GetNetworkScene();
+	KX_BlenderSceneConverter *GetSceneConverter() { return m_sceneConverter; }
 
 	/**
 	 * Replicate the logic bricks associated to this object.
@@ -480,7 +557,14 @@ public:
 	bool IsSuspended();
 	bool IsClearingZBuffer();
 	void EnableZBufferClearing(bool isclearingZbuffer);
+	// use of DBVT tree for camera culling
+	void SetDbvtCulling(bool b) { m_dbvt_culling = b; };
+	bool GetDbvtCulling() { return m_dbvt_culling; };
+	void SetDbvtOcclusionRes(int i) { m_dbvt_occlusion_res = i; };
+	int GetDbvtOcclusionRes() { return m_dbvt_occlusion_res; };
 	
+	void SetSceneConverter(class KX_BlenderSceneConverter* sceneConverter);
+
 	class PHY_IPhysicsEnvironment*		GetPhysicsEnvironment()
 	{
 		return m_physicsEnvironment;
@@ -489,31 +573,94 @@ public:
 	void SetPhysicsEnvironment(class PHY_IPhysicsEnvironment*	physEnv);
 
 	void	SetGravity(const MT_Vector3& gravity);
+
+	short GetAnimationFPS();
 	
 	/**
 	 * Sets the node tree for this scene.
 	 */
 	void SetNodeTree(SG_Tree* root);
 
-	KX_PYMETHOD_DOC(KX_Scene, getLightList);
-	KX_PYMETHOD_DOC(KX_Scene, getObjectList);
-	KX_PYMETHOD_DOC(KX_Scene, getName);
-/*	
-	KX_PYMETHOD_DOC(KX_Scene, getActiveCamera);
-	KX_PYMETHOD_DOC(KX_Scene, getActiveCamera);
-	KX_PYMETHOD_DOC(KX_Scene, findCamera);
-	
-	KX_PYMETHOD_DOC(KX_Scene, getGravity);
-	
-	KX_PYMETHOD_DOC(KX_Scene, setActivityCulling);
-	KX_PYMETHOD_DOC(KX_Scene, setActivityCullingRadius);
-	
-	KX_PYMETHOD_DOC(KX_Scene, setSceneViewport);
-	KX_PYMETHOD_DOC(KX_Scene, setSceneViewport);
+	/**
+	* 2D Filters
 	*/
+	void Update2DFilter(vector<STR_String>& propNames, void* gameObj, RAS_2DFilterManager::RAS_2DFILTER_MODE filtermode, int pass, STR_String& text);
+	void Render2DFilters(RAS_ICanvas* canvas);
 
-	virtual PyObject* _getattr(const STR_String& attr); /* name, active_camera, gravity, suspended, viewport, framing, activity_culling, activity_culling_radius */
-	virtual int _setattr(const STR_String &attr, PyObject *pyvalue);
+	KX_ObstacleSimulation* GetObstacleSimulation() {return m_obstacleSimulation;};
+
+#ifdef WITH_PYTHON
+	/* --------------------------------------------------------------------- */
+	/* Python interface ---------------------------------------------------- */
+	/* --------------------------------------------------------------------- */
+
+	KX_PYMETHOD_DOC(KX_Scene, addObject);
+	KX_PYMETHOD_DOC(KX_Scene, end);
+	KX_PYMETHOD_DOC(KX_Scene, restart);
+	KX_PYMETHOD_DOC(KX_Scene, replace);
+	KX_PYMETHOD_DOC(KX_Scene, suspend);
+	KX_PYMETHOD_DOC(KX_Scene, resume);
+	KX_PYMETHOD_DOC(KX_Scene, get);
+	KX_PYMETHOD_DOC(KX_Scene, drawObstacleSimulation);
+
+
+	/* attributes */
+	static PyObject*	pyattr_get_name(void* self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static PyObject*	pyattr_get_objects(void* self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static PyObject*	pyattr_get_objects_inactive(void* self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static PyObject*	pyattr_get_lights(void* self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static PyObject*	pyattr_get_cameras(void* self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static PyObject*	pyattr_get_active_camera(void* self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static int			pyattr_set_active_camera(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value);
+	static PyObject*	pyattr_get_drawing_callback_pre(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static int			pyattr_set_drawing_callback_pre(void *selv_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value);
+	static PyObject*	pyattr_get_drawing_callback_post(void *self_v, const KX_PYATTRIBUTE_DEF *attrdef);
+	static int			pyattr_set_drawing_callback_post(void *selv_v, const KX_PYATTRIBUTE_DEF *attrdef, PyObject *value);
+
+	virtual PyObject* py_repr(void) { return PyUnicode_FromString(GetName().ReadPtr()); }
+	
+	/* getitem/setitem */
+	static PyMappingMethods	Mapping;
+	static PySequenceMethods	Sequence;
+
+	/**
+	 * Run the registered python drawing functions.
+	 */
+	void RunDrawingCallbacks(PyObject* cb_list);
+	
+	PyObject* GetPreDrawCB() { return m_draw_call_pre; };
+	PyObject* GetPostDrawCB() { return m_draw_call_post; };
+#endif
+
+	/**
+	 * Sets the time the scene was suspended
+	 */ 
+	void setSuspendedTime(double suspendedtime);
+	/**
+	 * Returns the "curtime" the scene was suspended
+	 */ 
+	double getSuspendedTime();
+	/**
+	 * Sets the difference between the local time of the scene (when it
+	 * was running and not suspended) and the "curtime"
+	 */ 
+	void setSuspendedDelta(double suspendeddelta);
+	/**
+	 * Returns the difference between the local time of the scene (when it
+	 * was running and not suspended) and the "curtime"
+	 */
+	double getSuspendedDelta();
+	/**
+	 * Returns the Blender scene this was made from
+	 */
+	struct Scene *GetBlenderScene() { return m_blenderScene; }
+
+	bool MergeScene(KX_Scene *other);
+
+
+	//void PrintStats(int verbose_level) {
+	//	m_bucketmanager->PrintStats(verbose_level)
+	//}
 };
 
 typedef std::vector<KX_Scene*> KX_SceneList;
