@@ -1,15 +1,10 @@
-/**
- * $Id: RAS_IRasterizer.h,v 1.12 2004/10/24 11:03:18 kester Exp $
- *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -27,37 +22,51 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
+
+/** \file RAS_IRasterizer.h
+ *  \ingroup bgerast
+ */
+
 #ifndef __RAS_IRASTERIZER
 #define __RAS_IRASTERIZER
 
-#ifdef WIN32
+#if defined(WIN32) && !defined(FREE_WINDOWS)
 #pragma warning (disable:4786)
 #endif
 
+#include "STR_HashedString.h"
+
 #include "MT_CmMatrix4x4.h"
 #include "MT_Matrix4x4.h"
 
+#include "RAS_TexVert.h"
+
+#include <vector>
+using namespace std;
+
+#ifdef WITH_CXX_GUARDEDALLOC
+#include "MEM_guardedalloc.h"
+#endif
+
 class RAS_ICanvas;
 class RAS_IPolyMaterial;
-#include "RAS_MaterialBucket.h"
+
+typedef vector<unsigned short> KX_IndexArray;
+typedef vector<RAS_TexVert> KX_VertexArray;
+typedef vector< KX_VertexArray* >  vecVertexArray;
+typedef vector< KX_IndexArray* > vecIndexArrays;
 
 /**
  * 3D rendering device context interface. 
  */
 class RAS_IRasterizer
 {
-
 public:
-
 	RAS_IRasterizer(RAS_ICanvas* canv){};
 	virtual ~RAS_IRasterizer(){};
-	/**
-	 */
-	enum	{
-			RAS_RENDER_3DPOLYGON_TEXT = 16384
-	};
+
 	/**
 	 * Drawing types
 	 */
@@ -66,7 +75,18 @@ public:
 			KX_WIREFRAME,
 			KX_SOLID,
 			KX_SHADED,
-			KX_TEXTURED 
+			KX_TEXTURED,
+			KX_SHADOW
+	};
+
+	/**
+	 * Drawing modes
+	 */
+
+	enum DrawMode {
+		KX_MODE_LINES = 1,
+		KX_MODE_TRIANGLES,
+		KX_MODE_QUADS
 	};
 
 	/**
@@ -80,7 +100,9 @@ public:
 	/**
 	 */
 	enum    { 	 
-			KX_TWOSIDE = 512, 	 
+			RAS_RENDER_3DPOLYGON_TEXT = 64,	/* GEMAT_TEXT */
+			KX_BACKCULL = 16,		/* GEMAT_BACKCULL */
+			KX_TEX = 4096,			/* GEMAT_TEX */
 			KX_LINES = 32768 	 
 	};
 
@@ -95,9 +117,31 @@ public:
 			RAS_STEREO_ANAGLYPH,
 			RAS_STEREO_SIDEBYSIDE,
 			RAS_STEREO_VINTERLACE,
+			RAS_STEREO_DOME,
 			
 			RAS_STEREO_MAXSTEREO
 	};
+
+	/**
+	 * Texture gen modes.
+	 */
+	enum TexCoGen {
+		RAS_TEXCO_GEN,		//< GPU will generate texture coordinates
+		RAS_TEXCO_ORCO,		//< Vertex coordinates (object space)
+		RAS_TEXCO_GLOB,		//< Vertex coordinates (world space)
+		RAS_TEXCO_UV1,		//< UV coordinates
+		RAS_TEXCO_OBJECT,	//< Use another object's position as coordinates
+		RAS_TEXCO_LAVECTOR,	//< Light vector as coordinates
+		RAS_TEXCO_VIEW,		//< View vector as coordinates
+		RAS_TEXCO_STICKY,	//< Sticky coordinates
+		RAS_TEXCO_WINDOW,	//< Window coordinates
+		RAS_TEXCO_NORM,		//< Normal coordinates 
+		RAS_TEXTANGENT,		//<
+		RAS_TEXCO_UV2,		//<
+		RAS_TEXCO_VCOL,		//< Vertex Color
+		RAS_TEXCO_DISABLE	//< Disable this texture unit (cached)
+	};
+
 	/**
 	 * Render pass identifiers for stereo.
 	 */
@@ -116,7 +160,7 @@ public:
 	 * to be rendered with.
 	 * The material will be cached.
 	 */
-	virtual void	SetMaterial(const RAS_IPolyMaterial& mat)=0;
+	virtual bool	SetMaterial(const RAS_IPolyMaterial& mat)=0;
 	/**
 	 * Init initialises the renderer.
 	 */
@@ -129,6 +173,10 @@ public:
 	 * BeginFrame is called at the start of each frame.
 	 */
 	virtual bool	BeginFrame(int drawingmode, double time)=0;
+	/**
+	 * ClearColorBuffer clears the color buffer.
+	 */
+	virtual void	ClearColorBuffer()=0;
 	/**
 	 * ClearDepthBuffer clears the depth buffer.
 	 */
@@ -142,7 +190,8 @@ public:
 	 */
 	virtual void	EndFrame()=0;
 	/**
-	 * SetRenderArea sets the render area from the 2d canvas
+	 * SetRenderArea sets the render area from the 2d canvas.
+	 * Returns true if only of subset of the canvas is used.
 	 */
 	virtual void	SetRenderArea()=0;
 
@@ -156,6 +205,8 @@ public:
 	 * @return true if stereo mode is enabled.
 	 */
 	virtual bool	Stereo()=0;
+	virtual StereoMode GetStereoMode()=0;
+	virtual bool	InterlacedStereo()=0;
 	/**
 	 * Sets which eye buffer subsequent primitives will be rendered to.
 	 */
@@ -178,46 +229,18 @@ public:
 	
 	// Drawing Functions
 	/**
-	 * IndexPrimitives: Renders primitives.
-	 * @param vertexarrays is an array of vertex arrays
-	 * @param indexarrays is an array of index arrays
-	 * @param mode determines the type of primitive stored in the vertex/index arrays:
-	 *              0 triangles
-	 *              1 lines (default)
-	 *              2 quads
-	 * @param polymat (reserved)
-	 * @param useObjectColor will render the object using @param rgbacolor instead of 
-	 *  vertex colours.
+	 * IndexPrimitives: Renders primitives from mesh slot.
 	 */
-	virtual void	IndexPrimitives( const vecVertexArray& vertexarrays,
-							const vecIndexArrays & indexarrays,
-							int mode,
-							class RAS_IPolyMaterial* polymat,
-							class RAS_IRenderTools* rendertools,
-							bool useObjectColor,
-							const MT_Vector4& rgbacolor)=0;
-	/**
-	 * @copydoc IndexPrimitives
-	 * IndexPrimitivesEx will renormalize faces if @param vertexarrays[i].getFlag() & TV_CALCFACENORMAL
-	 */
-	virtual void	IndexPrimitives_Ex( const vecVertexArray& vertexarrays,
-							const vecIndexArrays & indexarrays,
-							int mode,
-							class RAS_IPolyMaterial* polymat,
-							class RAS_IRenderTools* rendertools,
-							bool useObjectColor,
-							const MT_Vector4& rgbacolor)=0;
+	virtual void IndexPrimitives(class RAS_MeshSlot& ms)=0;
+	virtual void IndexPrimitivesMulti(class RAS_MeshSlot& ms)=0;
+
 	/**
 	 * IndexPrimitives_3DText will render text into the polygons.
 	 * The text to be rendered is from @param rendertools client object's text property.
 	 */
-	virtual void	IndexPrimitives_3DText( const vecVertexArray& vertexarrays,
-							const vecIndexArrays & indexarrays,
-							int mode,
+	virtual void	IndexPrimitives_3DText(class RAS_MeshSlot& ms,
 							class RAS_IPolyMaterial* polymat,
-							class RAS_IRenderTools* rendertools,
-							bool useObjectColor,
-							const MT_Vector4& rgbacolor)=0;
+							class RAS_IRenderTools* rendertools)=0;
 
 	virtual void	SetProjectionMatrix(MT_CmMatrix4x4 & mat)=0;
 	/* This one should become our final version, methinks. */
@@ -231,15 +254,14 @@ public:
 	 * Sets the modelview matrix.
 	 */
 	virtual void	SetViewMatrix(const MT_Matrix4x4 & mat,
-						const MT_Vector3& campos,
-						const MT_Point3 &camLoc,
-						const MT_Quaternion &camOrientQuat)=0;
+								const MT_Matrix3x3 & ori,
+								const MT_Point3 & pos,
+								bool perspective)=0;
 	/**
 	 */
 	virtual const	MT_Point3& GetCameraPosition()=0;
-	/**
-	 */
-	virtual void	LoadViewMatrix()=0;
+	virtual bool	GetCameraOrtho()=0;
+
 	/**
 	 */
 	virtual void	SetFog(float start,
@@ -262,6 +284,7 @@ public:
 	/**
 	 */
 	virtual void	DisableFog()=0;
+	virtual bool	IsFogEnabled()=0;
 
 	virtual void	SetBackColor(float red,
 								 float green,
@@ -276,9 +299,6 @@ public:
 	 * @return the current drawing mode: KX_BOUNDINGBOX, KX_WIREFRAME, KX_SOLID, KX_SHADED or KX_TEXTURED.
 	 */
 	virtual int	GetDrawingMode()=0;
-	/**
-	 */
-	virtual void	EnableTextures(bool enable)=0;
 	/**
 	 * Sets face culling
 	 */	
@@ -307,10 +327,31 @@ public:
 		float top,
 		float frustnear,
 		float frustfar,
+		float focallength = 0.0f,
 		bool perspective = true
 	)=0;
+
 	/**
-	 * Sets the specular colour component of the lighting equation.
+	 * Generates a orthographic projection matrix from the specified frustum.
+	 * @param left the left clipping plane
+	 * @param right the right clipping plane
+	 * @param bottom the bottom clipping plane
+	 * @param top the top clipping plane
+	 * @param frustnear the near clipping plane
+	 * @param frustfar the far clipping plane
+	 * @return a 4x4 matrix representing the projection transform.
+	 */
+	virtual MT_Matrix4x4 GetOrthoMatrix(
+		float left,
+		float right,
+		float bottom,
+		float top,
+		float frustnear,
+		float frustfar
+	)=0;
+
+	/**
+	 * Sets the specular color component of the lighting equation.
 	 */
 	virtual void	SetSpecularity(float specX,
 								   float specY,
@@ -322,14 +363,68 @@ public:
 	 */
 	virtual void	SetShinyness(float shiny)=0;
 	/**
-	 * Sets the diffuse colour component of the lighting equation.
+	 * Sets the diffuse color component of the lighting equation.
 	 */
 	virtual void	SetDiffuse(float difX,
 							   float difY,
 							   float difZ,
 							   float diffuse)=0;
+	/**
+	 * Sets the emissive color component of the lighting equation.
+	 */ 
+	virtual void	SetEmissive(float eX,
+								float eY,
+								float eZ,
+								float e
+							   )=0;
 	
+	virtual void	SetAmbientColor(float red, float green, float blue)=0;
+	virtual void	SetAmbient(float factor)=0;
+
+	/**
+	 * Sets a polygon offset.  z depth will be: z1 = mult*z0 + add
+	 */
+	virtual void	SetPolygonOffset(float mult, float add) = 0;
+	
+	virtual	void	DrawDebugLine(const MT_Vector3& from,const MT_Vector3& to,const MT_Vector3& color)=0;
+	virtual	void	DrawDebugCircle(const MT_Vector3& center, const MT_Scalar radius, const MT_Vector3& color,
+									const MT_Vector3& normal, int nsector)=0;
+	virtual	void	FlushDebugShapes()=0;
+	
+
+
+	virtual void	SetTexCoordNum(int num) = 0;
+	virtual void	SetAttribNum(int num) = 0;
+	virtual void	SetTexCoord(TexCoGen coords, int unit) = 0;
+	virtual void	SetAttrib(TexCoGen coords, int unit) = 0;
+
+	virtual const MT_Matrix4x4&	GetViewMatrix() const = 0;
+	virtual const MT_Matrix4x4&	GetViewInvMatrix() const = 0;
+
+	virtual bool	QueryLists(){return false;}
+	virtual bool	QueryArrays(){return false;}
+	
+	virtual void	EnableMotionBlur(float motionblurvalue)=0;
+	virtual void	DisableMotionBlur()=0;
+	
+	virtual float	GetMotionBlurValue()=0;
+	virtual int		GetMotionBlurState()=0;
+	virtual void	SetMotionBlurState(int newstate)=0;
+
+	virtual void	SetAlphaBlend(int alphablend)=0;
+	virtual void	SetFrontFace(bool ccw)=0;
+
+	virtual void	SetAnisotropicFiltering(short level)=0;
+	virtual short	GetAnisotropicFiltering()=0;
+	
+	
+#ifdef WITH_CXX_GUARDEDALLOC
+public:
+	void *operator new(size_t num_bytes) { return MEM_mallocN(num_bytes, "GE:RAS_IRasterizer"); }
+	void operator delete( void *mem ) { MEM_freeN(mem); }
+#endif
 };
 
 #endif //__RAS_IRASTERIZER
 
+
